@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2009-2018, VU University Amsterdam
-                              CWI, Amsterdam
+    Copyright (c)  2009-2022, VU University Amsterdam
+                              CWI, Amsterdam,
+                              SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -90,7 +91,8 @@ have the same name and arity.
                        pass_to(phrase_from_file/3, 3)
                      ]).
 :- predicate_options(csv_read_file_row/3, 3,
-                     [ pass_to(csv//2, 2),
+                     [ line(-integer),
+                       pass_to(csv//2, 2),
                        pass_to(open/4, 4)
                      ]).
 :- predicate_options(csv_write_file/3, 3,
@@ -175,7 +177,7 @@ csv_read_stream(Stream, Rows, Options) :-
 %       * separator(+Code)
 %       The comma-separator.  Must be a character code.  Default is
 %       (of course) the comma. Character codes can be specified
-%       using the 0' notion. E.g., using =|separator(0';)|= parses
+%       using the 0' notation. E.g., using =|separator(0';)|= parses
 %       a semicolon separated file.
 %
 %       * ignore_quotes(+Boolean)
@@ -595,7 +597,7 @@ emit_codes([H|T]) --> [H], emit_codes(T).
 %               open(File, write, Out),
 %               forall(data(C1,C2,C3),
 %                      csv_write_stream(Out, [row(C1,C2,C3)], [])),
-%               close(Out)),
+%               close(Out)).
 %        ==
 
 csv_write_stream(Stream, Data, Options) :-
