@@ -3,9 +3,10 @@
     Author:        Jan Wielemaker and Anjo Anjewierden
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org/
-    Copyright (c)  2011-2018, University of Amsterdam
+    Copyright (c)  2011-2022, University of Amsterdam
                               VU University Amsterdam
                               CWI, Amsterdam
+                              SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -37,6 +38,7 @@
 :- module(prolog_breakpoints,
           [ set_breakpoint/4,           % +File, +Line, +CharPos, -Id
             set_breakpoint/5,           % +Owner, +File, +Line, +CharPos, -Id
+            set_breakpoint_condition/2, % +Id, +Cond
             delete_breakpoint/1,        % +Id
             breakpoint_property/2       % ?Id, ?Property
           ]).
@@ -69,10 +71,10 @@ other hooks the opportunity to react.
 %
 %   First, '$clause_from_source'/4 uses  the   SWI-Prolog  clause-source
 %   information  to  find  the  last    clause   starting  before  Line.
-%   '$break_pc'  generated  (on  backtracking),  a    list  of  possible
-%   break-points.
+%   '$break_pc'  generates  (on  backtracking),  a    list  of  possible
+%   breakpoints.
 %
-%   Note that in addition to setting the break-point, the system must be
+%   Note that in addition to setting the breakpoint, the system must be
 %   in debug mode for the  breakpoint   to  take  effect. With threading
 %   enabled, there are various different  ways   this  may  be done. See
 %   debug/0, tdebug/0 and tdebug/1. Therefore, this predicate does *not*
@@ -94,7 +96,9 @@ set_breakpoint(Owner, File, Line, Char, Id) :-
         debug(break, 'Clause ~p, PC=~p NextPC=~p', [ClauseRef, PC, NextPC]),
         '$clause_term_position'(ClauseRef, NextPC, List),
         debug(break, 'Location = ~w', [List]),
-        range(List, TermPos, A, Z),
+        range(List, TermPos, SubPos),
+        arg(1, SubPos, A),
+        arg(2, SubPos, Z),
         debug(break, 'Term from ~w-~w', [A, Z]),
         Z >= Char, !,
         Len is Z - A,
@@ -108,17 +112,24 @@ set_breakpoint(Owner, File, Line, Char, Id) :-
     nb_delete('$breakpoint'),
     known_breakpoint(ClauseRef, PC, _Location, Id).
 
-range(_,  Pos, _, _) :-
-    var(Pos), !, fail.
-range([], Pos, A, Z) :-
-    arg(1, Pos, A),
-    arg(2, Pos, Z).
-range([H|T], term_position(_, _, _, _, PosL), A, Z) :-
+range(_,  Pos, _), var(Pos) =>
+    fail.
+range(List, parentheses_term_position(_,_,Pos), SubPos) =>
+    range(List, Pos, SubPos).
+range([], Pos, SubPos) =>
+    SubPos = Pos.
+range([H|T], term_position(_, _, _, _, PosL), SubPos) =>
     nth1(H, PosL, Pos),
-    range(T, Pos, A, Z).
+    range(T, Pos, SubPos).
+range(exit, Pos, SubPos) =>
+    arg(2, Pos, End),
+    Dot is End,
+    EndDot is Dot+1,
+    SubPos = Dot-EndDot.
 
 :- dynamic
     known_breakpoint/4,             % ClauseRef, PC, Location, Id
+    breakpoint_condition/4,         % Id, CondString, CondTerm, VarOffsets
     break_id/1.
 
 next_break_id(Id) :-
@@ -171,11 +182,18 @@ breakpoint_property(Id, character_range(Start, Len)) :-
     known_breakpoint(ClauseRef,PC,Location,Id),
     (   Location = file_location(_File, _Line, Start, Len)
     ->  true
-    ;   break_location(ClauseRef, PC, _File, Start-End),
+    ;   break_location(ClauseRef, PC, _File, SubPos),
+        compound(SubPos),
+        arg(1, SubPos, Start),
+        arg(2, Start, End),
+        nonvar(Start), nonvar(End),
         Len is End+1-Start
     ).
 breakpoint_property(Id, clause(Reference)) :-
     known_breakpoint(Reference,_,_,Id).
+breakpoint_property(Id, condition(Cond)) :-
+    known_breakpoint(_,_,_,Id),
+    breakpoint_condition(Id, Cond, _CondTerm, _VarOffsets).
 
 location_line(file_location(_File, Line, _Start, _Len), Line).
 location_line(file_character_range(File, Start, _Len), Line) :-
@@ -238,31 +256,36 @@ onbreak(true, ClauseRef, PC) :-
     break_message(breakpoint(set, Id)).
 onbreak(false, ClauseRef, PC) :-
     debug(break, 'Remove breakpoint from ~p, PC ~d', [ClauseRef, PC]),
-    clause(known_breakpoint(ClauseRef, PC, _Location, Id), true, Ref),
-    call_cleanup(break_message(breakpoint(delete, Id)), erase(Ref)).
-onbreak(gc, ClauseRef, PC) :-
-    debug(break, 'Remove breakpoint from ~p, PC ~d (due to CGC)',
+    delete_breakpoint(ClauseRef, PC).
+onbreak(retract, ClauseRef, PC) :-
+    debug(break, 'Remove breakpoint from ~p, PC ~d (due to retract)',
           [ClauseRef, PC]),
-    retractall(known_breakpoint(ClauseRef, PC, _Location, _Id)).
+    delete_breakpoint(ClauseRef, PC).
+
+delete_breakpoint(ClauseRef, PC) :-
+    clause(known_breakpoint(ClauseRef, PC, _Location, Id), true, Ref),
+    retractall(breakpoint_condition(Id, _, _, _)),
+    call_cleanup(break_message(breakpoint(delete, Id)), erase(Ref)).
 
 break_message(Message) :-
     print_message(informational, Message).
 
-%!  break_location(+ClauseRef, +PC, -File, -AZ) is det.
+%!  break_location(+ClauseRef, +PC, -File, -Pos) is det.
 %
-%   True when File and AZ represent the  location of the goal called
-%   at PC in ClauseRef.
+%   True when File and Pos represent the  file and subterm position term
+%   for the goal called at PC in ClauseRef.
 %
-%   @param AZ is a term A-Z, where   A and Z are character positions
-%   in File.
+%   @arg Pos is a subterm position  term (see `subterm_positions` option
+%   of read_term/3, where positions are character positions in File. The
+%   first two argument always hold the start and end of the term.
 
-break_location(ClauseRef, PC, File, A-Z) :-
+break_location(ClauseRef, PC, File, SubPos) :-
     clause_info(ClauseRef, File, TermPos, _NameOffset),
     '$fetch_vm'(ClauseRef, PC, NPC, _VMI),
     '$clause_term_position'(ClauseRef, NPC, List),
     debug(break, 'ClausePos = ~w', [List]),
-    range(List, TermPos, A, Z),
-    debug(break, 'Range: ~d .. ~d', [A, Z]).
+    range(List, TermPos, SubPos),
+    debug(break, 'Subgoal at: ~p', [SubPos]).
 
 
                  /*******************************
@@ -276,6 +299,10 @@ prolog:message(breakpoint(no_source(ClauseRef, _File, Line))) -->
     [ 'Failed to find line ~d in body of clause ~p.  Breaking at start of body.'-
       [Line, ClauseRef]
     ].
+prolog:message(breakpoint_condition_error(Id, Error)) -->
+    [ 'Exception while evaluating breakpoint ~p condition:'-[Id], nl,
+      prolog:translate_message(Error)
+    ].
 prolog:message(breakpoint(SetClear, Id)) -->
     setclear(SetClear),
     breakpoint(Id).
@@ -290,10 +317,9 @@ setclear(delete) -->
 breakpoint(Id) -->
     breakpoint_name(Id),
     (   { breakpoint_property(Id, file(File)),
-          file_base_name(File, Base),
           breakpoint_property(Id, line_count(Line))
         }
-    ->  [ ' at ~w:~d'-[Base, Line] ]
+    ->  [ ' at ', url(File:Line) ]
     ;   []
     ).
 
@@ -306,3 +332,65 @@ breakpoint_name(Id) -->
     ).
 
 
+		 /*******************************
+		 *    CONDITIONAL BREAKPOINTS   *
+		 *******************************/
+
+%!  set_breakpoint_condition(+Id, +Cond) is det.
+%
+%   Set a condition for of the breakpoint   with given Id. The condition
+%   Cond is a string  that  represents  a   Prolog  goal  to  be invoked
+%   whenever the breakpoint is reached, if  goal fails the breakpoint is
+%   skipped and execution commences normally.
+%
+%   Variables in Cond that match by  name   to  variables  in the source
+%   definition of the clause in  which   the  breakpoint  is located are
+%   unified with the corresponding runtime value of the clause variables
+%   in the current execution  frame,   before  evaluating  the condition
+%   goal.
+%
+%   @error existence_error(breakpoint, Id).
+
+set_breakpoint_condition(Id, Cond) :-
+    known_breakpoint(ClauseRef, _PC, _Location, Id),
+    !,
+    term_string(CondGoal, Cond, [variable_names(Bindings)]),
+    clause_info(ClauseRef, _InfoFile, _TermPos, NameOffset),
+    clause_property(ClauseRef, module(Module)),
+    names_offsets(Bindings, NameOffset, FrameOffsetsCondVars),
+    retractall(breakpoint_condition(Id, _, _, _)),
+    asserta(breakpoint_condition(Id, Cond, Module:CondGoal, FrameOffsetsCondVars)).
+set_breakpoint_condition(Id, _Cond) :-
+    existence_error(breakpoint, Id).
+
+:- multifile prolog:break_hook/7.
+
+prolog:break_hook(Clause, PC, Frame, _Choice, _Goal, true, Action) :-
+    known_breakpoint(Clause, PC, _, Id),
+    (   breakpoint_condition(Id, _CondString, CondGoal, FrameOffsetsCondVars)
+    ->  check_breakpoint_condition(Id, Frame, CondGoal,
+                                   FrameOffsetsCondVars, Action)
+    ;   Action = trace
+    ).
+
+check_breakpoint_condition(Id, Frame, CondGoal, FrameOffsetsCondVars, Action) :-
+    maplist(unify_with_frame_variable(Frame), FrameOffsetsCondVars),
+    (   catch(CondGoal,
+              Error,
+              print_message(warning,
+                            breakpoint_condition_error(Id, Error)))
+    ->  Action = trace
+    ;   Action = continue
+    ).
+
+unify_with_frame_variable(Frame, Offset-Var) :-
+    prolog_frame_attribute(Frame, argument(Offset), Var).
+
+
+names_offsets([Name=Var|T], NameOffset, OffsetsVars) :-
+    (   arg(Offset, NameOffset, Name)
+    ->  OffsetsVars = [Offset-Var|R],
+        names_offsets(T, NameOffset, R)
+    ;   names_offsets(T, NameOffset, OffsetsVars)
+    ).
+names_offsets([], _, []).
