@@ -3,9 +3,10 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2002-2018, University of Amsterdam
+    Copyright (c)  2002-2022, University of Amsterdam
                               VU University Amsterdam
                               CWI, Amsterdam
+                              SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -47,8 +48,12 @@
             delete_directory_and_contents/1, % +Dir
             delete_directory_contents/1 % +Dir
           ]).
-:- use_module(library(apply)).
-:- use_module(library(error)).
+:- autoload(library(apply),[maplist/2,maplist/3,foldl/4]).
+:- autoload(library(error),
+	    [permission_error/3,must_be/2,domain_error/2]).
+:- autoload(library(lists),[member/2]).
+:- autoload(library(nb_set),[empty_nb_set/1,add_nb_set/3]).
+
 
 /** <module> Extended operations on files
 
@@ -77,7 +82,7 @@ implementations from this library is usually faster.
                      ]).
 
 
-:- use_foreign_library(foreign(files), install_files).
+:- use_foreign_library(foreign(files)).
 
 %!  set_time_file(+File, -OldTimes, +NewTimes) is det.
 %
@@ -158,7 +163,10 @@ relative_file_name(Path, RelTo, RelPath) :- % +,+,-
     atomic_list_concat(RL, /, AbsRelTo),
     delete_common_prefix(PL, RL, PL1, PL2),
     to_dot_dot(PL2, DotDot, PL1),
-    atomic_list_concat(DotDot, /, RelPath).
+    (   DotDot == []
+    ->  RelPath = '.'
+    ;   atomic_list_concat(DotDot, /, RelPath)
+    ).
 relative_file_name(Path, RelTo, RelPath) :-
     (   is_absolute_file_name(RelPath)
     ->  Path = RelPath
@@ -361,7 +369,7 @@ matches_type(directory, Entry) :-
     exists_directory(Entry).
 matches_type(Type, Entry) :-
     \+ exists_directory(Entry),
-    prolog_file_type(Ext, Type),
+    user:prolog_file_type(Ext, Type),
     file_name_extension(_, Ext, Entry).
 
 
@@ -514,7 +522,7 @@ delete_directory_contents(Dir) :-
 %   a plain `Mode`, which adds new   permissions, revokes permissions or
 %   sets the exact permissions. `Mode`  itself   is  an integer, a POSIX
 %   mode name or a list of POSIX   mode names. Defines names are `suid`,
-%   `sgid`, `svtx` and the all names   defined by the regular expression
+%   `sgid`, `svtx` and  all names  defined  by  the  regular  expression
 %   =|[ugo]*[rwx]*|=. Specifying none of "ugo" is the same as specifying
 %   all of them. For example, to make   a  file executable for the owner
 %   (user) and group, we can use:
