@@ -555,6 +555,7 @@ allocSocket(SOCKET socket)
 
   DEBUG(2, Sdprintf("[%d]: allocSocket(%d) --> %p\n",
 		    PL_thread_self(), socket, p));
+  DEBUG(4, PL_backtrace(10,1));
 
   return p;
 }
@@ -787,7 +788,7 @@ nbio_error(int code, nbio_error_map mapid)
 			 CompoundArg("error", 2),
 			   CompoundArg("socket_error", 2),
 			     AtomArg(symbol),
-			     AtomArg(msg),
+			     AtomMbArg(msg),
 			   PL_VARIABLE) &&
 	   PL_raise_exception(ex)
 	 );
@@ -1101,6 +1102,7 @@ int
 nbio_get_sockaddr(term_t Address, struct sockaddr_in *addr, term_t *varport)
 { int port;
 
+  memset(addr, 0, sizeof(*addr));
   addr->sin_family = AF_INET;
   addr->sin_addr.s_addr = INADDR_ANY;
 
@@ -1266,12 +1268,17 @@ nbio_accept(nbio_sock_t master, struct sockaddr *addr, socklen_t *addrlen)
       return NULL;
 #endif
 
+    if ( PL_handle_signals() < 0 )
+      return NULL;
+    DEBUG(3, Sdprintf("[%d] calling accept(%d)\n",
+		      PL_thread_self(), master->socket));
     slave = accept(master->socket, addr, addrlen);
+    DEBUG(3, Sdprintf("[%d] accept(%d) --> %d\n",
+		      PL_thread_self(), master->socket, slave));
 
     if ( slave == SOCKET_ERROR )
     { if ( need_retry(GET_ERRNO) )
-      { if ( PL_handle_signals() < 0 )
-	  return NULL;
+      {
 #ifdef __WINDOWS__
         if ( !wait_socket(master) )
           return NULL;
@@ -1426,7 +1433,7 @@ nbio_close_input(nbio_sock_t socket)
 
   VALID_SOCKET(socket);
 
-  DEBUG(2, Sdprintf("[%d]: nbio_close_input(%d, flags=0x%x)\n",
+  DEBUG(2, Sdprintf("[%d]: nbio_close_input(%p, flags=0x%x)\n",
 		    PL_thread_self(), socket, socket->flags));
   if ( true(socket, PLSOCK_INSTREAM) )
   { clear(socket, PLSOCK_INSTREAM);
@@ -1457,7 +1464,7 @@ nbio_close_output(nbio_sock_t socket)
 
   VALID_SOCKET(socket);
 
-  DEBUG(2, Sdprintf("[%d]: nbio_close_output(%d, flags=0x%x)\n",
+  DEBUG(2, Sdprintf("[%d]: nbio_close_output(%p, flags=0x%x)\n",
 		    PL_thread_self(), socket, socket->flags));
 
   if ( true(socket, PLSOCK_OUTSTREAM) )
