@@ -3,9 +3,10 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2000-2020, University of Amsterdam
+    Copyright (c)  2000-2022, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -39,8 +40,8 @@
 /*#define O_DEBUG 1*/
 #undef O_DEBUG
 #define _GNU_SOURCE			/* get pipe2() */
-#include <SWI-Stream.h>
 #include <SWI-Prolog.h>
+#include <SWI-Stream.h>
 #include "error.h"
 #include <stdio.h>
 #include <string.h>
@@ -91,14 +92,26 @@ static atom_t ATOM_window;
 static atom_t ATOM_timeout;
 static atom_t ATOM_release;
 static atom_t ATOM_infinite;
+static atom_t ATOM_text;
+static atom_t ATOM_binary;
+static atom_t ATOM_octet;
+static atom_t ATOM_utf8;
+static atom_t ATOM_ascii;
+static atom_t ATOM_iso_latin_1;
+static atom_t ATOM_unicode_be;
+static atom_t ATOM_unicode_le;
+
 static functor_t FUNCTOR_error2;
 static functor_t FUNCTOR_process_error2;
 static functor_t FUNCTOR_system_error2;
 static functor_t FUNCTOR_pipe1;
+static functor_t FUNCTOR_pipe2;
 static functor_t FUNCTOR_stream1;
 static functor_t FUNCTOR_exit1;
 static functor_t FUNCTOR_killed1;
 static functor_t FUNCTOR_eq2;		/* =/2 */
+static functor_t FUNCTOR_type1;
+static functor_t FUNCTOR_encoding1;
 
 #define MAYBE 2
 
@@ -119,6 +132,7 @@ typedef DWORD  pid_t;
 #endif
 
 typedef wchar_t echar;			/* environment character */
+#define ecslen(s) wcslen(s)
 
 #ifndef CREATE_BREAKAWAY_FROM_JOB
 #define CREATE_BREAKAWAY_FROM_JOB 0x1000000
@@ -126,6 +140,7 @@ typedef wchar_t echar;			/* environment character */
 
 #else
 typedef char echar;
+#define ecslen(s) strlen(s)
 #endif
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -150,6 +165,7 @@ typedef enum std_type
 typedef struct p_stream
 { term_t   term;			/* P in pipe(P) */
   std_type type;			/* type of stream */
+  IOENC	   encoding;			/* Encoding for the stream */
 #ifdef __WINDOWS__
   HANDLE   fd[2];			/* pipe handles */
 #else
@@ -278,11 +294,10 @@ get_echars_arg_ex(int i, term_t from, term_t arg, echar **sp, size_t *lenp)
 #define ECHARS(s) s
 #endif
 
-#ifndef __WINDOWS__
 static int
-already_in_env(const char *env, const char *e)
-{ for(; *env; env += strlen(env)+1)
-  { const char *s, *q;
+already_in_env(const echar *env, int count, const echar *e)
+{ for(; count-- > 0; env += ecslen(env)+1)
+  { const echar *s, *q;
 
     for(s=env, q=e; *s && *q && *s == *q && *s != '=' && *q != '='; s++,q++)
       ;
@@ -292,7 +307,6 @@ already_in_env(const char *env, const char *e)
 
   return FALSE;
 }
-#endif
 
 static int
 parse_environment(term_t t, p_options *info, int pass)
@@ -307,9 +321,8 @@ parse_environment(term_t t, p_options *info, int pass)
   int c = 0;
 #endif
 
-  assert(eb->size == 0);
-  assert(eb->allocated == 0);
-  assert(eb->buffer == NULL);
+  if ( eb->buffer )
+    return PL_permission_error("redefine", "environment", t);
 
   while( PL_get_list(tail, head, tail) )
   { echar *s;
@@ -333,28 +346,49 @@ parse_environment(term_t t, p_options *info, int pass)
   if ( !PL_get_nil_ex(tail) )
     return FALSE;
 
-#ifndef __WINDOWS__
+  if ( pass && count == 0 )
+    return TRUE;			/* environment([]) is a no-op */
+
   if ( pass )
   {
+#ifndef __WINDOWS__
 #ifdef HAVE__NSGETENVIRON
     char **environ = *_NSGetEnviron();
 #else
     extern char **environ;
 #endif
     char **e;
+    int count0 = count;
 
     for(e=environ; e && *e; e++)
-    { if ( !already_in_env(eb->buffer, *e) )
+    { if ( !already_in_env(eb->buffer, count0, *e) )
       { add_ecbuf(eb, *e, strlen(*e));
 	add_ecbuf(eb, ECHARS("\0"), 1);
 	count++;
       }
     }
-  }
+#else /*__WINDOWS__*/
+    echar *env = GetEnvironmentStringsW();
+
+    if ( env )
+    { int count0 = count;
+
+      while(*env)
+      { size_t len = wcslen(env);
+	if ( !already_in_env(eb->buffer, count0, env) )
+	{ add_ecbuf(eb, env, len+1);
+	  count++;
+	}
+	env += len+1;
+      }
+    }
 #endif /*__WINDOWS__*/
+  }
 
 #ifdef __WINDOWS__
   add_ecbuf(eb, ECHARS("\0"), 1);
+  if ( count == 0 )
+    add_ecbuf(eb, ECHARS("\0"), 1);
 #else
   info->envp = PL_malloc((count+1)*sizeof(char*));
 
@@ -370,6 +404,47 @@ parse_environment(term_t t, p_options *info, int pass)
 }
 
 
+static int
+get_type(term_t head, IOENC *enc)
+{ atom_t a;
+
+  if ( PL_get_atom_ex(head, &a) )
+  { if ( a == ATOM_text )
+      *enc = ENC_ANSI;
+    else if ( a == ATOM_binary )
+      *enc = ENC_OCTET;
+    else
+      return PL_domain_error("stream_type", head);
+
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
+/* TBD: provide a public API for translating encoding names to
+ * the IOENC enum
+ */
+
+static int
+get_encoding(term_t head, IOENC *enc)
+{ atom_t a;
+
+  if ( PL_get_atom_ex(head, &a) )
+  { IOENC e;
+
+    if ( (e=PL_atom_to_encoding(a)) != ENC_UNKNOWN )
+    { *enc = e;
+      return TRUE;
+    }
+
+    return PL_domain_error("encoding", head);
+  }
+
+  return FALSE;
+}
+
+
 static int
 get_stream(term_t t, p_options *info, p_stream *stream, atom_t name)
 { atom_t a;
@@ -384,8 +459,10 @@ get_stream(term_t t, p_options *info, p_stream *stream, atom_t name)
     } else
     { return PL_domain_error("process_stream", t);
     }
-  } else if ( PL_is_functor(t, FUNCTOR_pipe1) )
+  } else if ( PL_is_functor(t, FUNCTOR_pipe1) ||
+	      PL_is_functor(t, FUNCTOR_pipe2) )
   { stream->term = PL_new_term_ref();
+    stream->encoding = ENC_ANSI;
     _PL_get_arg(1, t, stream->term);
     if ( !PL_is_variable(stream->term) )
     { for (i = 0; i < info->pipes; i++)
@@ -395,6 +472,26 @@ get_stream(term_t t, p_options *info, p_stream *stream, atom_t name)
       if (i == info->pipes)
         return PL_uninstantiation_error(stream->term);
     }
+    if ( PL_is_functor(t, FUNCTOR_pipe2) )
+    { term_t tail = PL_new_term_ref();
+      term_t head = PL_new_term_ref();
+
+      _PL_get_arg(2, t, tail);
+      while(PL_get_list_ex(tail, head, tail))
+      { if ( PL_is_functor(head, FUNCTOR_type1) )
+	{ _PL_get_arg(1, head, head);
+	  if ( !get_type(head, &stream->encoding) )
+	    return FALSE;
+	} else if ( PL_is_functor(head, FUNCTOR_encoding1) )
+	{ _PL_get_arg(1, head, head);
+	  if ( !get_encoding(head, &stream->encoding) )
+	    return FALSE;
+	} else
+	  return PL_domain_error("pipe_option", head);
+      }
+      if ( !PL_get_nil_ex(tail) )
+	return FALSE;
+    }
     stream->type = std_pipe;
     info->pipes++;
     return TRUE;
@@ -682,13 +779,15 @@ static IOFUNCTIONS Sprocessfunctions =
 
 
 static IOSTREAM *
-#ifdef __WINDOWS__
-open_process_pipe(process_context *pc, int which, HANDLE fd)
-#else
-open_process_pipe(process_context *pc, int which, int fd)
-#endif
+open_process_pipe(process_context *pc, p_options *info, int which, int fdn)
 { void *handle;
-  int flags;
+#ifdef __WINDOWS__
+  HANDLE fd = info->streams[which].fd[fdn];
+#else
+  int fd = info->streams[which].fd[fdn];
+#endif
+  int flags = SIO_RECORDPOS|SIO_FBUF;
+  IOSTREAM *s;
 
   pc->open_mask |= (1<<which);
 #ifdef __WINDOWS__
@@ -697,16 +796,20 @@ open_process_pipe(process_context *pc, int which, int fd)
   pc->pipes[which] = fd;
 #endif
 
-#define ISO_FLAGS (SIO_RECORDPOS|SIO_FBUF|SIO_TEXT)
+  if ( info->streams[which].encoding != ENC_OCTET )
+    flags |= SIO_TEXT;
 
   if ( which == 0 )
-    flags = SIO_OUTPUT|ISO_FLAGS;
+    flags |= SIO_OUTPUT;
   else
-    flags = SIO_INPUT|ISO_FLAGS;
+    flags |= SIO_INPUT;
 
   handle = (void *)((uintptr_t)pc | (uintptr_t)which);
 
-  return Snew(handle, flags, &Sprocessfunctions);
+  if ( (s=Snew(handle, flags, &Sprocessfunctions)) )
+    s->encoding = info->streams[which].encoding;
+
+  return s;
 }
 
 
@@ -1237,14 +1340,14 @@ do_create_process(p_options *info)
 
       if ( info->streams[0].type == std_pipe )
       { CloseHandle(info->streams[0].fd[0]);
-	if ( (s = open_process_pipe(pc, 0, info->streams[0].fd[1])) )
+	if ( (s = open_process_pipe(pc, info, 0, 1)) )
 	  rc = PL_unify_stream(info->streams[0].term, s);
 	else
 	  CloseHandle(info->streams[0].fd[1]);
       }
       if ( info->streams[1].type == std_pipe )
       { CloseHandle(info->streams[1].fd[1]);
-	if ( rc && (s = open_process_pipe(pc, 1, info->streams[1].fd[0])) )
+	if ( rc && (s = open_process_pipe(pc, info, 1, 0)) )
 	  PL_unify_stream(info->streams[1].term, s);
 	else
 	  CloseHandle(info->streams[1].fd[0]);
@@ -1252,7 +1355,7 @@ do_create_process(p_options *info)
       if ( info->streams[2].type == std_pipe &&
            ( !info->streams[1].term || PL_compare(info->streams[1].term, info->streams[2].term) != 0 ) )
       { CloseHandle(info->streams[2].fd[1]);
-	if ( rc && (s = open_process_pipe(pc, 2, info->streams[2].fd[0])) )
+	if ( rc && (s = open_process_pipe(pc, info, 2, 0)) )
 	  rc = PL_unify_stream(info->streams[2].term, s);
 	else
 	  CloseHandle(info->streams[2].fd[0]);
@@ -1556,6 +1659,23 @@ close_ok(int fd)
   return rc;
 }
 
+static IOSTREAM *
+p_fdopen(p_options *info, int which, int fdn, char *mode)
+{ IOSTREAM *s;
+  char m[10];
+  char *mp = m;
+
+  *mp++ = mode[0];
+  if ( info->streams[which].encoding == ENC_OCTET )
+    *mp++ = 'b';
+  *mp = 0;
+
+  if ( (s=Sfdopen(info->streams[which].fd[fdn], m)) )
+    s->encoding = info->streams[which].encoding;
+
+  return s;
+}
+
 
 static int
 process_parent_side(p_options *info, int pid)
@@ -1574,14 +1694,14 @@ process_parent_side(p_options *info, int pid)
 
     if ( info->streams[0].type == std_pipe )
     { close_ok(info->streams[0].fd[0]);
-      if ( (s = open_process_pipe(pc, 0, info->streams[0].fd[1])) )
+      if ( (s = open_process_pipe(pc, info, 0, 1)) )
 	rc = PL_unify_stream(info->streams[0].term, s);
       else
 	close_ok(info->streams[0].fd[1]);
     }
     if ( info->streams[1].type == std_pipe )
     { close_ok(info->streams[1].fd[1]);
-      if ( rc && (s = open_process_pipe(pc, 1, info->streams[1].fd[0])) )
+      if ( rc && (s = open_process_pipe(pc, info, 1, 0)) )
 	rc = PL_unify_stream(info->streams[1].term, s);
       else
 	close_ok(info->streams[1].fd[0]);
@@ -1589,7 +1709,7 @@ process_parent_side(p_options *info, int pid)
     if ( info->streams[2].type == std_pipe &&
 	 ( !info->streams[1].term || PL_compare(info->streams[1].term, info->streams[2].term) != 0 ) )
     { close_ok(info->streams[2].fd[1]);
-      if ( rc && (s = open_process_pipe(pc, 2, info->streams[2].fd[0])) )
+      if ( rc && (s = open_process_pipe(pc, info, 2, 0)) )
 	rc = PL_unify_stream(info->streams[2].term, s);
       else
 	close_ok(info->streams[2].fd[0]);
@@ -1601,14 +1721,14 @@ process_parent_side(p_options *info, int pid)
 
     if ( info->streams[0].type == std_pipe )
     { close_ok(info->streams[0].fd[0]);
-      if ( (s = Sfdopen(info->streams[0].fd[1], "w")) )
+      if ( (s = p_fdopen(info, 0, 1, "w")) )
 	rc = PL_unify_stream(info->streams[0].term, s);
       else
 	close_ok(info->streams[0].fd[1]);
     }
     if ( info->streams[1].type == std_pipe )
     { close_ok(info->streams[1].fd[1]);
-      if ( rc && (s = Sfdopen(info->streams[1].fd[0], "r")) )
+      if ( rc && (s = p_fdopen(info, 1, 0, "r")) )
 	rc = PL_unify_stream(info->streams[1].term, s);
       else
 	close_ok(info->streams[1].fd[0]);
@@ -1616,7 +1736,7 @@ process_parent_side(p_options *info, int pid)
     if ( info->streams[2].type == std_pipe &&
 	 ( !info->streams[1].term || PL_compare(info->streams[1].term, info->streams[2].term) != 0 ) )
     { close_ok(info->streams[2].fd[1]);
-      if ( rc && (s = Sfdopen(info->streams[2].fd[0], "r")) )
+      if ( rc && (s = p_fdopen(info, 2, 0, "r")) )
 	PL_unify_stream(info->streams[2].term, s);
       else
 	close_ok(info->streams[2].fd[0]);
@@ -1673,12 +1793,16 @@ do_create_process_fork(p_options *info, create_method method)
   sigset_t set;
 
   blockSignal(SIGPROF, &set);
+
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
   if ( method == PCREATE_VFORK )
-    pid = vfork();
+    pid = vfork();				/* As long as it is there we use it */
   else
     pid = fork();
   if ( pid != 0 )				/* parent */
     restoreSignals(&set);
+#pragma GCC diagnostic pop
 
   if ( pid == 0 )				/* child */
   { int fd;
@@ -2142,8 +2266,19 @@ install_process()
   MKATOM(timeout);
   MKATOM(release);
   MKATOM(infinite);
+  MKATOM(text);
+  MKATOM(binary);
+  MKATOM(octet);
+  MKATOM(utf8);
+  MKATOM(ascii);
+  MKATOM(iso_latin_1);
+  MKATOM(unicode_be);
+  MKATOM(unicode_le);
 
   MKFUNCTOR(pipe, 1);
+  MKFUNCTOR(pipe, 2);
+  MKFUNCTOR(type, 1);
+  MKFUNCTOR(encoding, 1);
   MKFUNCTOR(stream, 1);
   MKFUNCTOR(error, 2);
   MKFUNCTOR(process_error, 2);
