@@ -3,8 +3,10 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2000-2016, University of Amsterdam
+    Copyright (c)  2000-2022, University of Amsterdam
                               VU University Amsterdam
+                              CWI, Amsterdam
+                              SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -45,6 +47,7 @@
             tcp_listen/2,               % +Socket, +BackLog
             tcp_fcntl/3,                % +Socket, +Command, ?Arg
             tcp_setopt/2,               % +Socket, +Option
+            tcp_getopt/2,               % +Socket, ?Option
             tcp_host_to_address/2,      % ?HostName, ?Ip-nr
             tcp_select/3,               % +Inputs, -Ready, +Timeout
             gethostname/1,              % -HostName
@@ -57,9 +60,9 @@
 
             negotiate_socks_connection/2% +DesiredEndpoint, +StreamPair
           ]).
-:- use_module(library(shlib)).
-:- use_module(library(debug)).
-:- use_module(library(lists)).
+:- autoload(library(debug),[debug/3]).
+:- autoload(library(lists),[last/2]).
+
 
 /** <module> Network socket (TCP and UDP) library
 
@@ -129,7 +132,7 @@ in case of failure or exceptions.
 
   ==
   dispatch(AcceptFd) :-
-          tcp_accept(AcceptFd, Socket, _Peer),
+          tcp_accept(AcceptFd, Socket, Peer),
           thread_create(process_client(Socket, Peer), _,
                         [ detached(true)
                         ]),
@@ -138,13 +141,31 @@ in case of failure or exceptions.
   process_client(Socket, Peer) :-
           setup_call_cleanup(
               tcp_open_socket(Socket, StreamPair),
-              handle_service(In, StreamPair),
+              handle_service(StreamPair),
               close(StreamPair)).
 
   handle_service(StreamPair) :-
           ...
   ==
 
+## Socket exceptions			{#socket-exceptions}
+
+Errors that are trapped by  the  low-level   library  are  mapped  to an
+exception of the shape below. In this term,  `Code` is a lower case atom
+that corresponds to the C macro name,   e.g., `epipe` for a broken pipe.
+`Message` is the human readable string for   the  error code returned by
+the OS or  the  same  as  `Code`  if   the  OS  does  not  provide  this
+functionality. Note that `Code` is derived from   a static set of macros
+that may or may not be defines for the   target OS. If the macro name is
+not known, `Code` is =|ERROR_nnn|=, where _nnn_ is an integer.
+
+    error(socket_error(Code, Message), _)
+
+Note that on Windows `Code` is a ``wsa*``   code  which makes it hard to
+write portable code that handles specific   socket errors. Even on POSIX
+systems the exact set of errors  produced   by  the network stack is not
+defined.
+
 ## TCP socket predicates                {#socket-predicates}
 */
 
@@ -159,9 +180,13 @@ in case of failure or exceptions.
                        nodelay(boolean)
                      ]).
 
-:- use_foreign_library(foreign(socket), install_socket).
+:- use_foreign_library(foreign(socket)).
 :- public tcp_debug/1.                  % set debugging.
 
+:- if(current_predicate(unix_domain_socket/1)).
+:- export(unix_domain_socket/1).  % -Socket
+:- endif.
+
 %!  tcp_socket(-SocketId) is det.
 %
 %   Creates an INET-domain stream-socket and   unifies an identifier
@@ -241,12 +266,13 @@ tcp_open_socket(Socket, Stream) :-
 
 %!  tcp_accept(+Socket, -Slave, -Peer) is det.
 %
-%   This predicate waits on a server socket for a connection request
-%   by a client. On success, it creates  a new socket for the client
-%   and binds the  identifier  to  Slave.   Peer  is  bound  to  the
-%   IP-address of the client.
+%   This predicate waits on a server socket  for a connection request by
+%   a client. On success, it creates  a   new  socket for the client and
+%   binds the identifier to Slave. Peer is   bound  to the IP-address of
+%   the client or the atom `af_unix` if Socket is an AF_UNIX socket (see
+%   unix_domain_socket/1).
 
-%!  tcp_connect(+SocketId, +HostAndPort) is det.
+%!  tcp_connect(+SocketId, +Address) is det.
 %
 %   Connect SocketId. After successful completion, tcp_open_socket/3
 %   can be used to create  I/O-Streams   to  the remote socket. This
@@ -270,6 +296,9 @@ tcp_open_socket(Socket, Stream) :-
 %             talk(StreamPair),
 %             close(StreamPair))
 %     ==
+%
+%   If SocketId is an AF_UNIX socket (see unix_domain_socket/1), Address
+%   is an atom or string denoting a file name.
 
 
                  /*******************************
@@ -309,11 +338,10 @@ tcp_connect(Socket, Address, Read, Write) :-
 %!  tcp_connect(+Address, -StreamPair, +Options) is det.
 %!  tcp_connect(+Socket, +Address, -StreamPair) is det.
 %
-%   Establish a TCP communication as a client. The +,-,+ mode is the
-%   preferred way for a  client  to   establish  a  connection. This
-%   predicate can be hooked to  support   network  proxies. To use a
-%   proxy, the hook  proxy_for_url/3  must   be  defined.  Permitted
-%   options are:
+%   Establish a TCP communication as a  client.   The  +,-,+ mode is the
+%   preferred way for a client to establish a connection. This predicate
+%   can be hooked to support network proxies.   To use a proxy, the hook
+%   proxy_for_url/3 must be defined. Permitted options are:
 %
 %      * bypass_proxy(+Boolean)
 %        Defaults to =false=. If =true=, do not attempt to use any
@@ -323,18 +351,22 @@ tcp_connect(Socket, Address, Read, Write) :-
 %        Defaults to =false=. If =true=, set nodelay on the
 %        resulting socket using tcp_setopt(Socket, nodelay)
 %
-%   The +,+,- mode is deprecated and   does  not support proxies. It
-%   behaves like tcp_connect/4,  but  creates   a  stream  pair (see
+%   The +,+,- mode is  deprecated  and   does  not  support  proxies. It
+%   behaves  like  tcp_connect/4,  but  creates    a  stream  pair  (see
 %   stream_pair/3).
 %
-%   @error proxy_error(tried(ResultList)) is raised  by mode (+,-,+)
-%   if proxies are defines  by  proxy_for_url/3   but  no  proxy can
-%   establsh the connection. `ResultList` contains one or more terms
-%   of the form false(Proxy)  for  a   hook  that  simply  failed or
-%   error(Proxy, ErrorTerm) for a hook that raised an exception.
+%   @arg Address is either a Host:Port  term   or  a  file name (atom or
+%   string). The latter connects  to  an   AF_UNIX  socket  and requires
+%   unix_domain_socket/1.
 %
-%   @see library(http/http_proxy) defines a  hook   that  allows  to
-%   connect through HTTP proxies that support the =CONNECT= method.
+%   @error proxy_error(tried(ResultList)) is raised by   mode (+,-,+) if
+%   proxies are defines by proxy_for_url/3 but no proxy can establsh the
+%   connection. `ResultList` contains one or  more   terms  of  the form
+%   false(Proxy)  for  a  hook  that    simply  failed  or  error(Proxy,
+%   ErrorTerm) for a hook that raised an exception.
+%
+%   @see library(http/http_proxy) defines a hook  that allows to connect
+%   through HTTP proxies that support the =CONNECT= method.
 
 % Main mode: +,-,+
 tcp_connect(Address, StreamPair, Options) :-
@@ -367,24 +399,38 @@ tcp_connect(Socket, Address, StreamPair) :-
 
 
 tcp_connect_direct(Address, Socket, StreamPair):-
-    tcp_socket(Socket),
+    make_socket(Address, Socket),
     catch(tcp_connect(Socket, Address, StreamPair),
           Error,
           ( tcp_close_socket(Socket),
             throw(Error)
           )).
 
+:- if(current_predicate(unix_domain_socket/1)).
+make_socket(Address, Socket) :-
+    (   atom(Address)
+    ;   string(Address)
+    ),
+    !,
+    unix_domain_socket(Socket).
+:- endif.
+make_socket(_Address, Socket) :-
+    tcp_socket(Socket).
+
+
 %!  tcp_select(+ListOfStreams, -ReadyList, +TimeOut)
 %
-%   Same as the built-in  wait_for_input/3,   but  integrates better
-%   with event processing and the  various   options  of sockets for
-%   Windows.   On   non-windows   systems     this    simply   calls
-%   wait_for_input/3.
+%   Same as the built-in wait_for_input/3. Used  to allow for interrupts
+%   and timeouts on Windows. A redesign  of the Windows socket interface
+%   makes  it  impossible  to  do  better  than  Windows  select()  call
+%   underlying wait_for_input/3. As input multiplexing typically happens
+%   in a background thread anyway we  accept   the  loss of timeouts and
+%   interrupts.
+%
+%   @deprecated Use wait_for_input/3
 
-:- if(\+predicate_property(tcp_select(_,_,_), defined)).
 tcp_select(ListOfStreams, ReadyList, TimeOut) :-
     wait_for_input(ListOfStreams, ReadyList, TimeOut).
-:- endif.
 
 
                  /*******************************
@@ -509,12 +555,29 @@ try_proxy(socks(Host, Port), Address, Socket, StreamPair) :-
 %     matching the address. The address is normally the address of
 %     the local subnet (i.e. 192.168.1.255).  See udp_send/4.
 %
+%     - ip_add_membership(+MultiCastGroup)
+%     - ip_add_membership(+MultiCastGroup, +LocalInterface)
+%     - ip_add_membership(+MultiCastGroup, +LocalInterface, +InterfaceIndex)
+%     - ip_drop_membership(+MultiCastGroup)
+%     - ip_drop_membership(+MultiCastGroup, +LocalInterface)
+%     - ip_drop_membership(+MultiCastGroup, +LocalInterface, +InterfaceIndex)
+%     Join/leave a multicast group.  Calls setsockopt() with the
+%     corresponding arguments.
+%
 %     - dispatch(+Boolean)
 %     In GUI environments (using XPCE or the Windows =swipl-win.exe=
 %     executable) this flags defines whether or not any events are
 %     dispatched on behalf of the user interface. Default is
 %     =true=. Only very specific situations require setting
 %     this to =false=.
+%
+%     - sndbuf(+Integer)
+%     Sets the send buffer size to Integer (bytes). On Windows this defaults
+%     (now) to 64kb. Higher latency links may benefit from increasing this
+%     further since the maximum theoretical throughput on a link is given by
+%     buffer-size / latency.
+%     See https://support.microsoft.com/en-gb/help/823764/slow-performance-occurs-when-you-copy-data-to-a-tcp-server-by-using-a
+%     for Microsoft's discussion
 
 %!  tcp_fcntl(+Stream, +Action, ?Argument) is det.
 %
@@ -534,6 +597,15 @@ tcp_fcntl(Socket, setfl, nonblock) :-
     !,
     tcp_setopt(Socket, nonblock).
 
+%!  tcp_getopt(+Socket, ?Option) is semidet.
+%
+%   Get  information  about  Socket.  Defined    properties  are  below.
+%   Requesting an unknown option results in a `domain_error` exception.
+%
+%     - file_no(-File)
+%     Get the OS file handle as an integer.  This may be used for
+%     debugging and integration.
+
 %!  tcp_host_to_address(?HostName, ?Address) is det.
 %
 %   Translate between a machines host-name and it's (IP-)address. If
@@ -627,13 +699,13 @@ negotiate_socks_connection(Host:Port, StreamPair):-
 The C-layer generates exceptions of the  following format, where Message
 is extracted from the operating system.
 
-        error(socket_error(Message), _)
+        error(socket_error(Code, Message), _)
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 :- multifile
     prolog:error_message//1.
 
-prolog:error_message(socket_error(Message)) -->
+prolog:error_message(socket_error(_Code, Message)) -->
     [ 'Socket error: ~w'-[Message] ].
 prolog:error_message(socks_error(Error)) -->
     socks_error(Error).
