@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2002-2017, University of Amsterdam
+    Copyright (c)  2002-2022, University of Amsterdam
                               VU University Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -378,15 +379,16 @@ cleanup(int rc, void *arg)
 { Event ev;
   schedule *sched = TheSchedule();
 
+  sched->stop = TRUE;
   while( (ev=sched->first) )
-  { removeEvent(ev);
-  }
+    removeEvent(ev);
 
   cleanupHandler();
 
   if ( scheduler_running )
-  { sched->stop = TRUE;
+  { LOCK();
     pthread_cond_signal(&cond);
+    UNLOCK();
 
     pthread_join(scheduler, NULL);
     scheduler_running = FALSE;
@@ -404,13 +406,13 @@ cleanup_thread(void *data)
 
   if ( sched->first )
   { Event ev, next;
-    pthread_t self = pthread_self();
+    int self = PL_thread_self();
 
     LOCK();
     for( ev=sched->first; ev; ev=next )
     { next = ev->next;
 
-      if ( pthread_equal(self, ev->thread_id) )
+      if ( self == ev->pl_thread_id )
       { DEBUG(1, Sdprintf("[%d] removing alarm %ld at exit\n",
 			  PL_thread_self(), (long)(intptr_t)ev));
 	if ( sched->scheduled == ev )
@@ -418,9 +420,8 @@ cleanup_thread(void *data)
 	freeEvent(ev);
       }
     }
-    UNLOCK();
-
     pthread_cond_signal(&cond);
+    UNLOCK();
   }
 }
 
@@ -499,7 +500,7 @@ alarm_loop(void * closure)
   signalled.bits = malloc(signalled.size*sizeof(int));
   signalled.high = 0;
 
-  pthread_mutex_lock(&mutex);		/* for condition variable */
+  LOCK();				/* for condition variable */
   DEBUG(1, Sdprintf("Iterating alarm_loop()\n"));
 
   while( !sched->stop )
@@ -572,6 +573,8 @@ alarm_loop(void * closure)
     }
   }
 
+  free(signalled.bits);
+
   return NULL;
 }
 
@@ -637,7 +640,9 @@ on_alarm(int sig)
   }
 
   DEBUG(1, Sdprintf("Processed pending events; signalling scheduler\n"));
+  LOCK();
   pthread_cond_signal(&cond);
+  UNLOCK();
 }
 
 
@@ -668,11 +673,9 @@ installEvent(Event ev)
     scheduler_running = TRUE;
   }
 
-  rc = insertEvent(ev);
-  UNLOCK();
-
-  if ( rc )
+  if ( (rc = insertEvent(ev)) )
     pthread_cond_signal(&cond);
+  UNLOCK();
 
   return rc;
 }
@@ -685,9 +688,8 @@ uninstallEvent(Event ev)
     ev->flags |= EV_DONE;
   unlinkEvent(ev);
   ev->flags &= ~(EV_FIRED|EV_DONE);
-  UNLOCK();
-
   pthread_cond_signal(&cond);
+  UNLOCK();
 
   return TRUE;
 }
@@ -699,9 +701,8 @@ removeEvent(Event ev)
   if ( TheSchedule()->scheduled == ev )
     ev->flags |= EV_DONE;
   freeEvent(ev);
-  UNLOCK();
-
   pthread_cond_signal(&cond);
+  UNLOCK();
 
   return TRUE;
 }
@@ -739,7 +740,10 @@ unify_timer(term_t t, Event ev)
 
 static int
 get_timer(term_t t, Event *ev)
-{ if ( PL_is_functor(t, FUNCTOR_alarm1) )
+{ if ( TheSchedule()->stop )
+    return FALSE;
+
+  if ( PL_is_functor(t, FUNCTOR_alarm1) )
   { term_t a = PL_new_term_ref();
     void *p;
 
@@ -920,12 +924,17 @@ uninstall_alarm(term_t alarm)
 
 static foreign_t
 remove_alarm(term_t alarm)
-{ Event ev = NULL;
+{ if ( !TheSchedule()->stop )
+  { Event ev = NULL;
 
-  if ( !get_timer(alarm, &ev) )
-    return FALSE;
 
-  return removeEvent(ev);
+    if ( !get_timer(alarm, &ev) )
+      return FALSE;
+
+    return removeEvent(ev);
+  }
+
+  return TRUE;
 }
 
 
