@@ -3,7 +3,8 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2009-2018, VU University Amsterdam
+    Copyright (c)  2009-2023, VU University Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -43,17 +44,12 @@
 #include <wchar.h>
 #include <wctype.h>
 #include <assert.h>
+#include "utf8.h"
 
 static size_t removed_dot_segments(size_t len, const pl_wchar_t *in,
 				   pl_wchar_t *out);
 static pl_wchar_t *remove_last_segment(const pl_wchar_t *base,
 				       const pl_wchar_t *o);
-static char *_utf8_put_char(char *out, int chr);
-
-#define ISUTF8_MB(c) ((unsigned)(c) >= 0xc0 && (unsigned)(c) <= 0xfd)
-#define utf8_put_char(out, chr) \
-	((chr) < 0x80 ? out[0]=(char)(chr), out+1 \
-		      : _utf8_put_char(out, (chr)))
 
 
 		 /*******************************
@@ -83,7 +79,7 @@ syntax_error(const char *culprit)
        PL_unify_term(ex,
 		     PL_FUNCTOR, FUNCTOR_error2,
 		       PL_FUNCTOR, FUNCTOR_syntax_error1,
-		         PL_CHARS, culprit,
+			 PL_CHARS, culprit,
 		       PL_VARIABLE) )
     return PL_raise_exception(ex);
 
@@ -99,7 +95,7 @@ type_error(const char *expected, term_t found)
        PL_unify_term(ex,
 		     PL_FUNCTOR, FUNCTOR_error2,
 		       PL_FUNCTOR, FUNCTOR_type_error2,
-		         PL_CHARS, expected,
+			 PL_CHARS, expected,
 			 PL_TERM, found,
 		       PL_VARIABLE) )
     return PL_raise_exception(ex);
@@ -116,7 +112,7 @@ domain_error(const char *expected, term_t found)
        PL_unify_term(ex,
 		     PL_FUNCTOR, FUNCTOR_error2,
 		       PL_FUNCTOR, FUNCTOR_domain_error2,
-		         PL_CHARS, expected,
+			 PL_CHARS, expected,
 			 PL_TERM, found,
 		       PL_VARIABLE) )
     return PL_raise_exception(ex);
@@ -511,7 +507,13 @@ add_normalized_range_charbuf(charbuf *cb, const range *r, int iri, int flags)
     { const pl_wchar_t *e;
 
       if ( (e=get_encoded_utf8(s, &c)) )
-      { s = e;
+      { if ( flags == ESC_QUERY &&
+	     (c == '=' || c == '&' || c == ';') )
+	{ while( s<e )
+	    add_charbuf(cb, *s++);
+	  continue;
+	}
+	s = e;
       } else if (hex(s+1, 2, &c) )
       { s += 3;
       } else
@@ -876,10 +878,12 @@ unify_query_string_components(term_t list, size_t len, const pl_wchar_t *qs)
       { return syntax_error("illegal_uri_query");
       }
 
+      PL_STRINGS_MARK();
       PL_put_variable(nv+0);
       PL_put_variable(nv+1);
       unify_decoded_atom(nv+0, &name, ESC_QNAME);
       unify_decoded_atom(nv+1, &value, ESC_QVALUE);
+      PL_STRINGS_RELEASE();
 
       if ( !PL_cons_functor_v(eq, FUNCTOR_equal2, nv) ||
 	   !PL_unify_list(tail, head, tail) ||
@@ -925,7 +929,7 @@ uri_query_components(term_t string, term_t list)
   size_t len;
 
   if ( PL_get_wchars(string, &len, &s, CVT_ATOM|CVT_STRING|CVT_LIST) )
-  { return  unify_query_string_components(list, len, s);
+  { return unify_query_string_components(list, len, s);
   } else if ( PL_is_list(list) )
   { term_t tail = PL_copy_term_ref(list);
     term_t head = PL_new_term_ref();
@@ -1260,8 +1264,7 @@ ranges_in_charbuf(charbuf *cb, uri_component_ranges *ranges)
 
 
 typedef struct
-{ atom_t	       atom;
-  pl_wchar_t	      *text;
+{ pl_wchar_t	      *text;
   uri_component_ranges ranges;
 } base_cache;
 
@@ -1276,10 +1279,8 @@ free_base_cache(void *cache)
   if ( PL_query(PL_QUERY_HALTING) )
     return;
 
-  if ( base->atom )
-  { PL_unregister_atom(base->atom);
+  if ( base->text )
     PL_free(base->text);
-  }
 
   PL_free(base);
 }
@@ -1305,34 +1306,24 @@ static base_cache base_store;
 
 static const uri_component_ranges *
 base_ranges(term_t t)
-{ atom_t a;
+{ size_t len;
+  pl_wchar_t *s;
 
-  if ( PL_get_atom(t, &a) )
+  if ( PL_get_wchars(t, &len, &s, CVT_ATOM|CVT_STRING|BUF_MALLOC|CVT_EXCEPTION) )
   { base_cache *base = myBase();
 
-    if ( base->atom != a )
-    { size_t len;
-      pl_wchar_t *s;
-
-      if ( base->atom )
-      { PL_unregister_atom(base->atom);
-	PL_free(base->text);
-	base->atom = 0;
+    if ( !base->text || wcscmp(base->text, s) != 0 )
+    { if ( base->text )
+      { PL_free(base->text);
 	base->text = NULL;
       }
-      if ( !PL_get_wchars(t, &len, &s, CVT_ATOM|BUF_MALLOC|CVT_EXCEPTION) )
-	return NULL;
-      base->atom = a;
-      PL_register_atom(a);
       base->text = s;
       parse_uri(&base->ranges, len, s);
     }
 
     return &base->ranges;
   } else
-  { type_error("atom", t);
-    return NULL;
-  }
+    return FALSE;
 }
 
 
@@ -1541,46 +1532,6 @@ removed_dot_segments(size_t len, const pl_wchar_t *in, pl_wchar_t *out)
 		 *	    IRI HANDLING	*
 		 *******************************/
 
-#define utf8_put_char(out, chr) \
-	((chr) < 0x80 ? out[0]=(char)(chr), out+1 \
-		      : _utf8_put_char(out, (chr)))
-
-
-static char *
-_utf8_put_char(char *out, int chr)
-{ if ( chr < 0x80 )
-  { *out++ = chr;
-  } else if ( chr < 0x800 )
-  { *out++ = 0xc0|((chr>>6)&0x1f);
-    *out++ = 0x80|(chr&0x3f);
-  } else if ( chr < 0x10000 )
-  { *out++ = 0xe0|((chr>>12)&0x0f);
-    *out++ = 0x80|((chr>>6)&0x3f);
-    *out++ = 0x80|(chr&0x3f);
-  } else if ( chr < 0x200000 )
-  { *out++ = 0xf0|((chr>>18)&0x07);
-    *out++ = 0x80|((chr>>12)&0x3f);
-    *out++ = 0x80|((chr>>6)&0x3f);
-    *out++ = 0x80|(chr&0x3f);
-  } else if ( chr < 0x4000000 )
-  { *out++ = 0xf8|((chr>>24)&0x03);
-    *out++ = 0x80|((chr>>18)&0x3f);
-    *out++ = 0x80|((chr>>12)&0x3f);
-    *out++ = 0x80|((chr>>6)&0x3f);
-    *out++ = 0x80|(chr&0x3f);
-  } else if ( (unsigned)chr < 0x80000000 )
-  { *out++ = 0xfc|((chr>>30)&0x01);
-    *out++ = 0x80|((chr>>24)&0x3f);
-    *out++ = 0x80|((chr>>18)&0x3f);
-    *out++ = 0x80|((chr>>12)&0x3f);
-    *out++ = 0x80|((chr>>6)&0x3f);
-    *out++ = 0x80|(chr&0x3f);
-  }
-
-  return out;
-}
-
-
 /** uri_iri(+URI, -IRI) is det.
     uri_iri(-URI, +IRI) is det.
 
