@@ -2,6 +2,7 @@ import gc
 import json
 import logging
 import os
+import uuid
 from datetime import time
 from tempfile import gettempdir, mkdtemp
 import sys
@@ -319,6 +320,25 @@ class TestPrologMQI(ParametrizedTestCase):
                 self.round_trip_prolog(client, "a('1b')")
                 self.round_trip_prolog(client, "'a b'(['1b', 'a b'])")
 
+    def test_goal_expansion(self):
+        with PrologMQI(
+            self.launchServer,
+            self.serverPort,
+            self.password,
+            self.useUnixDomainSocket,
+            prolog_path=self.prologPath,
+        ) as server:
+
+            with server.create_thread() as client:
+                # This requires goal expansion to work
+                result = client.query("A = point{x:1, y:2}.put([x=3,z=0]).")
+                assert [{'A': {'x': 3, 'y': 2, 'z': 0}}] == result
+
+                # This requires that goal expansion is also being done when it is in the body of a clause that gets asserted
+                result = client.query("retractall(test_goal_expansion), assert(test_goal_expansion(X, Y) :- Y = point{x:1, y:2}.put([x=X, z=0])).")
+                result = client.query("test_goal_expansion(2, Out)")
+                assert [{'Out': {'x': 2, 'y': 2, 'z': 0}}] == result
+
     def test_sync_query(self):
         if self.essentialOnly:
             print("skipped", flush=True, end=" ")
@@ -437,10 +457,15 @@ class TestPrologMQI(ParametrizedTestCase):
                 # Async queries with all results
                 ##########
 
+                # Most basic async query that fails with all results and no free variables
+                client.query_async("false", find_all=True)
+                result = client.query_async_result()
+                assert False is result
+
                 # Most basic async query with all results and no free variables
                 client.query_async("atom(a)", find_all=True)
                 result = client.query_async_result()
-                assert True == result
+                assert True is result
 
                 # Use characters that are encoded in UTF8 in: a one byte (1) two bytes (©) and three bytes (≠)
                 # To test message format and make sure it handles non-ascii characters
@@ -471,6 +496,12 @@ class TestPrologMQI(ParametrizedTestCase):
                 # Async queries with individual results
                 ##########
 
+                # Most basic async query that fails with all results and no free variables and find_all is False
+                client.query_async("false", find_all=False)
+                result = client.query_async_result()
+                assert False is result
+
+
                 # async query that has a parse error
                 query = "member(X, [first, second, third]"
                 caughtException = False
@@ -996,7 +1027,7 @@ class TestPrologMQI(ParametrizedTestCase):
                 # Rethrow the exception if we are configured to fail on unlikely tests failures
                 raise
             else:
-                stackTrace = ''.join(traceback.format_exception(etype=type(e), value=e, tb=e.__traceback__))
+                stackTrace = ''.join(traceback.format_exception(e, e, e.__traceback__))
                 print(
                     f"WARNING: {e} at {stackTrace}.\n This can happen if the system is heavily loaded and is thus a warning by default. To turn this into a failure set the environment variable 'SWIPL_TEST_FAIL_ON_UNLIKELY=y'."
                 )
@@ -1200,7 +1231,7 @@ class TestPrologMQI(ParametrizedTestCase):
 
         tempDir = gettempdir()
         # Put a space in to make sure escaping is working
-        tempFile = os.path.join(tempDir, "swiplserver output.txt")
+        tempFile = os.path.join(tempDir, str(uuid.uuid1()) + " output.txt")
         try:
             os.remove(tempFile)
         except:
@@ -1228,7 +1259,7 @@ class TestPrologMQI(ParametrizedTestCase):
         # Ensure that using embedded mode still sends password and port to STDOUT before redirecting output or
         # embedded mode will fail
         tempDir = gettempdir()
-        tempFile = os.path.join(tempDir, "swiplserveroutput.txt")
+        tempFile = os.path.join(tempDir, str(uuid.uuid1()) + ".txt")
 
         with PrologMQI(
             self.launchServer,
@@ -1297,6 +1328,52 @@ class TestPrologMQI(ParametrizedTestCase):
                 with PrologThread(server) as prolog_thread:
                     prolog_thread.query("true")
 
+    def test_serialization_exceptions(self):
+        with PrologMQI(
+            self.launchServer,
+            self.serverPort,
+            self.password,
+            self.useUnixDomainSocket,
+            prolog_path=self.prologPath,
+        ) as server:
+
+            with server.create_thread() as client:
+                # Send a json value that is known to throw during json serialization to make sure the code
+                # is robust to serialization failures
+                caughtException = False
+                try:
+                    result = client.query("X=json([X - 1]).")
+                except PrologError as error:
+                    assert error.is_prolog_exception("domain_error")
+                    caughtException = True
+                assert caughtException
+
+                caughtException = False
+                try:
+                    result = client.query("open_null_stream(X).")
+                except PrologError as error:
+                    assert error.is_prolog_exception("type_error")
+                    caughtException = True
+                assert caughtException
+
+    def test_variable_attributes(self):
+        with PrologMQI(
+            self.launchServer,
+            self.serverPort,
+            self.password,
+            self.useUnixDomainSocket,
+            prolog_path=self.prologPath
+        ) as server:
+
+            with server.create_thread() as client:
+                result = client.query("member(X, [A, B, C]), put_attr(X, my_module, x).")
+                [{'$residuals': [{'args': ['A', 'my_module', 'x'], 'functor': 'put_attr'}], 'X': 'A', 'A': 'A',
+                  'B': '_', 'C': '_'},
+                 {'$residuals': [{'args': ['B', 'my_module', 'x'], 'functor': 'put_attr'}], 'X': 'B', 'A': '_',
+                  'B': 'B', 'C': '_'},
+                 {'$residuals': [{'args': ['C', 'my_module', 'x'], 'functor': 'put_attr'}], 'X': 'C', 'A': '_',
+                  'B': '_', 'C': 'C'}] == result
+
 
 def run_tcpip_performance_tests(suite):
     suite.addTest(TestPrologMQI("skip_test_protocol_overhead"))
@@ -1327,12 +1404,13 @@ def load_tests(loader, standard_tests, pattern):
     # run_unix_domain_sockets_performance_tests(suite)
 
     # Tests a specific test
-    # suite.addTest(TestPrologMQI('test_sync_query'))
+    # suite.addTest(TestPrologMQI('test_async_query'))
     # return suite
 
     # Tests a specific test with parameters set
-    # suite.addTest(ParametrizedTestCase.parametrize(TestPrologMQI, test_item_name="test_sync_query", launchServer=False,
+    # suite.addTest(ParametrizedTestCase.parametrize(TestPrologMQI, test_item_name="test_variable_attributes", launchServer=False,
     #                                                serverPort=4242, password="debugnow"))
+    # return suite
 
     # Tests a specific test 100 times
     # for index in range(0, 100):
