@@ -1589,6 +1589,12 @@ url_protocol(https).
 url_protocol(ftp).
 url_protocol(mailto).
 
+peek_end_url(space) -->
+    peek(Punct, End),
+    { punct_token(Punct),
+      space_token(End)
+    },
+    !.
 peek_end_url(space) -->
     peek(End),
     { space_token(End) },
@@ -1598,6 +1604,12 @@ peek_end_url(Token) -->
     peek(Token),
     !.
 
+punct_token('.').
+punct_token('!').
+punct_token('?').
+punct_token(',').
+punct_token(';').
+
 space_token(' ') :- !.
 space_token('\r') :- !.
 space_token('\n') :- !.
@@ -2193,6 +2205,9 @@ nl -->
 peek(H, L, L) :-
     L = [H|_].
 
+peek(H1, H2, L, L) :-
+    L = [H1, H2|_].
+
 %!  tokens(-Tokens:list)// is nondet.
 %!  tokens(+Max, -Tokens:list)// is nondet.
 %
