@@ -5,6 +5,7 @@ user:file_search_path(library, '.').
 user:file_search_path(library, '../plunit').
 
 :- use_module(library(sgml)).
+:- use_module(library(apply)).
 :- use_module(library(debug)).
 :- use_module(library(plunit)).
 
@@ -62,8 +63,8 @@ test(float, F == 110.0) :-
 test(time_string, S == "2016-01-30") :-
     xsd_time_string(date(2016,01,30), Y, S),
     assertion(Y == xsd:date).
-test(time_string, S == "-0399-01-30") :-
-    xsd_time_string(date(-400,01,30), Y, S),
+test(time_string, S == "-0400-01-30") :-
+    xsd_time_string(date(-0400,01,30), Y, S),
     assertion(Y == xsd:date).
 test(time_string, S == "2016-01-30T20:29:15") :-
     xsd_time_string(date_time(2016,01,30,20,29,15), Y, S),
@@ -90,17 +91,17 @@ test(time_string, S == "20:29:15.75") :-
     xsd_time_string(time(20,29,15.75), Y, S),
     assertion(Y == xsd:time).
 test(time_string, S == "2016-01") :-
-    xsd_time_string(year_month(2016,1), Y, S),
+    xsd_time_string(year_month(2016,01), Y, S),
     assertion(Y == xsd:gYearMonth).
 test(time_string, S == "01-30") :-
-    xsd_time_string(month_day(1,30), Y, S),
+    xsd_time_string(month_day(01,30), Y, S),
     assertion(Y == xsd:gMonthDay).
 test(time_string, S == "2016") :-
     xsd_time_string(2016, xsd:gYear, S).
-test(time_string, S == "-0399") :-
-    xsd_time_string(-400, xsd:gYear, S).
+test(time_string, S == "-0400") :-
+    xsd_time_string(-0400, xsd:gYear, S).
 test(time_string, S == "01") :-
-    xsd_time_string(1, xsd:gMonth, S).
+    xsd_time_string(01, xsd:gMonth, S).
 test(time_string, S == "30") :-
     xsd_time_string(30, xsd:gDay, S).
 
@@ -109,7 +110,7 @@ test(time_string, S == "30") :-
 test(string_time, T = date(2016,01,30)) :-
     xsd_time_string(T, Y, "2016-01-30"),
     assertion(Y == xsd:date).
-test(string_time, T = date(-2017,01,30)) :-
+test(string_time, T = date(-2016,01,30)) :-
     xsd_time_string(T, Y, "-2016-01-30"),
     assertion(Y == xsd:date).
 test(string_time, T = date_time(2016,01,30,20,29,15)) :-
@@ -130,10 +131,10 @@ test(string_time, T = date_time(2016,01,30,20,29,15.10,0)) :-
 test(string_time, T = date_time(2016,01,30,20,29,15,3600)) :-
     xsd_time_string(T, Y, "2016-01-30T20:29:15+01:00"),
     assertion(Y == xsd:dateTime).
-test(string_time, T = date_time(2016,01,30,20,29,05.6,3600)) :-
+test(string_time, T = date_time(2016,01,30,20,29,5.6,3600)) :-
     xsd_time_string(T, Y, "2016-01-30T20:29:05.6+01:00"),
     assertion(Y == xsd:dateTime).
-test(string_time, T = date_time(2016,01,30,20,29,05.6,50400)) :-
+test(string_time, T = date_time(2016,01,30,20,29,5.6,50400)) :-
     xsd_time_string(T, Y, "2016-01-30T20:29:05.6+14:00"),
     assertion(Y == xsd:dateTime).
 test(string_time, T = time(20,29,15)) :-
@@ -142,19 +143,19 @@ test(string_time, T = time(20,29,15)) :-
 test(string_time, T = time(20,29,15.75)) :-
     xsd_time_string(T, Y, "20:29:15.75"),
     assertion(Y == xsd:time).
-test(string_time, T = year_month(2016,1)) :-
+test(string_time, T = year_month(2016,01)) :-
     xsd_time_string(T, Y, "2016-01"),
     assertion(Y == xsd:gYearMonth).
-test(string_time, T = month_day(1,30)) :-
+test(string_time, T = month_day(01,30)) :-
     xsd_time_string(T, Y, "01-30"),
     assertion(Y == xsd:gMonthDay).
 test(string_time, T = 2016) :-
     xsd_time_string(T, Y, "2016"),
     assertion(Y == xsd:gYear).
-test(string_time, T = -2017) :-
+test(string_time, T = -2016) :-
     xsd_time_string(T, Y, "-2016"),
     assertion(Y == xsd:gYear).
-test(string_time, T = 1) :-
+test(string_time, T = 01) :-
     xsd_time_string(T, xsd:gMonth, "01").
 test(string_time, T = 30) :-
     xsd_time_string(T, xsd:gDay, "30").
@@ -184,15 +185,13 @@ test(term_error, error(domain_error(hour,25))) :-
     xsd_time_string(date_time(2016,01,30,25,29,15), _, _).
 test(term_error, error(domain_error(hour,25))) :-
     xsd_time_string(date_time(2016,01,30,25,29,15,0), _, _).
-test(term_error, error(domain_error(tz_offset,100000))) :-
-    xsd_time_string(date_time(2016,01,30,20,29,15,100000), _, _).
+test(term_error, error(domain_error(tz_offset,100 000))) :-
+    xsd_time_string(date_time(2016,01,30,20,29,15,100 000), _, _).
 test(term_error, error(domain_error(month,20))) :-
     xsd_time_string(20, xsd:gMonth, _).
 test(term_error, error(domain_error(day,35))) :-
     xsd_time_string(35, xsd:gDay, _).
-test(term_error, error(domain_error(day,-1))) :-
-    xsd_time_string(-1, xsd:gDay, _).
-test(term_error, error(domain_error(year,0))) :-
-    xsd_time_string(0, xsd:gYear, _).
+test(term_error, error(domain_error(day,-01))) :-
+    xsd_time_string(-01, xsd:gDay, _).
 
 :- end_tests(xsd_time_string).
