@@ -368,13 +368,8 @@ pl_crypto_hash_context_new(term_t tcontext, term_t options)
   if ( !context )
     return FALSE;
 
+  memset(context, 0, sizeof(*context));
   context->magic    = HASH_CONTEXT_MAGIC;
-  context->ctx      = NULL;
-  context->mac_ctx  = NULL;
-  context->mac_key  = NULL;
-
-  context->parent_stream = NULL;
-  context->hash_stream   = NULL;
 
   if ( !hash_options(options, context) )
     return FALSE;
