@@ -47,7 +47,8 @@
 
 #include <QTime>
 #include <QDebug>
-#include <QRegExp>
+//#include <QRegExp>
+#include <QRegularExpression>
 #include <QAction>
 #include <QToolTip>
 #include <QKeyEvent>
@@ -318,9 +319,9 @@ void ConsoleEdit::keyPressEvent(QKeyEvent *event) {
         if ((accept = editable) && ctrl)
         #endif
         {   qDebug() << "^D" << thids;
-	    c.movePosition(c.End);
+            c.movePosition(c.End);
             setTextCursor(c);
-	    ret = true;
+            ret = true;
             status = eof;
         }
         break;
@@ -380,8 +381,8 @@ void ConsoleEdit::keyPressEvent(QKeyEvent *event) {
         else
             emit user_input(cmd);
 
-	if ( status != eof || !cmd.isEmpty() )
-	    status = running;
+        if ( status != eof || !cmd.isEmpty() )
+            status = running;
     }
 }
 
@@ -458,7 +459,7 @@ void ConsoleEdit::compinit2(QTextCursor c) {
             foreach (auto d, p.value()) {
                 QStringList la;
                 for (int n = 0; n < d.first; ++n)
-                    la.append(QString('A' + n));
+                    la.append(QString(QChar('A' + n)));
                 if (!la.isEmpty())
                     lpreds.append(QString("%1(%2) | %3").arg(a).arg(la.join(", ")).arg(d.second));
                 else
@@ -691,13 +692,6 @@ QString ConsoleEdit::titleLabel() {
 /** attempt to gracefully stop XPCE thread
  */
 bool ConsoleEdit::can_close() {
-    if (eng && is_running()) {
-        QMessageBox b(this);
-        b.setText(tr("[%1] is running a query.\nQuit anyway ?").arg(titleLabel())); //thread_id()
-        b.setStandardButtons(b.Yes|b.No);
-        b.setIcon(b.Question);
-        return b.exec() == b.Yes;
-    }
     return true;
 }
 
@@ -745,29 +739,28 @@ void ConsoleEdit::clickable_message_line(QTextCursor c, bool highlight) {
     c.movePosition(c.EndOfLine, c.KeepAnchor);
 
     QString line = c.selectedText();
-    static QRegExp msg("(ERROR|Warning):[ \t]*(([a-zA-Z]:)?[^:]+):([0-9]+)(:([0-9]+))?.*",
-		       Qt::CaseSensitive, QRegExp::RegExp2);
-    if ( msg.exactMatch(line) ) {
-        QStringList parts = msg.capturedTexts();
-     // qDebug() << "file" << parts[2] << "line" << parts[4] << "char" << parts[6];
-
-	if ( highlight ) {
-	    if (cposition != cposition_) {
-	        cposition = cposition_;
-		fposition = fposition_;
-		QTextCharFormat f = fposition_;
-		f.setFontUnderline(true);
-		c.setCharFormat(f);
-	    }
-	    return;
-	} else {
-	    auto cmd = QString("edit('"+parts[2]+"':"+parts[4]);
-	    if ( !parts[6].isEmpty() )
-	        cmd += ":"+parts[6];
-	    cmd += ")";
-	    qDebug() << cmd;
-	    query_run(cmd);
-	}
+    static QRegularExpression msg("(ERROR|Warning):[ \t]*(([a-zA-Z]:)?[^:]+):([0-9]+)(:([0-9]+))?.*",
+                                  QRegularExpression::CaseInsensitiveOption);
+    auto parts = msg.match(line);
+    if (parts.hasMatch()) {
+        if ( highlight ) {
+            if (cposition != cposition_) {
+                cposition = cposition_;
+                fposition = fposition_;
+                QTextCharFormat f = fposition_;
+                f.setFontUnderline(true);
+                c.setCharFormat(f);
+            }
+            return;
+        } else {
+            auto cmd = QString("edit('"+parts.captured(2)+"':"+parts.captured(4));
+            if ( !parts.captured(6).isEmpty() ) {
+                cmd += ":"+parts.captured(6);
+            }
+            cmd += ")";
+            qDebug() << cmd;
+            query_run(cmd);
+        }
     }
 
     if (fposition != QTextCharFormat()) {
@@ -805,7 +798,7 @@ void ConsoleEdit::onConsoleMenuAction() {
         onConsoleMenuActionMap(action);
     }
 }
-void ConsoleEdit::onConsoleMenuActionMap(const QString& action) {
+void ConsoleEdit::onConsoleMenuActionMap(QString action) {
     if (auto w = find_parent<pqMainWindow>(this)) {
         if (ConsoleEdit *target = w->consoleActive()) {
             qDebug() << action << target->status << QTime::currentTime();
@@ -882,7 +875,6 @@ void ConsoleEdit::add_history_line(QString line)
 void ConsoleEdit::eng_completed() {
     if (eng) {
         eng = 0;
-        // qApp->quit();
         QApplication::postEvent(qApp, new QCloseEvent);
     }
     else if (io) {
@@ -910,6 +902,7 @@ void ConsoleEdit::query_run(QString module, QString call) {
 }
 
 ConsoleEdit::exec_sync::exec_sync(int timeout_ms) {
+    Q_UNUSED(timeout_ms);
     stop_ = QThread::currentThread();
     go_ = 0;
 }
