@@ -224,7 +224,7 @@ static int WriteGIF(fp, pic, mask, ptype, w, h, rmap, gmap, bmap, numcols,
 	  *prow = Transparent;
 
 	m>>=1;
-	if ( !m )
+	if ( !m && x+1<w )
 	{ m = 0x80;
 	  b = *row++;
 	}
@@ -689,36 +689,8 @@ static void cl_block ()             /* table clear for block compress */
 
 
 /********************************/
-static void cl_hash(hsize)          /* reset code table */
-register count_int hsize;
-{
-  register count_int *htab_p = htab+hsize;
-  register long i;
-  register long m1 = -1;
-
-  i = hsize - 16;
-  do {                            /* might use Sys V memset(3) here */
-    *(htab_p-16) = m1;
-    *(htab_p-15) = m1;
-    *(htab_p-14) = m1;
-    *(htab_p-13) = m1;
-    *(htab_p-12) = m1;
-    *(htab_p-11) = m1;
-    *(htab_p-10) = m1;
-    *(htab_p-9) = m1;
-    *(htab_p-8) = m1;
-    *(htab_p-7) = m1;
-    *(htab_p-6) = m1;
-    *(htab_p-5) = m1;
-    *(htab_p-4) = m1;
-    *(htab_p-3) = m1;
-    *(htab_p-2) = m1;
-    *(htab_p-1) = m1;
-    htab_p -= 16;
-  } while ((i -= 16) >= 0);
-
-  for ( i += 16; i > 0; i-- )
-    *--htab_p = m1;
+static void cl_hash(count_int hsize)          /* reset code table */
+{ memset(htab, -1, sizeof(htab));
 }
 
 
@@ -1681,7 +1653,11 @@ typedef hist2d hist3d[HIST_C0_ELEMS];
 
 typedef short FSERROR;		/* 16 bits should be enough */
 typedef int LOCFSERROR;		/* use 'int' for calculation temps */
+#ifndef _BASETSD_H_		/* Microsoft defines it in basetsd.h */
+#ifndef _BASETSD_H		/* MinGW is slightly different */
 typedef long INT32;
+#endif
+#endif
 
 typedef FSERROR *FSERRPTR;	/* pointer to error array */
 
