@@ -40,10 +40,6 @@
 #define MAXCMDLINE 10240
 #endif
 
-#ifndef MAXPATHLEN
-#define MAXPATHLEN 1024
-#endif
-
 static int
 getEnvironmentSizeProcess(Process p)
 { int size = 0;
@@ -118,7 +114,7 @@ openProcess(Process p, CharArray cmd, int argc, CharArray *argv)
     PROCESS_INFORMATION *processinfo;
     SECURITY_ATTRIBUTES sa;
     char cmdline[MAXCMDLINE];
-    char dirbuf[MAXPATHLEN];
+    char dirbuf[PATH_MAX];
     char *cwd = NULL;
     Any a;
     char *env = createWin32EnvriomentBlock(p);
@@ -140,11 +136,11 @@ openProcess(Process p, CharArray cmd, int argc, CharArray *argv)
     }
 
     if ( notDefault(p->directory) )
-    { _xos_limited_os_filename(strName(p->directory->path), dirbuf);
+    { _xos_limited_os_filename(strName(p->directory->path), dirbuf, PATH_MAX);
       cwd = dirbuf;
     }
 
-    _xos_limited_os_filename(strName(p->name), cmdline);
+    _xos_limited_os_filename(strName(p->name), cmdline, MAXCMDLINE);
     for_vector(p->arguments, a,
 	       { strcat(cmdline, " ");
 		 strcat(cmdline, toCharp(a));
