@@ -63,13 +63,6 @@
 #include <sys/stat.h>
 #include <errno.h>
 
-#ifdef HAVE_SYS_PARAM_H
-#include <sys/param.h>
-#endif
-#ifndef MAXPATHLEN
-#define MAXPATHLEN 1024
-#endif
-
 #ifdef HAVE_PWD_H
 #include <pwd.h>
 #endif
@@ -84,7 +77,7 @@ static Name	getWorkingDirectoryPce(Pce pce);
 
 static status
 initialiseDirectory(Directory d, Name name)
-{ char path[MAXPATHLEN];
+{ char path[PATH_MAX];
   Name expanded;
   const char *ufn;
 
@@ -126,13 +119,18 @@ loadDirectory(Directory d, IOSTREAM *fd, ClassDef def)
 
 static status
 existsDirectory(Directory d)
-{ STAT_TYPE buf;
+{
+#ifdef O_XOS
+  return _xos_exists(nameToFN(d->path), _XOS_DIR);
+#else
+  STAT_TYPE buf;
 
   if ( STAT_FUNC(nameToFN(d->path), &buf) == -1 ||
        (buf.st_mode & S_IFMT) != S_IFDIR )
     fail;
 
   succeed;
+#endif
 }
 
 
@@ -217,12 +215,20 @@ scanDirectory(Directory d, Chain files, Chain dirs, Regex pattern, BoolObj all)
 
     for (dp=readdir(dirp); dp!=NULL; dp=readdir(dirp))
     { char *name = dp->d_name;
+#ifndef O_XOS
       STAT_TYPE buf;
 
       if ( STAT_FUNC(name, &buf) != 0 )
 	continue;
+#endif
 
-      if ( (notNil(files) && (buf.st_mode & S_IFMT) == S_IFREG) )
+      if ( notNil(files) &&
+#ifdef O_XOS
+	   _xos_exists(name, _XOS_FILE)
+#else
+	   (buf.st_mode & S_IFMT) == S_IFREG
+#endif
+	 )
       { if ( notDefault(pattern) )
 	{ CharArray ca = CtoScratchCharArray(name);
 					/* TBD: UNICODE */
@@ -237,7 +243,13 @@ scanDirectory(Directory d, Chain files, Chain dirs, Regex pattern, BoolObj all)
 	  continue;
 
 	appendChain(files, FNToName(name));
-      } else if ( (notNil(dirs) && (buf.st_mode & S_IFMT) == S_IFDIR) )
+      } else if ( notNil(dirs) &&
+#ifdef O_XOS
+		  _xos_exists(name, _XOS_DIR)
+#else
+		  (buf.st_mode & S_IFMT) == S_IFDIR
+#endif
+		)
       { if ( all != ON && name[0] == '.' )
 	  continue;
 
@@ -302,7 +314,7 @@ getFilesDirectory(Directory d, Regex pattern, BoolObj all)
 
 static Directory
 getParentDirectory(Directory d)
-{ char parent[MAXPATHLEN];
+{ char parent[PATH_MAX];
   const char *here = nameToFN(d->path);
 
   if ( IsDirSep(here[0]) && here[1] == EOS ) /* the root */
@@ -325,14 +337,14 @@ static Chain
 getRootsDirectory(Directory dir)
 { Chain ch = answerObject(ClassChain, EAV);
 #ifdef __WINDOWS__
-  char buf[MAXPATHLEN];
+  char buf[PATH_MAX];
   extern int get_logical_drive_strings(int, char *);
 
   if ( get_logical_drive_strings(sizeof(buf)-1, buf) )
   { char *s = buf;
 
     while(*s)
-    { char buf2[MAXPATHLEN];
+    { char buf2[PATH_MAX];
       char *cnfn;
 
       if ( (cnfn=_xos_canonical_filename(s, buf2, sizeof(buf2), 0)) )
@@ -356,21 +368,30 @@ getBaseNameDirectory(Directory d)
 
 static Date
 getTimeDirectory(Directory d, Name which)
-{ STAT_TYPE buf;
-  Name name = d->path;
+{ Name name = d->path;
+
+#if O_XOS
+  double t;
+  int id = (which == NAME_modified ? XOS_TIME_MODIFIED : XOS_TIME_ACCESS);
+
+  if ( _xos_get_file_time(nameToFN(name), id, &t) == 0 )
+    answer(CtoDate((time_t)t));
+#else
+  STAT_TYPE buf;
 
   if ( isDefault(which) )
     which = NAME_modified;
 
-  if ( STAT_FUNC(nameToFN(name), &buf) < 0 )
-  { errorPce(d, NAME_cannotStat, getOsErrorPce(PCE));
-    fail;
+  if ( STAT_FUNC(nameToFN(name), &buf) == 0 )
+  { if ( which == NAME_modified )
+      answer(CtoDate(buf.st_mtime));
+    else
+      answer(CtoDate(buf.st_atime));
   }
+#endif
 
-  if ( which == NAME_modified )
-    answer(CtoDate(buf.st_mtime));
-  else
-    answer(CtoDate(buf.st_atime));
+  errorPce(d, NAME_cannotStat, getOsErrorPce(PCE));
+  fail;
 }
 
 
@@ -432,17 +453,29 @@ accessDirectory(Directory d, Name mode)
 
 static status
 changedDirectory(Directory d)
-{ STAT_TYPE buf;
+{ time_t t;
+
+#ifdef O_XOS
+  double time;
+
+  if ( _xos_get_file_time(nameToFN(d->path), XOS_TIME_MODIFIED, &time) == 0 )
+    t = (time_t)time;
+  else
+    succeed;
+#else
+  STAT_TYPE buf;
 
   if ( STAT_FUNC(nameToFN(d->path), &buf) < 0 )
     succeed;			/* we signal non-extistence as changed */
+  t = buf.st_mtime;
+#endif
 
   if ( d->modified == MODIFIED_NOT_SET )
-  { d->modified = buf.st_mtime;
+  { d->modified = t;
     fail;
   }
-  if ( buf.st_mtime > d->modified )
-  { d->modified = buf.st_mtime;
+  if ( t > d->modified )
+  { d->modified = t;
     succeed;
   }
 
@@ -672,7 +705,7 @@ char *
 baseName(const char *f)
 { if ( f )
   { const char *base;
-    static char buf[MAXPATHLEN];
+    static char buf[PATH_MAX];
     int len;
 
     for(base = f; *f; f++)
@@ -693,7 +726,7 @@ baseName(const char *f)
 }
 
 
-static char   CWDdir[MAXPATHLEN];
+static char   CWDdir[PATH_MAX];
 
 static Name
 getWorkingDirectoryPce(Pce pce)
