@@ -3,7 +3,8 @@
     Author:        Jan Wielemaker and Anjo Anjewierden
     E-mail:        jan@swi.psy.uva.nl
     WWW:           http://www.swi.psy.uva.nl/projects/xpce/
-    Copyright (c)  1985-2002, University of Amsterdam
+    Copyright (c)  1985-2022, University of Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -43,13 +44,6 @@
 #include <errno.h>
 #include <string.h>
 
-#ifdef HAVE_SYS_PARAM_H
-#include <sys/param.h>
-#endif
-#ifndef MAXPATHLEN
-#define MAXPATHLEN 1024
-#endif
-
 #if HAVE_SYS_ACCESS_H			/* AIX 3.2.5 */
 #include <sys/access.h>
 #endif
@@ -153,17 +147,17 @@ initialiseFile(FileObj f, Name name, Name encoding)
 
 Name
 expandFileName(Name in)
-{ wchar_t expanded[MAXPATHLEN];
+{ wchar_t expanded[PATH_MAX];
   int len;
 
   if ( (len=expandFileNameW(charArrayToWC((CharArray)in, NULL),
-			    expanded, MAXPATHLEN)) > 0 )
+			    expanded, PATH_MAX)) > 0 )
   {
 #if O_XOS
-     wchar_t lng[MAXPATHLEN];
-     char buf[MAXPATHLEN];
+     wchar_t lng[PATH_MAX];
+     char buf[PATH_MAX];
 
-     if ( _xos_long_file_nameW(expanded, lng, MAXPATHLEN) &&
+     if ( _xos_long_file_nameW(expanded, lng, PATH_MAX) &&
 	  _xos_canonical_filenameW(lng, buf, sizeof(buf), 0) )
      { return UTF8ToName(buf);
      } else
@@ -273,9 +267,13 @@ closeFile(FileObj f)
 
 status
 existsFile(FileObj f, BoolObj mustbefile)
-{ STAT_TYPE buf;
-  const char *fn = charArrayToFN((CharArray)f->name);
+{ const char *fn = charArrayToFN((CharArray)f->name);
 
+#if O_XOS
+  int flag = (mustbefile == OFF ? 0 : _XOS_FILE);
+  return _xos_exists(fn, flag);
+#else
+  STAT_TYPE buf;
 #ifdef HAVE_ACCESS
   if ( mustbefile == OFF )
   { if ( access(fn, F_OK) == 0 )
@@ -288,6 +286,7 @@ existsFile(FileObj f, BoolObj mustbefile)
   if ( mustbefile != OFF && (buf.st_mode & S_IFMT) != S_IFREG )
     fail;
   succeed;
+#endif
 }
 
 
@@ -331,7 +330,7 @@ sameFile(FileObj f1, FileObj f2)
 
 static status
 absolutePathFile(FileObj f)
-{ char path[MAXPATHLEN];
+{ char path[PATH_MAX];
 
   if ( absolutePath(charArrayToUTF8((CharArray)f->name), path, sizeof(path)) > 0 )
   { assign(f, path, UTF8ToName(path));
@@ -344,7 +343,7 @@ absolutePathFile(FileObj f)
 
 Name
 getAbsolutePathFile(FileObj f)
-{ char path[MAXPATHLEN];
+{ char path[PATH_MAX];
 
   if ( notDefault(f->path) )
     answer(f->path);
@@ -444,8 +443,9 @@ out:
 
 static status
 backup_name(const char *old, const char *ext, char *bak, size_t len)
-{ if ( strlen(old) + strlen(ext) + 1 <= len )
-  { sprintf(bak, "%s%s", old, ext);
+{ if ( strlen(old) + strlen(ext) + 1 < len )
+  { strcpy(bak, old);
+    strcat(bak, ext);
     succeed;
   } else
   { errno = ENAMETOOLONG;
@@ -456,7 +456,7 @@ backup_name(const char *old, const char *ext, char *bak, size_t len)
 
 static Name
 getBackupFileNameFile(FileObj f, Name ext)
-{ char bak[MAXPATHLEN*2];
+{ char bak[PATH_MAX*2];
 
   if ( backup_name(nameToUTF8(f->name),
 		   isDefault(ext) ? "~" : nameToUTF8(ext),
@@ -544,19 +544,33 @@ getFilterFile(FileObj f)
   closeFile(f);
 
   for_cell(cell, FileFilters->attributes)
-  { char path[MAXPATHLEN];
+  { char path[PATH_MAX];
     Attribute a = cell->value;
     Name extension = a->name;
-    STAT_TYPE buf;
 
     if ( !isName(extension) )
     { errorPce(extension, NAME_unexpectedType, TypeName);
       fail;
     }
 
-    sprintf(path, "%s%s", strName(f->name), strName(extension));
+    char *fname = strName(f->name);
+    char *ext   = strName(extension);
+
+    if ( strlen(fname)+strlen(ext)+1 >= sizeof(path) )
+    { errorPce(f, NAME_representation, NAME_nameTooLong);
+      fail;
+    }
+
+    strcpy(path, fname);
+    strcat(path, ext);
+
+#if O_XOS
+    if ( _xos_exists(path, _XOS_FILE) )
+#else
+    STAT_TYPE buf;
     if ( STAT_FUNC(path, &buf) == 0 &&
 	 (buf.st_mode & S_IFMT) == S_IFREG )
+#endif
     { if ( !isName(a->value) )
       { errorPce(a->value, NAME_unexpectedType, TypeName);
 	fail;
@@ -646,7 +660,7 @@ openFile(FileObj f, Name mode, Name filter, CharArray extension)
   if ( isNil(filter) )
   { DEBUG(NAME_file, Cprintf("Opening %s (%s) using mode %s\n",
 			     pp(f->name), pp(f), fdmode));
-    f->fd = Sopen_file(charArrayToFN(path), fdmode);
+    f->fd = Sopen_iri_or_file(charArrayToFN(path), fdmode);
   } else
 #ifndef HAVE_POPEN
   { return errorPce(f, NAME_noPopen);
@@ -862,60 +876,74 @@ flushFile(FileObj f)
 }
 
 
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-We can use the faster fstat() here, but confirmed by various messages on
-the web, MS-Windows implementation  of   _fstat()  is  broken, returning
-EBADF for perfectly valid filedescriptors.  Shouldn't make a difference,
-only slow ...
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-
+#if !O_XOS
 static int
 statFile(FileObj f, STAT_TYPE *buf)
-{
-#ifndef __WINDOWS__
-  int fno;
+{ int fno;
 
   if ( f->fd != NULL && (fno = Sfileno(f->fd)) >= 0)
   { return FSTAT_FUNC(fno, buf);
   } else
-#endif
   { Name name = getOsNameFile(f);
 
     return STAT_FUNC(nameToFN(name), buf);
   }
 }
-
+#endif
 
 
 static Int
 getSizeFile(FileObj f)
-{ STAT_TYPE buf;
+{ if ( f->fd )
+  { int64_t size;
 
-  if ( statFile(f, &buf) == -1 )
-  { errorPce(f, NAME_cannotStat, getOsErrorPce(PCE));
-    fail;
+    if ( (size=Ssize(f->fd)) != -1 )
+      answer(toInt(size));
+  } else
+#if O_XOS
+  { uint64_t size;
+    Name name = getOsNameFile(f);
+
+    if ( _xos_file_size(nameToFN(name), &size) == 0 )
+      answer(toInt(size));
   }
+#else
+  { STAT_TYPE buf;
 
-  answer(toInt(buf.st_size));
+    if ( statFile(f, &buf) == 0 )
+      answer(toInt(buf.st_size));
+  }
+#endif
+
+  errorPce(f, NAME_cannotStat, getOsErrorPce(PCE));
+  fail;
 }
 
 
 static Date
 getTimeFile(FileObj f, Name which)
-{ STAT_TYPE buf;
-
-  if ( isDefault(which) )
+{ if ( isDefault(which) )
     which = NAME_modified;
 
-  if ( statFile(f, &buf) < 0 )
-  { errorPce(f, NAME_cannotStat, getOsErrorPce(PCE));
-    fail;
-  }
+#if O_XOS
+  Name name = getOsNameFile(f);
+  double t;
+  int id = (which == NAME_modified ? XOS_TIME_MODIFIED : XOS_TIME_ACCESS);
 
-  if ( which == NAME_modified )
-    answer(CtoDate(buf.st_mtime));
-  else
-    answer(CtoDate(buf.st_atime));
+  if ( _xos_get_file_time(nameToFN(name), id, &t) == 0 )
+    answer(CtoDate((time_t)t));
+#else
+  STAT_TYPE buf;
+
+  if ( statFile(f, &buf) == 0 )
+  { if ( which == NAME_modified )
+      answer(CtoDate(buf.st_mtime));
+    else
+      answer(CtoDate(buf.st_atime));
+  }
+#endif
+  errorPce(f, NAME_cannotStat, getOsErrorPce(PCE));
+  fail;
 }
 
 
@@ -929,7 +957,7 @@ getBaseNameFile(FileObj f)
 
 static Name
 getDirectoryNameFile(FileObj f)
-{ char dir[MAXPATHLEN];
+{ char dir[PATH_MAX];
 
   dirName(nameToUTF8(getOsNameFile(f)), dir, sizeof(dir));
 
@@ -1080,7 +1108,7 @@ putstdw(unsigned long w, IOSTREAM *fd)
 
   cvrt.l = w;
   rval = (cvrt.c[0] << 24) |
-         (cvrt.c[1] << 16) |
+	 (cvrt.c[1] << 16) |
 	 (cvrt.c[2] << 8) |
 	  cvrt.c[3];
   Sputw(rval, fd);
@@ -1206,7 +1234,7 @@ waccess(const wchar_t *name, int m)
 status
 findFile(FileObj f, CharArray path, Name mode)
 { wchar_t *base;
-  wchar_t basebuf[MAXPATHLEN];
+  wchar_t basebuf[PATH_MAX];
   const wchar_t *pathstr;
   size_t bl;
   int m;
@@ -1228,7 +1256,7 @@ findFile(FileObj f, CharArray path, Name mode)
   if ( notDefault(f->path) && access(nameToFN(f->path), m) == 0 )
     succeed;
 
-  if ( bl+1 > MAXPATHLEN )
+  if ( bl+1 > PATH_MAX )
     return errorPce(f, NAME_representation, NAME_nameTooLong);
   wcscpy(basebuf, base);
   base = basebuf;
@@ -1239,8 +1267,8 @@ findFile(FileObj f, CharArray path, Name mode)
     pathstr = charArrayToWC(path, NULL);
 
   while( pathstr && *pathstr )
-  { wchar_t name[MAXPATHLEN];
-    wchar_t bin[MAXPATHLEN];
+  { wchar_t name[PATH_MAX];
+    wchar_t bin[PATH_MAX];
     const wchar_t *end = pathstr;
     size_t l;
 
@@ -1259,7 +1287,7 @@ findFile(FileObj f, CharArray path, Name mode)
     }
 
     if ( wcschr(name, L'$') || name[0] == L'~' )
-    { if ( (l=expandFileNameW(name, bin, MAXPATHLEN)) > 0 )
+    { if ( (l=expandFileNameW(name, bin, PATH_MAX)) > 0 )
 	wcsncpy(name, bin, l);
       else
 	continue;
@@ -1287,16 +1315,16 @@ findFile(FileObj f, CharArray path, Name mode)
 /* Type declarations */
 
 static char *T_seek[] =
-        { "byte=int", "from=[{start,here,end}]" };
+	{ "byte=int", "from=[{start,here,end}]" };
 static char *T_format[] =
-        { "format=char_array", "argument=any ..." };
+	{ "format=char_array", "argument=any ..." };
 static char *T_open[] =
-        { "mode={read,write,append}", "filter=[name]",
+	{ "mode={read,write,append}", "filter=[name]",
 	  "extension=[char_array]" };
 static char *T_find[] =
-        { "path=[char_array]", "access=[{read,write,append,execute}]" };
+	{ "path=[char_array]", "access=[{read,write,append,execute}]" };
 static char *T_initialise[] =
-        { "path=[name]",
+	{ "path=[name]",
 	  "encoding=[{text,binary,iso_latin_1,utf8,unicode_be,unicode_le}]"
 	};
 
@@ -1408,9 +1436,9 @@ static classvardecl rc_file[] =
 static Name file_termnames[] = { NAME_name };
 
 ClassDecl(file_decls,
-          var_file, send_file, get_file, rc_file,
-          1, file_termnames,
-          "$Rev$");
+	  var_file, send_file, get_file, rc_file,
+	  1, file_termnames,
+	  "$Rev$");
 
 status
 makeClassFile(Class class)
