@@ -13,20 +13,20 @@
 # Configure using cmake -DMACOSX_DEPENDENCIES_FROM=$PREFIX
 
 PREFIX="$HOME/deps"
-export MACOSX_DEPLOYMENT_TARGET=10.12
+export MACOSX_DEPLOYMENT_TARGET=10.14
 
-GMP_VERSION=6.1.2
-SSL_VERSION=1.1.1f
-JPEG_VERSION=9b
-ZLIB_VERSION=1.2.11
-ARCHIVE_VERSION=3.4.0
+GMP_VERSION=6.2.1
+SSL_VERSION=3.0.7
+JPEG_VERSION=9e
+ZLIB_VERSION=1.2.13
+ARCHIVE_VERSION=3.6.1
 UUID_VERSION=1.6.2
 BDB_VERSION=6.1.26
-ODBC_VERSION=2.3.7
-PCRE_VERSION=8.43
-FFI_VERSION=3.3
-YAML_VERSION=0.1.7
-READLINE_VERSION=8.0
+ODBC_VERSION=2.3.11
+PCRE2_VERSION=10.39
+FFI_VERSION=3.4.4
+YAML_VERSION=0.2.5
+READLINE_VERSION=8.2
 
 # installation prefix.  This path should not have spaces in one of the
 # directory names.
@@ -36,7 +36,7 @@ src="$(pwd)"
 # Handy for running autoconf from a directory
 
 export LDFLAGS=-L$PREFIX/lib
-export CFLAGS="-mmacosx-version-min=10.12 -O2"
+export CFLAGS="-mmacosx-version-min=10.14 -O2"
 
 config()
 { if [ -r ./configure ]; then
@@ -78,10 +78,20 @@ download_ssl()
 
 build_ssl()
 { ( cd openssl-$SSL_VERSION
-    ./Configure --prefix=$PREFIX shared threads darwin64-x86_64-cc
+    case "$(uname -a)" in
+	*arm*) target=darwin64-arm64-cc
+	       ;;
+	*x86_64*) target=darwin64-x86_64-cc
+		  ;;
+	*) echo "Unknown Darmin target"
+	   return 1
+	   ;;
+    esac
+    ./Configure --prefix=$PREFIX shared threads $target
     make depend
     make
     make install_sw
+    make install_ssldirs	# installs deps/ssl/openssl.cnf
   )
 }
 
@@ -212,21 +222,20 @@ build_libarchive()
 #################################
 # Download and install libpcre
 
+download_libpcre2()
+{ PCRE2_FILE=pcre2-$PCRE2_VERSION.tar.gz
 
-download_libpcre()
-{ PCRE_FILE=pcre-$PCRE_VERSION.tar.gz
-
-  [ -f $PCRE_FILE ] || \
-    wget https://ftp.pcre.org/pub/pcre/$PCRE_FILE
-  tar xzf $PCRE_FILE
+  [ -f $PCRE2_FILE ] || \
+    wget https://github.com/PhilipHazel/pcre2/releases/download/pcre2-$PCRE2_VERSION/pcre2-$PCRE2_VERSION.tar.gz
+  tar xzf $PCRE2_FILE
 }
 
 
-build_libpcre()
-{ ( cd pcre-$PCRE_VERSION
+build_libpcre2()
+{ ( cd pcre2-$PCRE2_VERSION
     ./configure --prefix=$PREFIX \
 	--disable-static --disable-cpp --enable-utf8 --enable-unicode-properties
-    make pcre.dll
+    make pcre2.dll
     make install
   )
 }
@@ -257,7 +266,7 @@ build_libuuid()
 download_libffi()
 { FFI_FILE=libffi-$FFI_VERSION.tar.gz
   [ -f $FFI_FILE ] || \
-  curl ftp://sourceware.org/pub/libffi/$FFI_FILE > $FFI_FILE
+  wget https://github.com/libffi/libffi/releases/download/v$FFI_VERSION/$FFI_FILE
   tar zxvf $FFI_FILE
 }
 
@@ -294,6 +303,11 @@ build_libyaml()
 }
 
 
+build_emacs()
+{ cp /opt/local/include/emacs-module.h $PREFIX/include
+}
+
+
 ###########################
 # Do the whole lot for all prerequisites
 
@@ -319,7 +333,7 @@ download_prerequisites()
   download_libuuid
   download_libdb
   download_odbc
-  download_libpcre
+  download_libpcre2
   download_libffi
   download_libyaml
 }
@@ -334,7 +348,8 @@ build_prerequisites()
   build_libuuid
   build_libdb
   build_odbc
-  build_libpcre
+  build_libpcre2
   build_libffi
   build_libyaml
+  build_emacs
 }
