@@ -3,9 +3,10 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2020, University of Amsterdam
+    Copyright (C): 1985-2022, University of Amsterdam
 			      VU University Amsterdam
 			      CWI, Amsterdam
+			      SWI-Prolog Solutions b.v.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -47,9 +48,11 @@ test_arith :-
                     minint_promotion,
                     maxint,
                     maxint_promotion,
+		    round,
 		    float_overflow,
 		    float_zero,
 		    float_special,
+		    float_compare,
 		    arith_misc
 		  ]).
 
@@ -384,6 +387,19 @@ test(multiplication) :-
 
 :- end_tests(maxint_promotion).
 
+:- begin_tests(round).
+
+test(half_down, N == 0) :-
+	X is nexttoward(0.5,-10), N is round(X).
+test(nhalf_up, N == 0) :-
+	X is nexttoward(-0.5,10), N is round(X).
+test(maxint, A == 9223372036854775807 ) :-
+	A is round(9223372036854775807.000000).
+test(minint, A == -9223372036854775808 ) :-
+	A is round(-9223372036854775808.000000).
+
+:- end_tests(round).
+
 :- begin_tests(float_overflow).
 
 :- if(current_prolog_flag(bounded,false)).
@@ -454,6 +470,21 @@ test(cmp, fail) :-
 
 :- end_tests(float_special).
 
+:- begin_tests(float_compare).
+
+test(max_nan, X == 1) :-
+    X is max(1, nan).
+test(max_nan, X == 1) :-
+    X is max(nan, 1).
+
+test(min_nan, X == 1) :-
+    X is min(1, nan).
+test(min_nan, X == 1) :-
+    X is min(nan, 1).
+
+:- end_tests(float_compare).
+
+
 :- begin_tests(arith_misc).
 
 test(string) :-
