@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2017, University of Amsterdam
-                              VU University Amsterdam
+    Copyright (c)  2011-2022, University of Amsterdam
+			      VU University Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -38,6 +39,9 @@
 #include "pl-cstack.h"
 #include "../pl-setup.h"
 #include <time.h>
+#ifdef HAVE_SYS_RESOURCE_H
+#include <sys/resource.h>
+#endif
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 The task of the library is to save   the  <N> most recent C stack traces
@@ -340,25 +344,54 @@ save_backtrace(const char *why)
   return bt;
 }
 
+#ifdef __APPLE__
+/* Emits e.g. "prologToplevel (in libswipl.8.5.20.dylib) (pl-pro.c:560)" */
+#define ADDR2LINE_CMD "atos -o \"%s\" %p"
+#else
+/* Emits two lines: "function\nfile:line"  */
+#define ADDR2LINE_CMD "addr2line -fe \"%s\" %p"
+#endif
 
 static int
 addr2line(const char *fname, uintptr_t offset, char *buf, size_t size)
 { char cmd[MAXCMD];
 
-  if ( snprintf(cmd, size, "addr2line -fe \"%s\" %p",
-		fname, (void*)offset) < size )
+  if ( snprintf(cmd, size, ADDR2LINE_CMD, fname, (void*)offset) < size )
   { FILE *fd;
 
     if ( (fd=popen(cmd, "r")) )
     { int c;
       char *ebuf = &buf[size-1];
       char *o = buf;
-      int nl = 0;
+      const char *sep = "() at ";
 
+#ifdef __APPLE__
+      int field = 0;
+      while((c=fgetc(fd)) != EOF && o<ebuf)
+      { if ( field == 0 )
+	{ if ( c == ' ' )
+	  { if ( o+strlen(sep) < ebuf )
+	    { strcpy(o, sep);
+	      o += strlen(sep);
+	    }
+	    field++;
+	  } else
+	  { *o++ = c;		/* copy the function */
+	  }
+	} else if ( field < 3 )
+	{ if ( c == '(' )	/* skip two '(' */
+	    field++;
+	} else
+	{ if ( c == ')' )	/* copy to ')' */
+	    break;
+	  *o++ = c;
+	}
+      }
+#else
+      int nl = 0;
       while((c=fgetc(fd)) != EOF && o<ebuf)
       { if ( c == '\n' )
-	{ const char *sep = "() at ";
-	  nl++;
+	{ nl++;
 
 	  if ( nl == 1 && o+strlen(sep) < ebuf)
 	  { strcpy(o, sep);
@@ -368,6 +401,7 @@ addr2line(const char *fname, uintptr_t offset, char *buf, size_t size)
 	{ *o++ = c;
 	}
       }
+#endif
 
       *o = '\0';
 
@@ -397,12 +431,17 @@ print_trace(btrace *bt, int me)
 	if ( info.dli_fname )
 	{ char buf[512];
 
-	  if ( strstr(info.dli_fname, ".so") &&
+	  if ( ( strstr(info.dli_fname, ".so")
+#if __APPLE__
+		 || strstr(info.dli_fname, ".dylib")
+#endif
+	       ) &&
 	       addr2line(info.dli_fname, offset, buf, sizeof(buf)) )
 	    Sdprintf("  [%d] %s [%p]\n", i, buf, addr);
 	  else if ( info.dli_sname )
 	    Sdprintf("  [%d] %s(%s+%p) [%p]\n",
-		     i, info.dli_fname, info.dli_sname, addr-info.dli_saddr,
+		     i, info.dli_fname, info.dli_sname,
+		     (uintptr_t)addr-(uintptr_t)info.dli_saddr,
 		     addr);
 	  else
 	    Sdprintf("  [%d] %s(+%p) [%p]\n",
@@ -700,23 +739,23 @@ int backtrace(btrace_stack* trace, PEXCEPTION_POINTERS pExceptionInfo)
        { strncpy(trace->frame[depth].name,
 		 dwarf_symbol,
 		 MAX_FUNCTION_NAME_LENGTH);
-         trace->frame[depth].name[MAX_FUNCTION_NAME_LENGTH-1] = '\0';
+	 trace->frame[depth].name[MAX_FUNCTION_NAME_LENGTH-1] = '\0';
        } else
 #endif
        { SYMBOL_INFO* symbol = (SYMBOL_INFO*)&symbolScratch;
 
-         memset(symbol, 0, sizeof(SYMBOL_INFO) + MAX_SYMBOL_LEN);
-         symbol->SizeOfStruct = sizeof(SYMBOL_INFO);
-         symbol->MaxNameLen = MAX_SYMBOL_LEN;
-         hasSymbol = SymFromAddr(hProcess, frame.AddrPC.Offset, &offset, symbol);
-         if (hasSymbol)
-         { strncpy(trace->frame[depth].name,
+	 memset(symbol, 0, sizeof(SYMBOL_INFO) + MAX_SYMBOL_LEN);
+	 symbol->SizeOfStruct = sizeof(SYMBOL_INFO);
+	 symbol->MaxNameLen = MAX_SYMBOL_LEN;
+	 hasSymbol = SymFromAddr(hProcess, frame.AddrPC.Offset, &offset, symbol);
+	 if (hasSymbol)
+	 { strncpy(trace->frame[depth].name,
 		   symbol->Name,
 		   MAX_FUNCTION_NAME_LENGTH);
-           trace->frame[depth].name[MAX_FUNCTION_NAME_LENGTH-1] = '\0';
-         } else
-         { trace->frame[depth].name[0] = '\0';
-         }
+	   trace->frame[depth].name[MAX_FUNCTION_NAME_LENGTH-1] = '\0';
+	 } else
+	 { trace->frame[depth].name[0] = '\0';
+	 }
        }
      }
      depth++;
@@ -762,31 +801,31 @@ print_trace(btrace *bt, int me)
     { if (s->frame[depth].module[0])
       {
 #ifdef HAVE_LIBDWARF
-        IMAGEHLP_MODULE64 moduleInfo;
-        char dwarf_srclinebuf[MAX_PATH];
-        char *dwarf_srcline = dwarf_srclinebuf;
+	IMAGEHLP_MODULE64 moduleInfo;
+	char dwarf_srclinebuf[PATH_MAX];
+	char *dwarf_srcline = dwarf_srclinebuf;
 
-        memset(&moduleInfo,0,sizeof(IMAGEHLP_MODULE64));
-        memset(dwarf_srcline, 0, MAX_PATH);
-        moduleInfo.SizeOfStruct = sizeof(IMAGEHLP_MODULE64);
-        if ( SymGetModuleInfo64(hProcess, s->frame[depth].offset, &moduleInfo) &&
-             dwarf_addr2line(&moduleInfo, s->frame[depth].offset, &dwarf_srcline) )
-        { Sdprintf("  [%d] <%s>:%s() at %s [%p]\n", depth,
-                   s->frame[depth].module,
-                   s->frame[depth].name,
-                   dwarf_srcline,
-                   (void*)s->frame[depth].offset);
-        } else
+	memset(&moduleInfo,0,sizeof(IMAGEHLP_MODULE64));
+	memset(dwarf_srcline, 0, PATH_MAX);
+	moduleInfo.SizeOfStruct = sizeof(IMAGEHLP_MODULE64);
+	if ( SymGetModuleInfo64(hProcess, s->frame[depth].offset, &moduleInfo) &&
+	     dwarf_addr2line(&moduleInfo, s->frame[depth].offset, &dwarf_srcline) )
+	{ Sdprintf("  [%d] <%s>:%s() at %s [%p]\n", depth,
+		   s->frame[depth].module,
+		   s->frame[depth].name,
+		   dwarf_srcline,
+		   (void*)s->frame[depth].offset);
+	} else
 #endif
-        { Sdprintf("  [%d] <%s>:%s() [%p]\n", depth,
-                   s->frame[depth].module,
-                   s->frame[depth].name,
-                   (void*)s->frame[depth].offset);
-        }
+	{ Sdprintf("  [%d] <%s>:%s() [%p]\n", depth,
+		   s->frame[depth].module,
+		   s->frame[depth].name,
+		   (void*)s->frame[depth].offset);
+	}
       } else
       { Sdprintf("  [%d] <unknown module>:%s [%p]\n", depth,
-                 s->frame[depth].name,
-                 (void*)s->frame[depth].offset);
+		 s->frame[depth].name,
+		 (void*)s->frame[depth].offset);
       }
     }
     SymCleanup(hProcess);
@@ -977,7 +1016,7 @@ BeginPredDefs(cbtrace)
 #endif
 EndPredDefs
 
-	         /*******************************
+		 /*******************************
 		 *   FALLBACK IMPLEMENTATION	*
 		 *******************************/
 
@@ -1016,7 +1055,7 @@ initBackTrace(void)
 {
 }
 
-#ifdef HAVE_SIGNAL
+#if O_SIGNALS && defined(HAVE_SIGNAL)
 void
 sigCrashHandler(int sig)
 { int tid;
@@ -1052,6 +1091,75 @@ sigCrashHandler(int sig)
 { fatalError("Something went wrong");
 }
 
-#endif /*HAVE_SIGNAL*/
+#endif /*O_SIGNALS && HAVE_SIGNAL*/
 
 #endif /*BTRACE_DONE*/
+
+
+		 /*******************************
+		 *   STACK LOCATION AND SIZE	*
+		 *******************************/
+
+#if defined(HAVE_GETRLIMIT) && defined(O_PLMT)
+static size_t
+round_pages(size_t n)
+{ size_t psize;
+
+#if defined(HAVE_SYSCONF) && defined(_SC_PAGESIZE)
+  if ( (psize = sysconf(_SC_PAGESIZE)) == (size_t)-1 )
+    psize = 8192;
+#else
+  psize = 8192;
+#endif
+
+  return ROUND(n, psize);
+}
+#endif
+
+size_t
+CStackSize(DECL_LD)
+{
+#ifdef O_PLMT
+  PL_thread_info_t *info = LD->thread.info;
+
+  if ( info->c_stack_size )
+    return info->c_stack_size;
+
+  if ( info->pl_tid != 1 )
+  { DEBUG(1, Sdprintf("Thread-stack: %ld\n", LD->thread.info->c_stack_size));
+
+#ifdef HAVE_PTHREAD_GETATTR_NP
+    pthread_attr_t attr;
+
+    if ( pthread_getattr_np(info->tid, &attr) == 0 )
+    { pthread_attr_getstack(&attr, &info->c_stack_base, &info->c_stack_size);
+      pthread_attr_destroy(&attr);
+    } else
+    { info->c_stack_size = (size_t)-1;
+    }
+#endif
+
+    return info->c_stack_size;
+  }
+
+#ifdef HAVE_GETRLIMIT
+  struct rlimit rlim;
+
+  if ( getrlimit(RLIMIT_STACK, &rlim) == 0 &&
+       rlim.rlim_cur != RLIM_INFINITY && rlim.rlim_cur )
+  { size_t top = round_pages((size_t)&info);
+
+    DEBUG(1, Sdprintf("Stack: %ld\n", rlim.rlim_cur));
+    info->c_stack_size = rlim.rlim_cur;
+
+    info->c_stack_base = (void*)(top - info->c_stack_size);
+  } else
+  { info->c_stack_size = (size_t)-1;
+  }
+#endif
+
+  return info->c_stack_size;
+#else
+  return (size_t)-1;
+#endif
+}
