@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2021, University of Amsterdam
+    Copyright (c)  2011-2022, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
@@ -60,27 +60,6 @@
 #undef LD
 #define LD LOCAL_LD
 
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-General file operations and binding to Prolog
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-
-#ifdef __WINDOWS__
-static void
-set_posix_error(int win_error)
-{ int error = 0;
-
-  switch(win_error)
-  { case ERROR_ACCESS_DENIED:	  error = EACCES; break;
-    case ERROR_FILE_NOT_FOUND:    error = ENOENT; break;
-    case ERROR_SHARING_VIOLATION: error = EAGAIN; break;
-    case ERROR_ALREADY_EXISTS:    error = EEXIST; break;
-  }
-
-  errno = error;
-}
-#endif /*__WINDOWS__*/
-
-
 		 /*******************************
 		 *	      OS STUFF		*
 		 *******************************/
@@ -97,48 +76,10 @@ intervals since January 1, 1601 (UTC).
 int
 LastModifiedFile(const char *name, double *tp)
 {
-#ifdef __WINDOWS__
-  HANDLE hFile;
-  wchar_t wfile[MAXPATHLEN];
-
-#define nano * 0.000000001
-#define ntick 100.0
-#define SEC_TO_UNIX_EPOCH 11644473600.0
-
-  if ( !_xos_os_filenameW(name, wfile, MAXPATHLEN) )
-    return FALSE;
-
-  if ( (hFile=CreateFileW(wfile,
-			  0,
-			  FILE_SHARE_DELETE|FILE_SHARE_READ|FILE_SHARE_WRITE,
-			  NULL,
-			  OPEN_EXISTING,
-			  FILE_FLAG_BACKUP_SEMANTICS,
-			  NULL)) != INVALID_HANDLE_VALUE )
-  { FILETIME wt;
-    int rc;
-
-    rc = GetFileTime(hFile, NULL, NULL, &wt);
-    CloseHandle(hFile);
-
-    if ( rc )
-    { double t;
-
-      t  = (double)wt.dwHighDateTime * (4294967296.0 * ntick nano);
-      t += (double)wt.dwLowDateTime  * (ntick nano);
-      t -= SEC_TO_UNIX_EPOCH;
-
-      *tp = t;
-
-      return TRUE;
-    }
-  }
-
-  set_posix_error(GetLastError());
-
-  return FALSE;
+#ifdef O_XOS
+  return _xos_get_file_time(name, XOS_TIME_MODIFIED, tp) == 0;
 #else
-  char tmp[MAXPATHLEN];
+  char tmp[PATH_MAX];
   statstruct buf;
 
   if ( statfunc(OsPath(name, tmp), &buf) < 0 )
@@ -163,13 +104,23 @@ be accessed.
 
 static int64_t
 SizeFile(const char *path)
-{ char tmp[MAXPATHLEN];
+{
+#if O_XOS
+  uint64_t size;
+
+  if ( _xos_file_size(path, &size) == 0 )
+    return (int64_t)size;
+
+  return -1;
+#else
+  char tmp[PATH_MAX];
   statstruct buf;
 
   if ( statfunc(OsPath(path, tmp), &buf) < 0 )
     return -1;
 
   return buf.st_size;
+#endif
 }
 
 
@@ -203,7 +154,7 @@ access_mode(int mode)
 
 int
 AccessFile(const char *path, int mode)
-{ char tmp[MAXPATHLEN];
+{ char tmp[PATH_MAX];
 #ifdef HAVE_ACCESS
   return access(OsPath(path, tmp), access_mode(mode)) == 0 ? TRUE : FALSE;
 #else
@@ -215,7 +166,7 @@ int
 AccessDirectory(const char *path, int mode)
 {
 #if O_XOS
-  char tmp[MAXPATHLEN];
+  char tmp[PATH_MAX];
   return _xos_access_dir(OsPath(path, tmp), access_mode(mode)) == 0 ? TRUE : FALSE;
 #else
   return AccessFile(path, mode);
@@ -229,7 +180,7 @@ ExistsFile(const char *path)
 #ifdef O_XOS
   return _xos_exists(path, _XOS_FILE);
 #else
-  char tmp[MAXPATHLEN];
+  char tmp[PATH_MAX];
   statstruct buf;
 
   if ( statfunc(OsPath(path, tmp), &buf) == -1 || !S_ISREG(buf.st_mode) )
@@ -247,7 +198,7 @@ ExistsDirectory(const char *path)
 #ifdef O_XOS
   return _xos_exists(path, _XOS_DIR);
 #else
-  char tmp[MAXPATHLEN];
+  char tmp[PATH_MAX];
   char *ospath = OsPath(path, tmp);
   statstruct buf;
 
@@ -268,7 +219,7 @@ ReadLink(const char *f, char *buf)
 #ifdef HAVE_READLINK
   int n;
 
-  if ( (n=readlink(f, buf, MAXPATHLEN-1)) > 0 )
+  if ( (n=readlink(f, buf, PATH_MAX-1)) > 0 )
   { buf[n] = EOS;
     return buf;
   }
@@ -280,7 +231,7 @@ ReadLink(const char *f, char *buf)
 
 static char *
 DeRefLink1(const char *f, char *lbuf)
-{ char buf[MAXPATHLEN];
+{ char buf[PATH_MAX];
   char *l;
 
   if ( (l=ReadLink(f, buf)) )
@@ -316,7 +267,7 @@ removed). Returns NULL if more than 20 links have been followed.
 
 char *
 DeRefLink(const	char *link, char *buf)
-{ char tmp[MAXPATHLEN];
+{ char tmp[PATH_MAX];
   char *f;
   int n = 20;				/* avoid loop! */
 
@@ -361,7 +312,7 @@ SameFile(const char *f1, const char *f2)
 #ifdef __unix__				/* doesn't work on most not Unix's */
   { statstruct buf1;
     statstruct buf2;
-    char tmp[MAXPATHLEN];
+    char tmp[PATH_MAX];
 
     if ( statfunc(OsPath(f1, tmp), &buf1) != 0 ||
 	 statfunc(OsPath(f2, tmp), &buf2) != 0 )
@@ -371,7 +322,7 @@ SameFile(const char *f1, const char *f2)
   }
 #endif
 #ifdef O_XOS
-  return _xos_same_file(f1, f2);
+  return _xos_same_file(f1, f2) == TRUE;
 #endif /*O_XOS*/
     /* Amazing! There is no simple way to check two files for identity. */
     /* stat() and fstat() both return dummy values for inode and device. */
@@ -389,7 +340,7 @@ otherwise.
 
 int
 RemoveFile(const char *path)
-{ char tmp[MAXPATHLEN];
+{ char tmp[PATH_MAX];
 
 #ifdef HAVE_REMOVE
   return remove(OsPath(path, tmp)) == 0 ? TRUE : FALSE;
@@ -401,8 +352,8 @@ RemoveFile(const char *path)
 
 static int
 RenameFile(const char *old, const char *new)
-{ char oldbuf[MAXPATHLEN];
-  char newbuf[MAXPATHLEN];
+{ char oldbuf[PATH_MAX];
+  char newbuf[PATH_MAX];
   char *osold, *osnew;
 
   osold = OsPath(old, oldbuf);
@@ -430,7 +381,8 @@ RenameFile(const char *old, const char *new)
 static int
 MarkExecutable(const char *name)
 {
-#if (defined(HAVE_STAT) && defined(HAVE_CHMOD)) || defined(__unix__)
+#if ( (defined(HAVE_STAT) && defined(HAVE_CHMOD)) || \
+      defined(__unix__) ) && !defined(__WINDOWS__)
   statstruct buf;
   mode_t um;
 
@@ -501,7 +453,9 @@ get_file_name(term_t n, char **namep, char *tmp, int flags)
   { fid_t fid;
 
     if ( (fid = PL_open_foreign_frame()) )
-    { predicate_t pred = PL_predicate("absolute_file_name", 3, "system");
+    { if ( !GD->procedures.absolute_file_name3 )
+	GD->procedures.absolute_file_name3 =
+	  PL_predicate("absolute_file_name", 3, "system");
       term_t av = PL_new_term_refs(3);
       term_t options = PL_copy_term_ref(av+2);
       int rc = TRUE;
@@ -520,7 +474,8 @@ get_file_name(term_t n, char **namep, char *tmp, int flags)
 	rc = add_option(options, FUNCTOR_access1, ATOM_execute);
 
       if ( rc ) rc = PL_unify_nil(options);
-      if ( rc ) rc = PL_call_predicate(NULL, cflags, pred, av);
+      if ( rc ) rc = PL_call_predicate(NULL, cflags,
+				       GD->procedures.absolute_file_name3, av);
       if ( rc ) rc = PL_get_nchars(av+1, &len, namep,
 				   CVT_ATOMIC|BUF_STACK|REP_FN);
       if ( rc && strlen(*namep) != len )
@@ -547,12 +502,12 @@ get_file_name(term_t n, char **namep, char *tmp, int flags)
     return PL_error(NULL, 0, "file name contains a 0-code",
 		    ERR_DOMAIN, ATOM_file_name, n);
   }
-  if ( len >= MAXPATHLEN )
+  if ( len >= PATH_MAX )
     return PL_error(NULL, 0, NULL, ERR_REPRESENTATION,
 		    ATOM_max_path_length);
 
   if ( truePrologFlag(PLFLAG_FILEVARS) )
-  { if ( !(name = expandVars(name, tmp, MAXPATHLEN)) )
+  { if ( !(name = expandVars(name, tmp, PATH_MAX)) )
       return FALSE;
   }
 
@@ -587,8 +542,8 @@ get_file_name(term_t n, char **namep, char *tmp, int flags)
 
 int
 PL_get_file_name(term_t n, char **namep, int flags)
-{ char buf[MAXPATHLEN];
-  char ospath[MAXPATHLEN];
+{ char buf[PATH_MAX];
+  char ospath[PATH_MAX];
   char *name;
   int rc;
 
@@ -608,8 +563,8 @@ PL_get_file_name(term_t n, char **namep, int flags)
 
 int
 PL_get_file_nameW(term_t n, wchar_t **namep, int flags)
-{ char buf[MAXPATHLEN];
-  char ospath[MAXPATHLEN];
+{ char buf[PATH_MAX];
+  char ospath[PATH_MAX];
   char *name;
   int rc;
 
@@ -627,9 +582,9 @@ PL_get_file_nameW(term_t n, wchar_t **namep, int flags)
     { int chr;
 
       PL_utf8_code_point(&s, NULL, &chr);
-      addBuffer(b, (wchar_t)chr, wchar_t);
+      addWcharBuffer(b, chr);
     }
-    addBuffer(b, (wchar_t)0, wchar_t);
+    addWcharBuffer(b, 0);
 
     *namep = baseBuffer(b, wchar_t);
   }
@@ -734,7 +689,7 @@ PRED_IMPL("access_file", 2, access_file, 0)
     return TRUE;
 
   if ( md == ACCESS_WRITE && !AccessFile(n, ACCESS_EXIST) )
-  { char tmp[MAXPATHLEN];
+  { char tmp[PATH_MAX];
     char *dir;
 
     if ( !(dir = DirName(n, tmp)) )
@@ -754,7 +709,7 @@ PRED_IMPL("access_file", 2, access_file, 0)
 static
 PRED_IMPL("read_link", 3, read_link, 0)
 { char *n, *l, *t;
-  char buf[MAXPATHLEN];
+  char buf[PATH_MAX];
 
   term_t file = A1;
   term_t link = A2;
@@ -842,7 +797,7 @@ PRED_IMPL("same_file", 2, same_file, 0)
 static
 PRED_IMPL("file_base_name", 2, file_base_name, 0)
 { char *n, *b;
-  char tmp[MAXPATHLEN];
+  char tmp[PATH_MAX];
 
   if ( !PL_get_chars(A1, &n, CVT_ALL|REP_FN|CVT_EXCEPTION) )
     return FALSE;
@@ -857,7 +812,7 @@ PRED_IMPL("file_base_name", 2, file_base_name, 0)
 static
 PRED_IMPL("file_directory_name", 2, file_directory_name, 0)
 { char *n, *d;
-  char tmp[MAXPATHLEN];
+  char tmp[PATH_MAX];
 
   if ( !PL_get_chars(A1, &n, CVT_ALL|REP_FN|CVT_EXCEPTION) )
     return FALSE;
@@ -908,7 +863,7 @@ PRED_IMPL("$tmp_file_stream", 4, tmp_file_stream, 0)
   if ( !PL_get_chars(A1, &ext, CVT_ATOM|CVT_STRING|CVT_EXCEPTION) ||
        !PL_get_atom_ex(A2, &encoding) )
     return FALSE;
-  if ( (enc = atom_to_encoding(encoding)) == ENC_UNKNOWN )
+  if ( (enc = PL_atom_to_encoding(encoding)) == ENC_UNKNOWN )
   { if ( encoding == ATOM_binary )
     { enc = ENC_OCTET;
       mode = "wb";
@@ -1027,7 +982,7 @@ PRED_IMPL("rename_file", 2, rename_file, 0)
 static
 PRED_IMPL("$absolute_file_name", 2, absolute_file_name, 0)
 { char *n;
-  char tmp[MAXPATHLEN];
+  char tmp[PATH_MAX];
 
   term_t name = A1;
   term_t expanded = A2;
@@ -1043,7 +998,7 @@ PRED_IMPL("$absolute_file_name", 2, absolute_file_name, 0)
 
 static
 PRED_IMPL("$cwd", 1, cwd, 0)
-{ char buf[MAXPATHLEN];
+{ char buf[PATH_MAX];
   const char *wd;
 
   if ( !(wd = PL_cwd(buf, sizeof(buf))) )
@@ -1104,7 +1059,7 @@ static
 PRED_IMPL("file_name_extension", 3, file_name_extension, 0)
 { PRED_LD
   char *b = NULL, *e = NULL, *f;
-  char buf[MAXPATHLEN];
+  char buf[PATH_MAX];
 
   term_t base = A1;
   term_t ext  = A2;
@@ -1146,7 +1101,7 @@ PRED_IMPL("file_name_extension", 3, file_name_extension, 0)
       e++;
     if ( has_extension(b, e) || e[0] == EOS )
       return PL_unify(base, full);
-    if ( strlen(b) + 1 + strlen(e) + 1 > MAXPATHLEN )
+    if ( strlen(b) + 1 + strlen(e) + 1 > PATH_MAX )
       return name_too_long();
     strcpy(buf, b);
     s = buf + strlen(buf);
@@ -1171,23 +1126,23 @@ PRED_IMPL("prolog_to_os_filename", 2, prolog_to_os_filename, 0)
 
   if ( !PL_is_variable(pl) )
   { char *n;
-    wchar_t buf[MAXPATHLEN];
+    char buf[PATH_MAX];
 
     if ( PL_get_chars(pl, &n, CVT_ALL|REP_UTF8|CVT_EXCEPTION) )
-    { if ( !_xos_os_filenameW(n, buf, MAXPATHLEN) )
+    { if ( !_xos_os_filename(n, buf, PATH_MAX) )
 	return name_too_long();
 
-      return PL_unify_wchars(os, PL_ATOM, -1, buf);
+      return PL_unify_chars(os, PL_ATOM|REP_UTF8, -1, buf);
     }
     return FALSE;
   }
 
   if ( PL_get_wchars(os, NULL, &wn, CVT_ALL) )
-  { wchar_t lbuf[MAXPATHLEN];
-    char buf[MAXPATHLEN];
+  { wchar_t lbuf[PATH_MAX];
+    char buf[PATH_MAX];
 
-    _xos_long_file_nameW(wn, lbuf, MAXPATHLEN);
-    _xos_canonical_filenameW(lbuf, buf, MAXPATHLEN, 0);
+    _xos_long_file_nameW(wn, lbuf, PATH_MAX);
+    _xos_canonical_filenameW(lbuf, buf, PATH_MAX, 0);
 
     return PL_unify_chars(pl, PL_ATOM|REP_UTF8, -1, buf);
   }
@@ -1227,7 +1182,7 @@ PRED_IMPL("$my_file", 1, my_file, 0)
 
 #ifdef HAVE_GETUID
 { statstruct buf;
-  char tmp[MAXPATHLEN];
+  char tmp[PATH_MAX];
 
   if ( statfunc(OsPath(n, tmp), &buf) < 0 )
   { perror(tmp);
