@@ -40,7 +40,8 @@
 
 #include "pl-incl.h"
 #include "pl-ctype.h"
-#include "os/pl-utf8.h"
+#include "pl-utf8.h"
+#include "../pl-fli.h"
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -122,9 +123,9 @@ typedef struct
 { tmp_buffer	pattern;
 } compiled_pattern;
 
-static char *compile_pattern(compiled_pattern *pattern, char *s,
-			     int curl, int flags);
-static int match_pattern(matchcode *pattern, char *s, int flags);
+static const char *compile_pattern(compiled_pattern *pattern, const char *s,
+				   int curl, int flags);
+static int match_pattern(matchcode *pattern, const char *s, int flags);
 
 #define Output(c)  addBuffer(&Out->pattern, c, matchcode)
 #define Here()	   entriesBuffer(&Out->pattern, matchcode)
@@ -153,23 +154,23 @@ compilePattern(char *p, compiled_pattern *cbuf, int flags)
 }
 
 static int
-utf8_peek(char *p, int n)
+utf8_peek(const char *p, int n)
 { int c;
 
   while(n-- > 0)
     p = utf8_skip_char(p);
 
-  utf8_get_char(p, &c);
+  PL_utf8_code_point(&p, NULL, &c);
 
   return c;
 }
 
-static char *
-compile_pattern(compiled_pattern *Out, char *p, int curl, int mflags)
+static const char *
+compile_pattern(compiled_pattern *Out, const char *p, int curl, int mflags)
 { int c;
 
   for(;;)
-  { p = utf8_get_char(p, &c);
+  { PL_utf8_code_point(&p, NULL, &c);
 
     switch(c)
     { case EOS:
@@ -178,7 +179,7 @@ compile_pattern(compiled_pattern *Out, char *p, int curl, int mflags)
       case '\\':
       { int c2;
 
-	p = utf8_get_char(p, &c2);
+	PL_utf8_code_point(&p, NULL, &c2);
 	Output(c2 == EOS ? c : c2);
 	if ( c2 == EOS )
 	  break;
@@ -198,10 +199,7 @@ compile_pattern(compiled_pattern *Out, char *p, int curl, int mflags)
 	Output(0);
 
 	for(;;)
-	{ int cn;
-
-	  p = utf8_get_char(p, &c);
-	  utf8_get_char(p, &cn);
+	{ PL_utf8_code_point(&p, NULL, &c);
 
 	  switch( c )
 	  { case EOS:
@@ -209,7 +207,7 @@ compile_pattern(compiled_pattern *Out, char *p, int curl, int mflags)
 	    case '\\':
 	    { int c2;
 
-	      p = utf8_get_char(p, &c2);
+	      PL_utf8_code_point(&p, NULL, &c2);
 	      if ( c2 == EOS )
 		return PL_syntax_error("Unmatched '['", NULL),NULL;
 	      Output(c2);
@@ -298,12 +296,12 @@ compile_pattern(compiled_pattern *Out, char *p, int curl, int mflags)
 
 
 static int
-matchPattern(char *s, compiled_pattern *cbuf, int flags)
+matchPattern(const char *s, compiled_pattern *cbuf, int flags)
 { return match_pattern(baseBuffer(&cbuf->pattern, matchcode), s, flags);
 }
 
 static bool
-match_pattern(matchcode *p, char *s, int flags)
+match_pattern(matchcode *p, const char *s, int flags)
 { matchcode c;
 
   for(;;)
@@ -320,7 +318,7 @@ match_pattern(matchcode *p, char *s, int flags)
 	int sz = *p++;
 	matchcode *anyend = &p[sz];
 
-	s = utf8_get_char(s, &c2);
+	PL_utf8_code_point(&s, NULL, &c2);
 	if ( (flags&M_IGNCASE) )
 	  c2 = makeLowerW(c2);
 
@@ -346,7 +344,7 @@ match_pattern(matchcode *p, char *s, int flags)
 	do
 	{ if ( match_pattern(p, s, flags) )
 	    return TRUE;
-	  s = utf8_get_char(s, &c2);
+	  PL_utf8_code_point(&s, NULL, &c2);
 	} while(c2);
 
 	return FALSE;
@@ -362,7 +360,7 @@ match_pattern(matchcode *p, char *s, int flags)
       default:						/* character */
       { int c2;
 
-	s = utf8_get_char(s, &c2);
+	PL_utf8_code_point(&s, NULL, &c2);
 	if ( (flags&M_IGNCASE) )
 	  c2 = makeLowerW(c2);
 
@@ -379,21 +377,22 @@ match_pattern(matchcode *p, char *s, int flags)
 /** wildcard_match(+Pattern, +Name [, +Options]) is semidet.
 */
 
-static const opt_spec wildcard_options[] =
+static const PL_option_t wildcard_options[] =
 { { ATOM_case_sensitive,    OPT_BOOL },
   { NULL_ATOM,		    0 }
 };
 
 
+#define wildcard_match(pattern, string, options) LDFUNC(wildcard_match, pattern, string, options)
 static int
-wildcard_match(term_t pattern, term_t string, term_t options ARG_LD)
+wildcard_match(DECL_LD term_t pattern, term_t string, term_t options)
 { char *p, *s;
   int rc = FALSE;
   int mflags = 0;
   int case_sensitive = TRUE;
 
   if ( options &&
-       !scan_options(options, 0, ATOM_wildcard_option,
+       !PL_scan_options(options, 0, "wildcard_option",
 		     wildcard_options, &case_sensitive) )
     return FALSE;
   if ( !case_sensitive )
@@ -418,14 +417,14 @@ static
 PRED_IMPL("wildcard_match", 2, wildcard_match, 0)
 { PRED_LD
 
-  return wildcard_match(A1, A2, 0 PASS_LD);
+  return wildcard_match(A1, A2, 0);
 }
 
 static
 PRED_IMPL("wildcard_match", 3, wildcard_match, 0)
 { PRED_LD
 
-  return wildcard_match(A1, A2, A3 PASS_LD);
+  return wildcard_match(A1, A2, A3);
 }
 
 
@@ -473,8 +472,9 @@ free_expand_info(GlobInfo info)
 
 
 #ifndef __WINDOWS__
+#define mb_add_path(path, info) LDFUNC(mb_add_path, path, info)
 static void
-mb_add_path(const char *path, GlobInfo info ARG_LD)
+mb_add_path(DECL_LD const char *path, GlobInfo info)
 { int idx = (int)entriesBuffer(&info->strings, char);
   PL_chars_t txt;
 
@@ -484,7 +484,7 @@ mb_add_path(const char *path, GlobInfo info ARG_LD)
   txt.encoding  = ENC_ANSI;
   txt.storage   = PL_CHARS_HEAP;
   txt.canonical = FALSE;
-  if ( PL_canonicalise_text(&txt) &&
+  if ( (PL_canonicalise_text(&txt) == TRUE) &&
        PL_mb_text(&txt, REP_UTF8) )
   { addMultipleBuffer(&info->strings, txt.text.t, txt.length+1, char);
     addBuffer(&info->files, idx, int);
@@ -534,8 +534,9 @@ un_escape(char *to, const char *from, const char *end)
 }
 
 
+#define utf8_exists_file(name) LDFUNC(utf8_exists_file, name)
 static int
-utf8_exists_file(const char *name ARG_LD)
+utf8_exists_file(DECL_LD const char *name)
 {
 #ifndef __WINDOWS__
   PL_chars_t txt;
@@ -547,7 +548,7 @@ utf8_exists_file(const char *name ARG_LD)
   txt.encoding  = ENC_UTF8;
   txt.storage   = PL_CHARS_HEAP;
   txt.canonical = FALSE;
-  rc = ( PL_canonicalise_text(&txt) &&
+  rc = ( (PL_canonicalise_text(&txt) == TRUE) &&
 	 PL_mb_text(&txt, REP_FN) &&
 	 AccessFile(txt.text.t, ACCESS_EXIST) );
   PL_free_text(&txt);
@@ -555,12 +556,14 @@ utf8_exists_file(const char *name ARG_LD)
 
   return rc;
 #else
-  return AccessFile(name, ACCESS_EXIST);
+  // _xos_access_dir simply uses _waccess().  That is good enough for us.
+  return _xos_access_dir(name, F_OK) == 0 ? TRUE : FALSE;
 #endif
 }
 
+#define utf8_opendir(name) LDFUNC(utf8_opendir, name)
 static DIR*
-utf8_opendir(const char *name ARG_LD)
+utf8_opendir(DECL_LD const char *name)
 {
 #ifndef __WINDOWS__
   PL_chars_t txt;
@@ -572,7 +575,7 @@ utf8_opendir(const char *name ARG_LD)
   txt.encoding  = ENC_UTF8;
   txt.storage   = PL_CHARS_HEAP;
   txt.canonical = FALSE;
-  if ( PL_canonicalise_text(&txt) &&
+  if ( (PL_canonicalise_text(&txt) == TRUE) &&
        PL_mb_text(&txt, REP_FN) )
     rc = opendir(txt.text.t);
   else
@@ -590,8 +593,8 @@ static int
 expand(const char *pattern, GlobInfo info)
 { GET_LD
   const char *pat = pattern;
-  char prefix[MAXPATHLEN];		/* before first pattern */
-  char patbuf[MAXPATHLEN];		/* pattern buffer */
+  char prefix[PATH_MAX];		/* before first pattern */
+  char patbuf[PATH_MAX];		/* pattern buffer */
   size_t prefix_len;
   int end, dot;
   int mflags = 0;
@@ -623,16 +626,16 @@ expand(const char *pattern, GlobInfo info)
 
 	    end = info->end;
 	    for( ; info->start < end; info->start++ )
-	    { char path[MAXPATHLEN];
+	    { char path[PATH_MAX];
 	      const char *entry = expand_entry(info, info->start);
 	      size_t plen = strlen(entry);
 
-	      if ( plen+prefix_len+2 <= MAXPATHLEN )
+	      if ( plen+prefix_len+2 <= PATH_MAX )
 	      { strcpy(path, entry);
 		if ( prefix[0] && plen > 0 && path[plen-1] != '/' )
 		  path[plen++] = '/';
 		strcpy(&path[plen], prefix);
-		if ( end == 1 || utf8_exists_file(path PASS_LD) )
+		if ( end == 1 || utf8_exists_file(path) )
 		  utf8_add_path(path, info);
 	      }
 	    }
@@ -681,8 +684,8 @@ expand(const char *pattern, GlobInfo info)
     for(; info->start < end; info->start++)
     { DIR *d;
       struct dirent *e;
-      char path[MAXPATHLEN];
-      char tmp[MAXPATHLEN];
+      char path[PATH_MAX];
+      char tmp[PATH_MAX];
       const char *current = expand_entry(info, info->start);
       size_t clen = strlen(current);
 
@@ -692,20 +695,16 @@ expand(const char *pattern, GlobInfo info)
       strcpy(path, current);
       strcpy(&path[clen], prefix);
 
-      if ( (d=utf8_opendir((path[0] ? OsPath(path, tmp) : ".") PASS_LD)) )
+      if ( (d=utf8_opendir((path[0] ? OsPath(path, tmp) : "."))) )
       { size_t plen = clen+prefix_len;
 
 	if ( plen > 0 && path[plen-1] != '/' )
 	  path[plen++] = '/';
 
 	for(e=readdir(d); e; e = readdir(d))
-	{
-#ifdef __MSDOS__
-	  strlwr(e->d_name);
-#endif
-	  if ( (dot || e->d_name[0] != '.') &&
+	{ if ( (dot || e->d_name[0] != '.') &&
 	       matchPattern(e->d_name, &info->pattern, mflags) )
-	  { char newp[MAXPATHLEN];
+	  { char newp[PATH_MAX];
 
 	    if ( plen+strlen(e->d_name)+1 < sizeof(newp) )
 	    { strcpy(newp, path);
@@ -713,10 +712,12 @@ expand(const char *pattern, GlobInfo info)
 #ifdef __WINDOWS__			/* readdir() emulation emits UTF-8 */
 	      utf8_add_path(newp, info);
 #else
-	      mb_add_path(newp, info PASS_LD);
+	      mb_add_path(newp, info);
 #endif
 	    }
 	  }
+	  if ( PL_handle_signals() < 0 )
+	    return FALSE;
 	}
 	closedir(d);
       }
@@ -729,6 +730,23 @@ expand(const char *pattern, GlobInfo info)
 }
 
 
+static int
+utf8casecmp(const char *s1, const char *s2)
+{ while(*s1 && *s2)
+  { int c1, c2;
+
+    s1 = utf8_get_char(s1, &c1);
+    s2 = utf8_get_char(s2, &c2);
+    c1 = makeLowerW(c1);
+    c2 = makeLowerW(c2);
+
+    if ( c1 != c2 )
+      return c1-c2;
+  }
+
+  return *s1 && *s2 ? 0 : *s1 ? 1 : -1;
+}
+
 static int
 compareBagEntries(const void *a1, const void *a2)
 { GET_LD
@@ -741,9 +759,9 @@ compareBagEntries(const void *a1, const void *a2)
   s2 = expand_str(info, i2);
 
   if ( truePrologFlag(PLFLAG_FILE_CASE) )
-    return mbscoll(s1, s2);
+    return strcmp(s1, s2);
   else
-    return mbscasecoll(s1, s2);
+    return utf8casecmp(s1, s2);
 }
 
 
@@ -761,7 +779,7 @@ sort_expand(GlobInfo info)
 static
 PRED_IMPL("expand_file_name", 2, expand_file_name, 0)
 { PRED_LD
-  char spec[MAXPATHLEN];
+  char spec[PATH_MAX];
   char *s;
   glob_info info;
   term_t l    = PL_copy_term_ref(A2);
