@@ -3,8 +3,8 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2021, University of Amsterdam
-                              VU University Amsterdam
+    Copyright (c)  2011-2022, University of Amsterdam
+			      VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
     All rights reserved.
@@ -41,7 +41,9 @@
 #include <windows.h>
 #include <process.h>			/* getpid() */
 #endif
+#define _GNU_SOURCE			/* get dladdr() */
 #include "pl-prologflag.h"
+#include "pl-utf8.h"
 #include "pl-ctype.h"
 #include "../pl-arith.h"
 #include "../pl-tabling.h"
@@ -54,10 +56,22 @@
 #include "../pl-modul.h"
 #include "../pl-version.h"
 #include <ctype.h>
+#include <time.h>
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
-#include <time.h>
+#ifdef __WINDOWS__
+#include "../pl-nt.h"
+#endif
+#ifdef HAVE_DLADDR
+#include <dlfcn.h>
+#endif
+
+#undef false
+#undef true
+#undef bool
+#define true(s, a)         ((s)->flags & (a))
+#define false(s, a)        (!true((s), (a)))
 
 
 		 /*******************************
@@ -505,7 +519,7 @@ setOccursCheck(atom_t a)
 static int
 setEncoding(atom_t a)
 { GET_LD
-  IOENC enc = atom_to_encoding(a);
+  IOENC enc = PL_atom_to_encoding(a);
 
   if ( enc == ENC_UNKNOWN )
   { term_t value = PL_new_term_ref();
@@ -720,35 +734,35 @@ set_prolog_flag_unlocked(DECL_LD Module m, atom_t k, term_t value, int flags)
 	  freeHeap(f, sizeof(*f));
 	  return FALSE;
 	}
-        f->flags = FT_ATOM;
-        PL_register_atom(f->value.a);
+	f->flags = FT_ATOM;
+	PL_register_atom(f->value.a);
 	break;
       case FT_BOOL:
       { int b;
 	if ( !PL_get_bool_ex(value, &b) )
 	  goto wrong_type;
-        f->flags = FT_BOOL;
+	f->flags = FT_BOOL;
 	f->value.a = (b ? ATOM_true : ATOM_false);
 	break;
       }
       case FT_INTEGER:
 	if ( !PL_get_int64_ex(value, &f->value.i) )
 	  goto wrong_type;
-        f->flags = FT_INTEGER;
+	f->flags = FT_INTEGER;
 	break;
       case FT_FLOAT:
 	if ( !PL_get_float_ex(value, &f->value.f) )
 	  goto wrong_type;
-        f->flags = FT_FLOAT;
+	f->flags = FT_FLOAT;
 	break;
       case FT_TERM:
 	if ( !PL_is_ground(value) )
 	{ PL_error(NULL, 0, NULL, ERR_INSTANTIATION);
 	  goto wrong_type;
 	}
-        if ( !(f->value.t = PL_record(value)) )
+	if ( !(f->value.t = PL_record(value)) )
 	  goto wrong_type;
-        f->flags = FT_TERM;
+	f->flags = FT_TERM;
 	break;
     }
 
@@ -928,6 +942,8 @@ set_prolog_flag_unlocked(DECL_LD Module m, atom_t k, term_t value, int flags)
 	  return FALSE;
       } else if ( k == ATOM_string_stack_tripwire )
       { LD->fli.string_buffers.tripwire = (unsigned int)i;
+      } else if ( k == ATOM_heartbeat )
+      { LD->yield.frequency = i/16;
       }
       break;
     }
@@ -988,7 +1004,7 @@ PRED_IMPL("set_prolog_flag", 2, set_prolog_flag, PL_FA_ISO)
 /** create_prolog_flag(+Key, +Value, +Options) is det.
 */
 
-static const opt_spec prolog_flag_options[] =
+static const PL_option_t prolog_flag_options[] =
 { { ATOM_type,   OPT_ATOM },
   { ATOM_access, OPT_ATOM },
   { ATOM_keep,   OPT_BOOL },
@@ -1003,7 +1019,7 @@ PRED_IMPL("create_prolog_flag", 3, create_prolog_flag, PL_FA_ISO)
   atom_t access = ATOM_read_write;
   int keep = FALSE;
 
-  if ( !scan_options(A3, 0, ATOM_prolog_flag_option, prolog_flag_options,
+  if ( !PL_scan_options(A3, 0, "prolog_flag_option", prolog_flag_options,
 		     &type, &access, &keep) )
     return FALSE;
 
@@ -1069,21 +1085,21 @@ PL_current_prolog_flag(atom_t name, int type, void *value)
 	  *vp = f->value.a;
 	  return TRUE;
 	}
-        return FALSE;
+	return FALSE;
       case PL_INTEGER:
 	if ( (f->flags&FT_MASK) == FT_INTEGER )
 	{ int64_t *vp = value;
 	  *vp = f->value.i;
 	  return TRUE;
 	}
-        return FALSE;
+	return FALSE;
       case PL_FLOAT:
 	if ( (f->flags&FT_MASK) == FT_FLOAT )
 	{ double *vp = value;
 	  *vp = f->value.f;
 	  return TRUE;
 	}
-        return FALSE;
+	return FALSE;
       case PL_TERM:
 	if ( (f->flags&FT_MASK) == FT_TERM )
 	{ term_t *vp = value;
@@ -1091,7 +1107,7 @@ PL_current_prolog_flag(atom_t name, int type, void *value)
 
 	  return PL_recorded(f->value.t, t);
 	}
-        return FALSE;
+	return FALSE;
     }
   }
 
@@ -1150,16 +1166,16 @@ unify_prolog_flag_value(Module m, atom_t key, prolog_flag *f, term_t val)
     switch ( getUnknownModule(m) )
     { case UNKNOWN_ERROR:
 	v = ATOM_error;
-        break;
+	break;
       case UNKNOWN_WARNING:
 	v = ATOM_warning;
-        break;
+	break;
       case UNKNOWN_FAIL:
 	v = ATOM_fail;
-        break;
+	break;
       default:
 	assert(0);
-        return FALSE;
+	return FALSE;
     }
 
     return PL_unify_atom(val, v);
@@ -1401,11 +1417,11 @@ set_arch(void)
     { switch(cputype&0xff)
       { case 7:
 	  setPrologFlag("arch", FT_ATOM|FF_READONLY, "x86_64-darwin");
-          return;
-        case 12:
+	  return;
+	case 12:
 	  setPrologFlag("arch", FT_ATOM|FF_READONLY, "arm64-darwin");
-          return;
-        default:
+	  return;
+	default:
 	  Sdprintf("sysctlbyname() cputype = %d (unknown)\n", (int)cputype);
       }
     }
@@ -1415,7 +1431,32 @@ set_arch(void)
   setPrologFlag("arch", FT_ATOM|FF_READONLY, PLARCH);
 }
 
+static void
+set_libswipl(void)
+{
+#ifdef __WINDOWS__
+  { char buf[PATH_MAX];
+    char *s;
+    if ( (s=findModulePath("libswipl.dll", buf, sizeof(buf))) )
+    { setPrologFlag("libswipl", FT_ATOM|FF_READONLY, s);
+      return;
+    }
+  }
+#endif
 
+#ifdef HAVE_DLADDR
+  Dl_info info;
+
+  if ( dladdr((void*)(intptr_t)PL_initialise, &info) && info.dli_fname )
+  { setPrologFlag("libswipl", FT_ATOM|FF_READONLY, info.dli_fname);
+    return;
+  }
+#endif
+
+#ifdef LIBPL_PATH
+  setPrologFlag("libswipl", FT_ATOM, LIBPL_PATH);
+#endif
+}
 
 		 /*******************************
 		 *	INITIALISE FEATURES	*
@@ -1431,16 +1472,19 @@ set_arch(void)
 #define C_LIBPLSO ""
 #endif
 
+#ifdef __WINDOWS__
+#include "../pl-nt.h"
+#endif
+
 static void
 initPrologFlagTable(void)
 { if ( !GD->prolog_flag.table )
   { initPrologThreads();	/* may be called before PL_initialise() */
 
-    GD->prolog_flag.table = newHTable(64);
+    GD->prolog_flag.table = newHTable(256);
   }
 }
 
-
 void
 initPrologFlags(void)
 { GET_LD
@@ -1459,17 +1503,22 @@ initPrologFlags(void)
 #endif
   setPrologFlag("file_name_case_handling", FT_ATOM,
 		stringAtom(currentFileNameCaseHandling()));
-  setPrologFlag("version", FT_INTEGER|FF_READONLY, PLVERSION);
+  setPrologFlag("path_max", FT_INTEGER|FF_READONLY, (intptr_t)PATH_MAX);
+  setPrologFlag("version", FT_INTEGER|FF_READONLY, (intptr_t)PLVERSION);
   setPrologFlag("dialect", FT_ATOM|FF_READONLY, "swi");
   if ( systemDefaults.home )
     setPrologFlag("home", FT_ATOM|FF_READONLY, systemDefaults.home);
 #ifdef PLSHAREDHOME
   setPrologFlag("shared_home", FT_ATOM|FF_READONLY, PLSHAREDHOME);
+#endif
+  set_libswipl();
+#ifdef EXEC_FORMAT
+  setPrologFlag("executable_format", FT_ATOM|FF_READONLY, EXEC_FORMAT);
 #endif
   if ( GD->paths.executable )
     setPrologFlag("executable", FT_ATOM|FF_READONLY, GD->paths.executable);
 #if defined(HAVE_GETPID) || defined(EMULATE_GETPID)
-  setPrologFlag("pid", FT_INTEGER|FF_READONLY, getpid());
+  setPrologFlag("pid", FT_INTEGER|FF_READONLY, (intptr_t)getpid());
 #endif
   setPrologFlag("optimise", FT_BOOL, GD->cmdline.optimise, PLFLAG_OPTIMISE);
   setPrologFlag("optimise_unify", FT_BOOL, TRUE, PLFLAG_OPTIMISE_UNIFY);
@@ -1479,16 +1528,9 @@ initPrologFlags(void)
   setPrologFlag("protect_static_code", FT_BOOL, FALSE,
 		PLFLAG_PROTECT_STATIC_CODE);
   setPrologFlag("last_call_optimisation", FT_BOOL, TRUE, PLFLAG_LASTCALL);
+  setPrologFlag("vmi_builtin", FT_BOOL, TRUE, PLFLAG_VMI_BUILTIN);
   setPrologFlag("warn_override_implicit_import", FT_BOOL, TRUE,
 		PLFLAG_WARN_OVERRIDE_IMPLICIT_IMPORT);
-  setPrologFlag("c_cc",	     FT_ATOM, C_CC);
-  setPrologFlag("c_libs",    FT_ATOM, C_LIBS);
-#ifdef C_LIBDIR
-  setPrologFlag("c_libdir",  FT_ATOM, C_LIBDIR);
-#endif
-  setPrologFlag("c_libplso", FT_ATOM, C_LIBPLSO);
-  setPrologFlag("c_ldflags", FT_ATOM, C_LDFLAGS);
-  setPrologFlag("c_cflags",  FT_ATOM, C_CFLAGS);
   setPrologFlag("tmp_dir", FT_ATOM, SWIPL_TMP_DIR);
 #if defined(O_LARGEFILES) || SIZEOF_LONG == 8
   setPrologFlag("large_files", FT_BOOL|FF_READONLY, TRUE, 0);
@@ -1497,19 +1539,28 @@ initPrologFlags(void)
   setPrologFlag("gc",	  FT_BOOL,	       TRUE,  PLFLAG_GC);
   setPrologFlag("trace_gc",  FT_BOOL,	       FALSE, PLFLAG_TRACE_GC);
 #ifdef O_ATOMGC
-  setPrologFlag("agc_margin",FT_INTEGER,	       GD->atoms.margin);
+  setPrologFlag("agc_margin", FT_INTEGER, (intptr_t)GD->atoms.margin);
+  setPrologFlag("agc_close_streams", FT_BOOL, FALSE, PLFLAG_AGC_CLOSE_STREAMS);
 #endif
-  setPrologFlag("table_space", FT_INTEGER, GD->options.tableSpace);
+  setPrologFlag("table_space", FT_INTEGER, (intptr_t)GD->options.tableSpace);
 #ifdef O_PLMT
-  setPrologFlag("shared_table_space", FT_INTEGER, GD->options.sharedTableSpace);
+  setPrologFlag("shared_table_space", FT_INTEGER, (intptr_t)GD->options.sharedTableSpace);
 #endif
-  setPrologFlag("stack_limit", FT_INTEGER, LD->stacks.limit);
-#if defined(HAVE_DLOPEN) || defined(HAVE_SHL_LOAD) || defined(EMULATE_DLOPEN)
-  setPrologFlag("open_shared_object",	  FT_BOOL|FF_READONLY, TRUE, 0);
-  setPrologFlag("shared_object_extension",	  FT_ATOM|FF_READONLY, SO_EXT);
+  setPrologFlag("stack_limit", FT_INTEGER, (intptr_t)LD->stacks.limit);
+#ifdef O_DYNAMIC_EXTENSIONS
+  setPrologFlag("open_shared_object",	     FT_BOOL|FF_READONLY, TRUE, 0);
+  setPrologFlag("shared_object_extension",   FT_ATOM|FF_READONLY, SO_EXT);
   setPrologFlag("shared_object_search_path", FT_ATOM|FF_READONLY, SO_PATH);
+  setPrologFlag("c_cc",	     FT_ATOM, C_CC);
+  setPrologFlag("c_libs",    FT_ATOM, C_LIBS);
+#ifdef C_LIBDIR
+  setPrologFlag("c_libdir",  FT_ATOM, C_LIBDIR);
 #endif
-  setPrologFlag("address_bits", FT_INTEGER|FF_READONLY, sizeof(void*)*8);
+  setPrologFlag("c_libplso", FT_ATOM, C_LIBPLSO);
+  setPrologFlag("c_ldflags", FT_ATOM, C_LDFLAGS);
+  setPrologFlag("c_cflags",  FT_ATOM, C_CFLAGS);
+#endif
+  setPrologFlag("address_bits", FT_INTEGER|FF_READONLY, (intptr_t)sizeof(void*)*8);
 #ifdef HAVE_POPEN
   setPrologFlag("pipe", FT_BOOL, TRUE, 0);
 #endif
@@ -1517,7 +1568,7 @@ initPrologFlags(void)
   setPrologFlag("threads",	FT_BOOL, !GD->options.nothreads, 0);
   if ( GD->options.xpce >= 0 )
     setPrologFlag("xpce",	FT_BOOL, GD->options.xpce, 0);
-  setPrologFlag("system_thread_id", FT_INTEGER|FF_READONLY, 0, 0);
+  setPrologFlag("system_thread_id", FT_INTEGER|FF_READONLY, (intptr_t)0);
   setPrologFlag("gc_thread",    FT_BOOL,
 		!GD->options.nothreads &&
 		truePrologFlag(PLFLAG_GCTHREAD), PLFLAG_GCTHREAD);
@@ -1540,26 +1591,26 @@ initPrologFlags(void)
 #endif
   setPrologFlag("on_error", FT_ATOM, GD->options.on_error);
   setPrologFlag("on_warning", FT_ATOM, GD->options.on_warning);
-  setPrologFlag("break_level", FT_INTEGER|FF_READONLY, 0, 0);
+  setPrologFlag("break_level", FT_INTEGER|FF_READONLY, (intptr_t)0);
   setPrologFlag("user_flags", FT_ATOM, "silent");
   setPrologFlag("editor", FT_ATOM, "default");
   setPrologFlag("debugger_show_context", FT_BOOL, FALSE, 0);
   setPrologFlag("autoload",  FT_ATOM, "true");
   setPrologFlagMask(PLFLAG_AUTOLOAD);
-#ifndef O_GMP
+#ifndef O_BIGNUM
   setPrologFlag("max_integer",	   FT_INT64|FF_READONLY, PLMAXINT);
   setPrologFlag("min_integer",	   FT_INT64|FF_READONLY, PLMININT);
 #endif
-  setPrologFlag("max_tagged_integer", FT_INTEGER|FF_READONLY, PLMAXTAGGEDINT);
-  setPrologFlag("min_tagged_integer", FT_INTEGER|FF_READONLY, PLMINTAGGEDINT);
-#ifdef O_GMP
+  setPrologFlag("max_tagged_integer", FT_INTEGER|FF_READONLY, (intptr_t)PLMAXTAGGEDINT);
+  setPrologFlag("min_tagged_integer", FT_INTEGER|FF_READONLY, (intptr_t)PLMINTAGGEDINT);
+#ifdef O_BIGNUM
   setPrologFlag("bounded",	      FT_BOOL|FF_READONLY,	   FALSE, 0);
   setPrologFlag("prefer_rationals", FT_BOOL, O_PREFER_RATIONALS, PLFLAG_RATIONAL);
   setPrologFlag("rational_syntax",  FT_ATOM,
 		O_RATIONAL_SYNTAX == RAT_NATURAL ? "natural" :
-		                                   "compatibility");
+						   "compatibility");
 #ifdef __GNU_MP__
-  setPrologFlag("gmp_version",	   FT_INTEGER|FF_READONLY, __GNU_MP__);
+  setPrologFlag("gmp_version",	   FT_INTEGER|FF_READONLY, (intptr_t)__GNU_MP__);
 #endif
 #else
   setPrologFlag("bounded",		   FT_BOOL|FF_READONLY,	   TRUE, 0);
@@ -1568,9 +1619,9 @@ initPrologFlags(void)
     setPrologFlag("integer_rounding_function", FT_ATOM|FF_READONLY, "down");
   else
     setPrologFlag("integer_rounding_function", FT_ATOM|FF_READONLY, "toward_zero");
-  setPrologFlag("max_char_code", FT_INTEGER|FF_READONLY, PLMAXWCHAR);
+  setPrologFlag("max_char_code", FT_INTEGER|FF_READONLY, (intptr_t)UNICODE_MAX);
   setPrologFlag("max_arity", FT_ATOM|FF_READONLY, "unbounded");
-  setPrologFlag("max_procedure_arity", FT_INTEGER|FF_READONLY, MAXARITY);
+  setPrologFlag("max_procedure_arity", FT_INTEGER|FF_READONLY, (intptr_t)MAXARITY);
   setPrologFlag("answer_format", FT_ATOM, "~p");
   setPrologFlag("colon_sets_calling_context", FT_BOOL|FF_READONLY, TRUE, 0);
   setPrologFlag("character_escapes", FT_BOOL, TRUE, PLFLAG_CHARESCAPE);
@@ -1606,7 +1657,7 @@ initPrologFlags(void)
 		ALLOW_VARNAME_FUNCTOR);
   setPrologFlag("allow_dot_in_atom", FT_BOOL, FALSE,
 		PLFLAG_DOT_IN_ATOM);
-  setPrologFlag("toplevel_var_size", FT_INTEGER, 1000);
+  setPrologFlag("toplevel_var_size", FT_INTEGER, (intptr_t)1000);
   setPrologFlag("toplevel_print_anon", FT_BOOL, TRUE, 0);
   setPrologFlag("toplevel_name_variables", FT_BOOL, TRUE, 0);
   setPrologFlag("toplevel_prompt", FT_ATOM, "~m~d~l~! ?- ");
@@ -1628,18 +1679,20 @@ initPrologFlags(void)
 #ifdef __ANDROID__
   setPrologFlag("android", FT_BOOL|FF_READONLY, TRUE, 0);
 # ifdef __ANDROID_API__
-  setPrologFlag("android_api",FT_INTEGER|FF_READONLY, __ANDROID_API__);
+  setPrologFlag("android_api",FT_INTEGER|FF_READONLY, (intptr_t)__ANDROID_API__);
 # endif
 #endif
 #endif
 
-  setPrologFlag("encoding", FT_ATOM, stringAtom(encoding_to_atom(LD->encoding)));
+  setPrologFlag("encoding", FT_ATOM,
+		stringAtom(PL_encoding_to_atom(LD->encoding)));
 
   setPrologFlag("tty_control", FT_BOOL,
 		truePrologFlag(PLFLAG_TTY_CONTROL), PLFLAG_TTY_CONTROL);
   setPrologFlag("signals", FT_BOOL|FF_READONLY,
 		truePrologFlag(PLFLAG_SIGNALS), PLFLAG_SIGNALS);
   setPrologFlag("packs", FT_BOOL, GD->cmdline.packs, 0);
+  setPrologFlag("heartbeat", FT_INTEGER, (intptr_t)0);
 
 #if defined(__WINDOWS__) && defined(_DEBUG)
   setPrologFlag("kernel_compile_mode", FT_ATOM|FF_READONLY, "debug");
@@ -1669,12 +1722,15 @@ initPrologFlags(void)
   setVersionPrologFlag();
   setArgvPrologFlag("os_argv", GD->cmdline.os_argc,   GD->cmdline.os_argv);
   setArgvPrologFlag("argv",    GD->cmdline.appl_argc, GD->cmdline.appl_argv);
+#ifdef __SANITIZE_ADDRESS__
+  setPrologFlag("asan", FT_BOOL|FF_READONLY, TRUE, 0);
+#endif
 }
 
 
 static void
 setTmpDirPrologFlag(void)
- { char envbuf[MAXPATHLEN];
+ { char envbuf[PATH_MAX];
    char *td = NULL;
 
 #ifdef __unix__
@@ -1718,7 +1774,7 @@ setTZPrologFlag(void)
 #define timezone _timezone
 #endif
 
-  setPrologFlag("timezone", FT_INTEGER|FF_READONLY, timezone);
+  setPrologFlag("timezone", FT_INTEGER|FF_READONLY, (intptr_t)timezone);
 }
 
 
@@ -1751,10 +1807,10 @@ setVersionPrologFlag(void)
 
   if ( !PL_unify_term(t,
 		      PL_FUNCTOR_CHARS, PLNAME, 4,
-		        PL_INT, major,
-		        PL_INT, minor,
-		        PL_INT, patch,
-		        PL_TERM, o) )
+			PL_INT, major,
+			PL_INT, minor,
+			PL_INT, patch,
+			PL_TERM, o) )
     sysError("Could not set version");
 
   setPrologFlag("version_data", FF_READONLY|FT_TERM, t);
@@ -1774,12 +1830,12 @@ abi_version_dict(term_t dict)
 			  ATOM_built_in };
   term_t values = PL_new_term_refs(6);
 
-  return ( PL_unify_integer(values+0, PL_version(PL_VERSION_FLI)) &&
-	   PL_unify_integer(values+1, PL_version(PL_VERSION_REC)) &&
-	   PL_unify_integer(values+2, PL_version(PL_VERSION_QLF)) &&
-	   PL_unify_integer(values+3, PL_version(PL_VERSION_QLF_LOAD)) &&
-	   PL_unify_integer(values+4, PL_version(PL_VERSION_VM)) &&
-	   PL_unify_integer(values+5, PL_version(PL_VERSION_BUILT_IN)) &&
+  return ( PL_unify_integer(values+0, PL_version_info(PL_VERSION_FLI)) &&
+	   PL_unify_integer(values+1, PL_version_info(PL_VERSION_REC)) &&
+	   PL_unify_integer(values+2, PL_version_info(PL_VERSION_QLF)) &&
+	   PL_unify_integer(values+3, PL_version_info(PL_VERSION_QLF_LOAD)) &&
+	   PL_unify_integer(values+4, PL_version_info(PL_VERSION_VM)) &&
+	   PL_unify_integer(values+5, PL_version_info(PL_VERSION_BUILT_IN)) &&
 
 	   PL_put_dict(dict, ATOM_abi, 6, keys, values) );
 }
