@@ -3,8 +3,8 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2021, University of Amsterdam
-                              VU University Amsterdam
+    Copyright (c)  2011-2022, University of Amsterdam
+			      VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
     All rights reserved.
@@ -37,11 +37,7 @@
 
 #ifdef __WINDOWS__
 #include "windows/uxnt.h"
-#ifdef WIN64
-#include "config/win64.h"
-#else
-#include "config/win32.h"
-#endif
+#include "config/wincfg.h"
 #include <winsock2.h>
 #define CRLF_MAPPING 1
 #else
@@ -626,6 +622,7 @@ S__fillbuf(IOSTREAM *s)
   { char chr;
     ssize_t n;
 
+  again:
     n = (*s->functions->read)(s->handle, &chr, 1);
     if ( n == 1 )
     { c = char_to_int(chr);
@@ -635,7 +632,15 @@ S__fillbuf(IOSTREAM *s)
 	s->flags |= SIO_FEOF;
       return -1;
     } else
-    { S__seterror(s);
+    { if ( errno == EINTR )
+      { if ( PL_handle_signals() < 0 )
+	{ Sset_exception(s, PL_exception(0));
+	  errno = EPLEXCEPTION;
+	  return -1;
+	}
+	goto again;
+      }
+      S__seterror(s);
       return -1;
     }
   } else
@@ -662,6 +667,7 @@ S__fillbuf(IOSTREAM *s)
       len = s->bufsize;
     }
 
+  again2:
     n = (*s->functions->read)(s->handle, s->limitp, len);
     if ( n > 0 )
     { s->limitp += n;
@@ -679,6 +685,13 @@ S__fillbuf(IOSTREAM *s)
 	S__seterror(s);
 	return -1;
 #endif
+      } else if ( errno == EINTR )
+      { if ( PL_handle_signals() < 0 )
+	{ Sset_exception(s, PL_exception(0));
+	  errno = EPLEXCEPTION;
+	  return -1;
+	}
+	goto again2;
       } else
       {	S__seterror(s);
 	return -1;
@@ -862,6 +875,40 @@ reperror(int c, IOSTREAM *s)
   return -1;
 }
 
+static int
+put_usc2(int c, IOSTREAM *s, int be)
+{ if ( be )
+  { if ( put_byte(c>>8, s) < 0 ||
+	 put_byte(c&0xff, s) < 0 )
+      return -1;
+  } else
+  { if ( put_byte(c&0xff, s) < 0 ||
+	 put_byte(c>>8, s) < 0 )
+      return -1;
+  }
+
+  return 0;
+}
+
+static int
+put_utf16(int c, IOSTREAM *s, int be)
+{ if ( c > 0xffff )
+  { int c1, c2;
+
+    utf16_encode(c, &c1, &c2);
+    if ( put_usc2(c1, s, s->encoding == ENC_UTF16BE) < 0 ||
+	 put_usc2(c2, s, s->encoding == ENC_UTF16BE) < 0 )
+      return -1;
+  } else if ( IS_UTF16_SURROGATE(c) )
+  { if ( reperror(c, s) < 0 )
+      return -1;
+  } else
+  { if ( put_usc2(c, s, s->encoding == ENC_UTF16BE) < 0 )
+      return -1;
+  }
+
+  return 0;
+}
 
 
 static int
@@ -924,19 +971,25 @@ put_code(int c, IOSTREAM *s)
 
       break;
     }
-    case ENC_UNICODE_BE:
-      if ( put_byte(c>>8, s) < 0 )
-	return -1;
-      if ( put_byte(c&0xff, s) < 0 )
+    case ENC_UTF16BE:
+      if ( put_utf16(c, s, TRUE) < 0 )
 	return -1;
       break;
-    case ENC_UNICODE_LE:
-      if ( put_byte(c&0xff, s) < 0 )
-	return -1;
-      if ( put_byte(c>>8, s) < 0 )
+    case ENC_UTF16LE:
+      if ( put_utf16(c, s, FALSE) < 0 )
 	return -1;
       break;
     case ENC_WCHAR:
+#if SIZEOF_WCHAR_T == 2
+#ifdef WORDS_BIGENDIAN
+      if ( put_utf16(c, s, TRUE) < 0 )
+	return -1;
+#else
+      if ( put_utf16(c, s, FALSE) < 0 )
+	return -1;
+#endif
+      break;
+#else
     { pl_wchar_t chr = c;
       unsigned char *q = (unsigned char *)&chr;
       unsigned char *e = &q[sizeof(pl_wchar_t)];
@@ -948,6 +1001,7 @@ put_code(int c, IOSTREAM *s)
 
       break;
     }
+#endif
     case ENC_UNKNOWN:
       return -1;
   }
@@ -1005,15 +1059,19 @@ Scanrepresent(int c, IOSTREAM *s)
 	return 0;
       return -1;
     }
+    case ENC_UTF16BE:
+    case ENC_UTF16LE:
+#if SIZEOF_WCHAR_T > 2
+      if ( IS_UTF16_SURROGATE(c) )
+	return -1;
+#endif
+    /*FALLTHROUGH*/
     case ENC_WCHAR:
-      if ( sizeof(wchar_t) > 2 )
-	return 0;
+#if SIZEOF_WCHAR_T == 2
+      if ( IS_UTF16_SURROGATE(c) )
+	return -1;
+#endif
     /*FALLTHROUGH*/
-    case ENC_UNICODE_BE:
-    case ENC_UNICODE_LE:
-      if ( c <= 0xffff )
-	return 0;
-      return -1;
     case ENC_UTF8:
       return 0;
     default:
@@ -1023,6 +1081,49 @@ Scanrepresent(int c, IOSTREAM *s)
 }
 
 
+static int
+get_ucs2(IOSTREAM *s, int be)
+{ int c1, c2;
+
+  c1 = get_byte(s);
+  if ( c1 == EOF )
+    return -1;
+  c2 = get_byte(s);
+
+  if ( c2 == EOF )
+  { Sseterr(s, SIO_WARN, "EOF in unicode character");
+    return UTF8_MALFORMED_REPLACEMENT;
+  } else
+  { if ( be )
+      return (c1<<8)+c2;
+    else
+      return (c2<<8)+c1;
+  }
+}
+
+
+static int
+get_utf16(IOSTREAM *s, int be)
+{ int c = get_ucs2(s, be);
+
+  if ( IS_UTF16_LEAD(c) )
+  { int c2 = get_ucs2(s, be);
+
+    if ( c2 == EOF )
+    { Sseterr(s, SIO_WARN, "EOF in unicode character");
+      return UTF8_MALFORMED_REPLACEMENT;
+    } else if ( !IS_UTF16_TRAIL(c2) )
+    { Sseterr(s, SIO_WARN, "Illegal UTF-16 continuation");
+      return UTF8_MALFORMED_REPLACEMENT;
+    } else
+    { return utf16_decode(c, c2);
+    }
+  } else
+  { return c;
+  }
+}
+
+
 int
 Sgetcode(IOSTREAM *s)
 { int c;
@@ -1105,30 +1206,21 @@ retry:
       }
       break;
     }
-    case ENC_UNICODE_BE:
-    case ENC_UNICODE_LE:
-    { int c1, c2;
-
-      c1 = get_byte(s);
-      if ( c1 == EOF )
-      { c = -1;
-	goto out;
-      }
-      c2 = get_byte(s);
-
-      if ( c2 == EOF )
-      { Sseterr(s, SIO_WARN, "EOF in unicode character");
-	c = UTF8_MALFORMED_REPLACEMENT;
-      } else
-      { if ( s->encoding == ENC_UNICODE_BE )
-	  c = (c1<<8)+c2;
-	else
-	  c = (c2<<8)+c1;
-      }
-
+    case ENC_UTF16BE:
+      c = get_utf16(s, TRUE);
+      break;
+    case ENC_UTF16LE:
+      c = get_utf16(s, FALSE);
       break;
-    }
     case ENC_WCHAR:
+#if SIZEOF_WCHAR_T == 2
+#ifdef WORDS_BIGENDIAN
+      c = get_utf16(s, TRUE);
+#else
+      c = get_utf16(s, FALSE);
+#endif
+      break;
+#else
     { pl_wchar_t chr;
       char *p = (char*)&chr;
       size_t n;
@@ -1153,6 +1245,7 @@ retry:
       c = chr;
       break;
     }
+#endif
     default:
       assert(0);
       c = -1;
@@ -1163,7 +1256,7 @@ out:
   { switch(s->newline)
     { case SIO_NL_DETECT:
 	s->newline = SIO_NL_DOS;
-        /*FALLTHROUGH*/
+	/*FALLTHROUGH*/
       case SIO_NL_DOS:
 	goto retry;
     }
@@ -1420,8 +1513,8 @@ typedef struct
 
 static const bomdef bomdefs[] =
 { { ENC_UTF8,       3, "\357\273\277" }, /* 0xef, 0xbb, 0xbb */
-  { ENC_UNICODE_BE, 2, "\376\377" },	 /* 0xfe, 0xff */
-  { ENC_UNICODE_LE, 2, "\377\376" },	 /* 0xff, 0xfe */
+  { ENC_UTF16BE,    2, "\376\377" },	 /* 0xfe, 0xff */
+  { ENC_UTF16LE,    2, "\377\376" },	 /* 0xff, 0xfe */
   { ENC_UNKNOWN,    0, NULL }
 };
 
@@ -1463,8 +1556,8 @@ int
 SwriteBOM(IOSTREAM *s)
 { switch(s->encoding)
   { case ENC_UTF8:
-    case ENC_UNICODE_LE:
-    case ENC_UNICODE_BE:
+    case ENC_UTF16LE:
+    case ENC_UTF16BE:
     { if ( Sputcode(0xfeff, s) != -1 )
       { s->flags |= SIO_BOM;
 
@@ -1705,8 +1798,8 @@ Sunit_size(IOSTREAM *s)
     case ENC_ANSI:
     case ENC_UTF8:
       return 1;
-    case ENC_UNICODE_BE:
-    case ENC_UNICODE_LE:
+    case ENC_UTF16BE:
+    case ENC_UTF16LE:
       return 2;
     case ENC_WCHAR:
       return sizeof(wchar_t);
@@ -1806,7 +1899,7 @@ Sseek64(IOSTREAM *s, int64_t pos, int whence)
 
   if ( s->functions->seek64 )
     pos = (*s->functions->seek64)(s->handle, pos, whence);
-  else if ( pos <= LONG_MAX )
+  else if ( s->functions->seek && pos <= LONG_MAX && pos >= LONG_MIN )
     pos = (*s->functions->seek)(s->handle, (long)pos, whence);
   else
   { errno = EINVAL;
@@ -1907,6 +2000,11 @@ unallocStream(IOSTREAM *s)
   if ( s->context )
     Sdprintf("WARNING: unallocStream(): stream has context??\n");
 
+  if ( s->exception )
+  { PL_erase(s->exception);
+    s->exception = NULL;
+  }
+
   if ( !(s->flags & SIO_STATIC) )
     PL_free(s);
 }
@@ -1919,8 +2017,10 @@ object, which in turn calls the  ->unlink   which  may wish to close the
 associated stream.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-int
-Sclose(IOSTREAM *s)
+#define SIO_CLOSE_GC	0x4
+
+static int
+S__close(IOSTREAM *s, int flags)
 { int rval = 0;
 
   if ( s->magic != SIO_MAGIC )		/* already closed!? */
@@ -1938,7 +2038,19 @@ Sclose(IOSTREAM *s)
     return -1;
   }
 
-  SLOCK(s);
+  if ( (flags & (SIO_CLOSE_TRYLOCK|SIO_CLOSE_FORCE)) )
+  { if ( !STRYLOCK(s) )
+    { if ( (flags&SIO_CLOSE_FORCE) )
+      { PL_free(s->mutex);
+	s->mutex = NULL;
+      } else
+      { errno = EDEADLK;
+	return -1;
+      }
+    }
+  } else
+    SLOCK(s);
+
   s->flags |= SIO_CLOSING;
   rval = S__removebuf(s);
   if ( s->mbstate )
@@ -1965,8 +2077,8 @@ Sclose(IOSTREAM *s)
     if ( rval == 0 )
       rval = rc;
   }
-  if ( rval < 0 )
-    reportStreamError(s);
+  if ( rval < 0 && !(flags&SIO_CLOSE_GC) )
+    reportStreamError(s);		/* Cannot throw error from AGC */
   run_close_hooks(s);			/* deletes Prolog registration */
   s->magic = SIO_CMAGIC;
   SUNLOCK(s);
@@ -1983,6 +2095,16 @@ Sclose(IOSTREAM *s)
   return rval;
 }
 
+int
+Sclose(IOSTREAM *s)
+{ return S__close(s, 0);
+}
+
+int
+Sgcclose(IOSTREAM *s, int flags)
+{ return S__close(s, SIO_CLOSE_GC|flags);
+}
+
 
 		 /*******************************
 		 *	     STRING I/O		*
@@ -2018,12 +2140,16 @@ out:
 char *
 Sgets(char *buf)
 { char *s = Sfgets(buf, Slinesize, Sinput);
-  char *q;
 
   if ( s )				/* delete trailing \n */
-  { q = &s[strlen(s)];
-    if ( q > s && q[-1] == '\n' )
-      *--q = '\0';
+  { char *q;
+
+    for(q=s; q<s+Slinesize; q++)
+    { if ( *q == '\n' )
+      { *q = '\0';
+	break;
+      }
+    }
   }
 
   return s;
@@ -2098,7 +2224,9 @@ next_chr(const char **s, IOENC enc)
     }
     case ENC_WCHAR:
     { const wchar_t *w = (const wchar_t*)*s;
-      wint_t c = *w++;
+      int c;
+
+      w = get_wchar(w, &c);
       *s = (const char*)w;
       return c;
     }
@@ -2111,6 +2239,24 @@ next_chr(const char **s, IOENC enc)
 #define OUTCHR(s, c)	do { printed++; \
 			     if ( Sputcode((c), (s)) < 0 ) goto error; \
 			   } while(0)
+#define OUTFS() \
+	do \
+	{ if ( fs == fbuf ) \
+	  { while(fs < fe) \
+	    { int c = next_chr((const char**)&fs, enc); \
+	      OUTCHR(s, c); \
+	    } \
+	  } else \
+	  { for(;;) \
+	    { int c = next_chr((const char**)&fs, enc); \
+	      if ( c ) \
+		OUTCHR(s, c); \
+	      else \
+		break; \
+	    } \
+	  } \
+	} while(0)
+
 #define valdigit(c)	((c) - '0')
 #define A_LEFT	0			/* left-aligned field */
 #define A_RIGHT 1			/* right-aligned field */
@@ -2140,7 +2286,8 @@ typedef enum
 { INT_INT       = 0,
   INT_LONG      = 1,
   INT_LONG_LONG = 2,
-  INT_SIZE_T    = 3
+  INT_SIZE_T    = 3,
+  INT_PTRDIFF_T = 4
 } int_type;
 
 int
@@ -2224,6 +2371,10 @@ Svfprintf(IOSTREAM *s, const char *fm, va_list args)
 	    islong = INT_SIZE_T;
 	    fm++;
 	    break;
+	  case 't':
+	    islong = INT_PTRDIFF_T;
+	    fm++;
+	    break;
 	  case 'L':			/* %Ls: ISO-Latin-1 string */
 	    enc = ENC_ISO_LATIN_1;
 	    fm++;
@@ -2240,8 +2391,11 @@ Svfprintf(IOSTREAM *s, const char *fm, va_list args)
 
 	switch(*fm)
 	{ case 'c':
-	    *fe++ = va_arg(args, int);
+	  { int c = va_arg(args, int);
+	    fe = utf8_put_char(fe, c);
+	    enc = ENC_UTF8;
 	    break;
+	  }
 	  case 'p':
 	  { void *ptr = va_arg(args, void*);
 	    char fmbuf[8], *fp=fmbuf;
@@ -2268,15 +2422,18 @@ Svfprintf(IOSTREAM *s, const char *fm, va_list args)
 	    switch( islong )
 	    { case INT_INT:
 		vi = va_arg(args, int);
-	        break;
+		break;
 	      case INT_LONG:
 		vl = va_arg(args, long);
-	        break;
+		break;
 	      case INT_LONG_LONG:
-	        vll = va_arg(args, int64_t);
+		vll = va_arg(args, int64_t);
 		break;
 	      case INT_SIZE_T:
-	        vll = va_arg(args, size_t);
+		vll = va_arg(args, size_t);
+		break;
+	      case INT_PTRDIFF_T:
+		vll = va_arg(args, ptrdiff_t);
 		break;
 	      default:
 		assert(0);
@@ -2288,28 +2445,29 @@ Svfprintf(IOSTREAM *s, const char *fm, va_list args)
 	    switch( islong )
 	    { case INT_INT:
 		*fp++ = *fm;
-	        *fp   = '\0';
+		*fp   = '\0';
 		SNPRINTF3(fmbuf, vi);
 		break;
 	      case INT_LONG:
 		*fp++ = 'l';
-	        *fp++ = *fm;
+		*fp++ = *fm;
 		*fp   = '\0';
 		SNPRINTF3(fmbuf, vl);
 		break;
 	      case INT_LONG_LONG:
 	      case INT_SIZE_T:
+	      case INT_PTRDIFF_T:
 #ifdef __WINDOWS__
-	        *fp++ = 'I';		/* Synchronise with INT64_FORMAT! */
-	        *fp++ = '6';
+		*fp++ = 'I';		/* Synchronise with INT64_FORMAT! */
+		*fp++ = '6';
 		*fp++ = '4';
 #else
-	        *fp++ = 'l';
-	        *fp++ = 'l';
+		*fp++ = 'l';
+		*fp++ = 'l';
 #endif
-	        *fp++ = *fm;
-	        *fp   = '\0';
-	        SNPRINTF3(fmbuf, vll);
+		*fp++ = *fm;
+		*fp   = '\0';
+		SNPRINTF3(fmbuf, vll);
 		break;
 	    }
 
@@ -2344,21 +2502,26 @@ Svfprintf(IOSTREAM *s, const char *fm, va_list args)
 	  case 's':
 	    fs = va_arg(args, char *);
 	    if ( !fs )
-	      fs = "(null)";
+	    { fs = "(null)";
+	      enc = ENC_ISO_LATIN_1;
+	    }
 	    break;
 	}
 
+	/* Now `fs` is either the result of some sub-formatting or the
+	   argument of `%s`.  In the latter case it is by definition
+	   0-terminated.  In the former case it may hold 0-bytes, e.g.,
+	   `%c` using a 0 argument.  `fe` points at the end.
+	*/
+
 	if ( has_arg1 )			/* aligned field */
 	{ if ( fs == fbuf )
 	    *fe = '\0';
 
 	  if ( align == A_LEFT )
-	  { int w = 0;
-	    while(*fs)
-	    { int c = next_chr((const char**)&fs, enc);
-	      OUTCHR(s, c);
-	      w++;
-	    }
+	  { int printed0 = printed;
+	    OUTFS();
+	    int w = printed-printed0;
 	    while(w < arg1)
 	    { OUTCHR(s, pad);
 	      w++;
@@ -2394,21 +2557,10 @@ Svfprintf(IOSTREAM *s, const char *fm, va_list args)
 		w--;
 	      }
 	    }
-	    while(*fs)
-	    { int c = next_chr((const char**)&fs, enc);
-	      OUTCHR(s, c);
-	    }
+	    OUTFS();
 	  }
 	} else
-	{ if ( fs == fbuf )		/* unaligned field, just output */
-	  { while(fs < fe)
-	      OUTCHR(s, *fs++);
-	  } else
-	  { while(*fs)
-	    { int c = next_chr((const char**)&fs, enc);
-	      OUTCHR(s, c);
-	    }
-	  }
+	{ OUTFS();
 	}
 	fm++;
 	if ( fs_malloced )
@@ -2478,7 +2630,7 @@ Svsprintf(char *buf, const char *fm, va_list args)
   s.limitp    = (char *)(~0L);
   s.buffer    = buf;
   s.flags     = SIO_FBUF|SIO_OUTPUT;
-  s.encoding  = ENC_ISO_LATIN_1;
+  s.encoding  = ENC_UTF8;
 
   if ( (rval = Svfprintf(&s, fm, args)) >= 0 )
     *s.bufp = '\0';
@@ -2982,7 +3134,7 @@ Scontrol_file(void *handle, int action, void *arg)
 
       if ( fstat(fd, &buf) == 0 )
       {	*rval = buf.st_size;
-        return 0;
+	return 0;
       }
       return -1;
     }
@@ -3204,23 +3356,23 @@ Sopen_file(const char *path, const char *how)
     { case 'b':				/* binary */
 	flags &= ~SIO_TEXT;
 	enc = ENC_OCTET;
-        break;
+	break;
       case 'r':				/* no record */
 	flags &= ~SIO_RECORDPOS;
-        break;
+	break;
       case 'L':				/* lock r: read, w: write */
 	wait = FALSE;
-        /*FALLTHROUGH*/
+	/*FALLTHROUGH*/
       case 'l':				/* lock r: read, w: write */
 	if ( *++how == 'r' )
 	  lock = lread;
-        else if ( *how == 'w' )
+	else if ( *how == 'w' )
 	  lock = lwrite;
-        else
+	else
 	{ errno = EINVAL;
 	  return NULL;
 	}
-        break;
+	break;
       case 'm':
 	if ( get_mode(how+1, &mode) )
 	{ how += 3;
@@ -3231,7 +3383,7 @@ Sopen_file(const char *path, const char *how)
 	}
       default:
 	errno = EINVAL;
-        return NULL;
+	return NULL;
     }
   }
 
@@ -3329,6 +3481,36 @@ Sopen_file(const char *path, const char *how)
 }
 
 
+IOSTREAM *
+Sopen_iri_or_file(const char *path, const char *how)
+{ int sl;
+  IOSTREAM *s;
+
+  if ( (sl=file_name_is_iri(path)) )
+  { atom_t mname = how[0] == 'r' ? ATOM_read : ATOM_write;
+    const char *m;
+
+    if ( !iri_hook(path, IRI_OPEN, mname, 0, &s) )
+      return NULL;
+
+    for(m=&how[1]; *m; m++)
+    { if ( *m == 'b' )
+      { clear(s, SIO_TEXT);
+	s->encoding = ENC_OCTET;
+	s->newline  = SIO_NL_POSIX;
+      } else if ( *m == 'r' )
+      { clear(s, SIO_RECORDPOS);
+	s->position = NULL;
+      }
+    }
+  } else
+  { s = Sopen_file(path, how);
+  }
+
+  return s;
+}
+
+
 IOSTREAM *
 Sfdopen(int fd, const char *type)
 { intptr_t lfd;
@@ -3422,12 +3604,8 @@ Swinsock(IOSTREAM *s)
 
 #ifdef HAVE_POPEN
 #ifdef __WINDOWS__
-#include "windows/popen.c"
-
-#undef popen
-#undef pclose
-#define popen(cmd, how) pt_popen(cmd, how)
-#define pclose(fd)	pt_pclose(fd)
+#define popen _popen
+#define pclose _pclose
 #endif
 
 static ssize_t
@@ -3512,7 +3690,7 @@ Sopen_pipe(const char *command, const char *type)
       { case 'r':
 	  flags |= SIO_INPUT;
 	  break;
-        case 'w':
+	case 'w':
 	  flags |= SIO_OUTPUT;
 	  break;
 	case 'b':
@@ -3769,6 +3947,7 @@ Sopenmem(char **bufp, size_t *sizep, const char *mode)
 { memfile *mf = malloc(sizeof(memfile));
   int flags = SIO_FBUF|SIO_RECORDPOS|SIO_NOMUTEX|SIO_TEXT;
   size_t size;
+  IOSTREAM *s;
 
   if ( !mf )
   { errno = ENOMEM;
@@ -3804,10 +3983,10 @@ Sopenmem(char **bufp, size_t *sizep, const char *mode)
 	break;
       case 'b':
 	flags &= ~SIO_TEXT;
-        break;
+	break;
       case 'F':
 	mf->free_on_close = TRUE;
-        break;
+	break;
       default:
 	free(mf);
 	errno = EINVAL;
@@ -3818,7 +3997,10 @@ Sopenmem(char **bufp, size_t *sizep, const char *mode)
   mf->sizep	= sizep;
   mf->here      = 0;
 
-  return Snew(mf, flags, &Smemfunctions);
+  if ( (s=Snew(mf, flags, &Smemfunctions)) )
+    s->newline = SIO_NL_POSIX;
+
+  return s;
 }
 
 		 /*******************************
@@ -3933,11 +4115,6 @@ IOFUNCTIONS Sbufferfunctions =
   NULL  /* close */
 };
 
-/*
-   FIXME: this should probably use UTF-8 encoding rather than
-   ENC_ISO_LATIN_1.
-*/
-
 static IOSTREAM *
 Sopen_buffer(IOSTREAM *s, char *buf, size_t size)
 {
@@ -3947,7 +4124,7 @@ Sopen_buffer(IOSTREAM *s, char *buf, size_t size)
   s->buffer    = buf;
   s->flags     = SIO_FBUF|SIO_OUTPUT;
   s->functions = &Sbufferfunctions;
-  s->encoding  = ENC_ISO_LATIN_1;
+  s->encoding  = ENC_UTF8;
   s->magic     = SIO_MAGIC;
 
   return s;
@@ -3963,12 +4140,12 @@ Sclose_buffer(IOSTREAM *s)
 		 *	 STANDARD HANDLES	*
 		 *******************************/
 
-#define STDIO(n, f) { NULL, NULL, NULL, NULL, \
-		      EOF, SIO_MAGIC, 0, f, {0, 0, 0}, NULL, \
-		      (void *)(n), &Sttyfunctions, \
-		      -1, \
-		      ENC_ISO_LATIN_1 \
-		    }
+#define STDIO(n, f) \
+	{ .bufp = NULL, .limitp = NULL, .buffer = NULL, .unbuffer = NULL, \
+	  .lastc = EOF, .magic = SIO_MAGIC, .flags = (f),		  \
+	  .handle = (void *)(n), .functions = &Sttyfunctions,		  \
+	  .timeout = -1, .encoding = ENC_ISO_LATIN_1			  \
+	}
 
 #define SIO_STDIO (SIO_FILE|SIO_STATIC|SIO_NOCLOSE|SIO_ISATTY|SIO_TEXT)
 #define STDIO_STREAMS \
@@ -4016,10 +4193,6 @@ SinitStreams(void)
       s->newline = SIO_NL_DOS;
 #endif
     }
-
-#ifdef __WINDOWS__
-    pt_init();				/* init popen() issues */
-#endif
   }
 }
 
@@ -4108,6 +4281,10 @@ Scleanup(void)
   for(i=0; i<=2; i++)
   { IOSTREAM *s = &S__iob[i];
 
+    if ( s->mbstate )
+      free(s->mbstate);
+    if ( s->locale )
+      releaseLocale(s->locale);
     s->bufp = s->buffer;		/* avoid actual flush */
     S__removebuf(s);
 
