@@ -4,7 +4,7 @@
     E-mail:        keri.harris@securitease.com
     WWW:           http://www.swi-prolog.org
     Copyright (c)  2011-2021, University of Amsterdam
-                              VU University Amsterdam
+			      VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
     All rights reserved.
@@ -108,12 +108,14 @@ const debug_topic debug_topics[] =
   DEBUG_TOPIC(MSG_DETERMINISM),
 						/* Parser */
   DEBUG_TOPIC(MSG_READ_TOKEN),
+  DEBUG_TOPIC(MSG_LOCALE),
 
   DEBUG_TOPIC(MSG_CONTINUE),
 
   DEBUG_TOPIC(MSG_CLEANUP_THREAD),
   DEBUG_TOPIC(MSG_INITIALISE),
   DEBUG_TOPIC(MSG_COPYTERM),
+  DEBUG_TOPIC(MSG_PRED_COUNT),
 
   DEBUG_TOPIC(MSG_BACKTRACK),
 
@@ -178,6 +180,7 @@ const debug_topic debug_topics[] =
   DEBUG_TOPIC(MSG_RECONSULT_PRED),
   DEBUG_TOPIC(MSG_RECONSULT_CLAUSE),
   DEBUG_TOPIC(MSG_RECONSULT_MODULE),
+  DEBUG_TOPIC(MSG_AUTOLOAD),
 
   DEBUG_TOPIC(MSG_TRIE_PUT_TERM),
   DEBUG_TOPIC(MSG_TRIE_GC),
@@ -220,7 +223,10 @@ const debug_topic debug_topics[] =
   DEBUG_TOPIC(TABLING_NO_SIMPLIFY),
   DEBUG_TOPIC(TABLING_NO_AC),
 
+  DEBUG_TOPIC(MSG_GVAR_LAZY),
+
   DEBUG_TOPIC(MSG_THREAD_WAIT),
+  DEBUG_TOPIC(MSG_THREAD_SIGNAL),
 
   DEBUG_TOPIC(MSG_TRANSACTION),
   DEBUG_TOPIC(MSG_COMMIT),
@@ -236,6 +242,10 @@ const debug_topic debug_topics[] =
 };
 
 
+#ifdef __WINDOWS__
+#define strncasecmp(s1, s2, n) _strnicmp(s1, s2, n)
+#endif
+
 static const debug_topic *
 get_next_debug_topic(const char *topic, const debug_topic *dt)
 { int cmplen = strlen(topic) + 1;
@@ -291,11 +301,11 @@ prolog_debug_topic(const char *topic, int flag)
       success = TRUE;
 
       if ( code <= DBG_LEVEL9 )
-        GD->debug_level = code;
+	GD->debug_level = code;
       else if (flag)
-        set_bit(GD->debug_topics, code);
+	set_bit(GD->debug_topics, code);
       else
-        clear_bit(GD->debug_topics, code);
+	clear_bit(GD->debug_topics, code);
     }
   }
 
@@ -338,7 +348,7 @@ prolog_debug_from_string(const char *spec, int flag)
       buf[end-spec] = EOS;
       if ( !prolog_debug_topic(buf, flag) )
       { if (quiet)
-          return FALSE;
+	  return FALSE;
 	Sdprintf("ERROR: Unknown debug topic: %s\n", buf);
 	PL_halt(1);
       }
@@ -416,4 +426,3 @@ BeginPredDefs(debug)
   PRED_DEF("prolog_debug", 1, pl_prolog_debug, 0)
   PRED_DEF("prolog_nodebug", 1, pl_prolog_nodebug, 0)
 EndPredDefs
-
