@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2013-2018, VU University Amsterdam
+    Copyright (c)  2013-2021, VU University Amsterdam
 			      CWI, Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -38,25 +39,38 @@
 
 #define DICT_SORTED	0x1		/* Sort dict entries */
 
-COMMON(int)	PL_is_dict(term_t t);
-COMMON(int)	PL_for_dict(term_t dict,
-			   int (*func)(term_t key,
-				       term_t value,
-				       int last,
-				       void *closure),
-			   void *closure,
-			   int flags);
+#if USE_LD_MACROS
+#define	pl_for_dict(dict, func, closure, flags)	LDFUNC(pl_for_dict, dict, func, closure, flags)
+#define	dict_order(dict, dupl)			LDFUNC(dict_order, dict, dupl)
+#define	dict_order_term_refs(av, indexes, cnt)	LDFUNC(dict_order_term_refs, av, indexes, cnt)
+#define	dict_lookup_ptr(dict, name)		LDFUNC(dict_lookup_ptr, dict, name)
+#endif /*USE_LD_MACROS*/
 
-COMMON(functor_t) dict_functor(int pairs);
-COMMON(int)	  dict_order(Word dict, int ex ARG_LD);
-COMMON(int)	  dict_order_term_refs(term_t *av, int *indexes, int cnt ARG_LD);
-COMMON(Word)	  dict_lookup_ptr(word dict, word name ARG_LD);
-COMMON(int)	  resortDictsInClause(Clause clause);
-COMMON(void)	  resortDictsInTerm(term_t t);
+#define LDFUNC_DECLARATIONS
 
-#define termIsDict(w) termIsDict__LD(w PASS_LD)
+int	PL_is_dict(term_t t);
+int	pl_for_dict(term_t dict,
+		   int LDFUNCP (*func)(term_t key,
+			       term_t value,
+			       int last,
+			       void *closure),
+		   void *closure,
+		   int flags);
+
+functor_t dict_functor(int pairs);
+int	  dict_order(Word dict, Word dupl);
+int	  dict_order_term_refs(term_t *av, int *indexes, int cnt);
+Word	  dict_lookup_ptr(word dict, word name);
+int	  resortDictsInClause(Clause clause);
+void	  resortDictsInTerm(term_t t);
+
+#undef LDFUNC_DECLARATIONS
+
+#define PL_for_dict(dict, funcname, closure, flags) pl_for_dict(dict, LDFUNC_REF(funcname), closure, flags)
+
+#define termIsDict(w) LDFUNC(termIsDict, w)
 static inline int
-termIsDict__LD(word w ARG_LD)
+termIsDict(DECL_LD word w)
 { Functor f = valueTerm(w);
   FunctorDef fd = valueFunctor(f->definition);
 
