@@ -3,9 +3,10 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2020, University of Amsterdam
+    Copyright (c)  1985-2022, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
+			      SWI-Prolog Solutions b.v
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -35,8 +36,26 @@
 */
 
 /*#define O_DEBUG 1*/			/* include crash/0 */
-#include "pl-incl.h"
+#include "pl-ext.h"
+#include "pl-prims.h"
+#include "pl-sys.h"
+#include "pl-pro.h"
+#include "pl-write.h"
+#include "pl-read.h"
+#include "pl-funct.h"
+#include "pl-proc.h"
+#include "pl-trace.h"
+#include "pl-dwim.h"
+#include "pl-modul.h"
+#include "pl-gc.h"
+#include "pl-flag.h"
+#include "pl-xterm.h"
+#include "pl-supervisor.h"
+#include "pl-fli.h"
+#include "pl-nt.h"
 #include "os/pl-ctype.h"
+#include "os/pl-fmt.h"
+#include "os/pl-prologflag.h"
 
 #if O_DEBUG
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -151,10 +170,6 @@ static const PL_extension foreigns[] = {
 #endif
   FRG("context_module",		1, pl_context_module,	     META),
 
-#if O_STRING
-  FRG("sub_string",		5, pl_sub_string,	     NDET),
-#endif /* O_STRING */
-
   FRG("format",			2, pl_format,		     META),
 #ifdef O_DEBUG
   FRG("$check_definition",	1, pl_check_definition,      META),
@@ -189,7 +204,6 @@ static const PL_extension foreigns[] = {
 #ifdef O_PLMT
   FRG("thread_create",		3, pl_thread_create,	 META|ISO),
   FRG("thread_exit",		1, pl_thread_exit,		0),
-  FRG("thread_signal",		2, pl_thread_signal,	 META|ISO),
   FRG("open_xterm",		5, pl_open_xterm,		0),
 #endif
 
@@ -334,6 +348,7 @@ registerBuiltins(const PL_extension *f)
       if ( f->flags & PL_FA_VARARGS )	       set(def, P_VARARG);
       if ( f->flags & PL_FA_CREF )	       set(def, P_FOREIGN_CREF);
       if ( f->flags & PL_FA_ISO )	       set(def, P_ISO);
+      if ( f->flags & PL_FA_SIG_ATOMIC )       set(def, P_SIG_ATOMIC);
 
       def->impl.foreign.function = f->function;
       createForeignSupervisor(def, f->function);
@@ -376,6 +391,7 @@ DECL_PLIST(wic);
 DECL_PLIST(attvar);
 DECL_PLIST(gvar);
 DECL_PLIST(win);
+DECL_PLIST(apple);
 DECL_PLIST(file);
 DECL_PLIST(files);
 DECL_PLIST(glob);
@@ -405,13 +421,20 @@ DECL_PLIST(zip);
 DECL_PLIST(cbtrace);
 DECL_PLIST(wrap);
 DECL_PLIST(event);
+DECL_PLIST(transaction);
+DECL_PLIST(undo);
+DECL_PLIST(error);
+DECL_PLIST(coverage);
+#ifdef __EMSCRIPTEN__
+DECL_PLIST(wasm);
+#endif
 
 void
 initBuildIns(void)
 { ExtensionCell ecell;
   Module m = MODULE_system;
 
-  GD->procedures.dirty = newHTable(32);
+  initProcedures();
 
   registerBuiltins(foreigns);
   REG_PLIST(alloc);
@@ -462,6 +485,9 @@ initBuildIns(void)
   REG_PLIST(win);
   REG_PLIST(dde);
 #endif
+#ifdef __APPLE__
+  REG_PLIST(apple);
+#endif
 #ifdef O_LOCALE
   REG_PLIST(locale);
 #endif
@@ -475,6 +501,15 @@ initBuildIns(void)
   REG_PLIST(cbtrace);
   REG_PLIST(wrap);
   REG_PLIST(event);
+  REG_PLIST(transaction);
+  REG_PLIST(undo);
+  REG_PLIST(error);
+#ifdef O_COVERAGE
+  REG_PLIST(coverage);
+#endif
+#ifdef __EMSCRIPTEN__
+  REG_PLIST(wasm);
+#endif
 
 #define LOOKUPPROC(name) \
 	{ GD->procedures.name = lookupProcedure(FUNCTOR_ ## name, m); \
@@ -491,6 +526,7 @@ initBuildIns(void)
   LOOKUPPROC(is2);
   LOOKUPPROC(strict_equal2);
   LOOKUPPROC(not_strict_equal2);
+  LOOKUPPROC(arg3);
   LOOKUPPROC(print_message2);
   LOOKUPPROC(dcall1);
   LOOKUPPROC(setup_call_catcher_cleanup4);
@@ -500,6 +536,8 @@ initBuildIns(void)
 #ifdef O_ATTVAR
   LOOKUPPROC(dwakeup1);
 #endif
+  GD->procedures.heartbeat0 = lookupProcedure(FUNCTOR_heartbeat0,
+					      PL_new_module(PL_new_atom("prolog")));
   PROCEDURE_exception_hook4  =
 	PL_predicate("prolog_exception_hook", 4, "user");
   PROCEDURE_tune_gc3 =
@@ -528,6 +566,8 @@ initBuildIns(void)
 #ifdef O_PLMT
   PL_meta_predicate(PL_predicate("thread_create",    3, "system"), "0?+");
   PL_meta_predicate(PL_predicate("thread_signal",    2, "system"), "+0");
+  PL_meta_predicate(PL_predicate("thread_wait",	     2, "system"), "0:");
+  PL_meta_predicate(PL_predicate("thread_update",    2, "system"), "0:");
 #endif
   PL_meta_predicate(PL_predicate("thread_idle",      2, "system"), "0+");
   PL_meta_predicate(PL_predicate("prolog_frame_attribute", 3, "system"), "++:");
@@ -539,7 +579,7 @@ initBuildIns(void)
   PL_meta_predicate(PL_predicate("prolog_listen",    3, "system"), "+:+");
   PL_meta_predicate(PL_predicate("prolog_unlisten",  2, "system"), "+:");
   PL_meta_predicate(PL_predicate("with_tty_raw",     1, "system"), "0");
-  PL_meta_predicate(PL_predicate("$sig_atomic",      1, "system"), "0");
+  PL_meta_predicate(PL_predicate("sig_atomic",       1, "system"), "0");
 
   for( ecell = ext_head; ecell; ecell = ecell->next )
     bindExtensions(ecell->module, ecell->extensions);
