@@ -4,7 +4,7 @@
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
     Copyright (c)  1985-2020, University of Amsterdam
-                              VU University Amsterdam
+			      VU University Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -174,7 +174,10 @@ indexOfWord(DECL_LD word w)
       case TAG_INTEGER:
 	if ( storage(w) == STG_INLINE )
 	  break;
-      /*FALLTHROUGH*/
+#if O_BIGNUM
+	return bignum_index(addressIndirect(w));
+#endif
+	/*FALLTHROUGH for int64 and no BIGNUM */
       case TAG_STRING:
       case TAG_FLOAT:
       { Word p = addressIndirect(w);
@@ -715,7 +718,6 @@ unallocClauseIndexTableEntries(ClauseIndex ci)
     }
   }
 
-  ATOMIC_INC(&GD->statistics.indexes.destroyed);
   freeHeap(ci->entries, ci->buckets * sizeof(struct clause_bucket));
 }
 
@@ -724,6 +726,7 @@ void
 unallocClauseIndexTable(ClauseIndex ci)
 { unallocClauseIndexTableEntries(ci);
   freeHeap(ci, sizeof(struct clause_index));
+  ATOMIC_INC(&GD->statistics.indexes.destroyed);
 }
 
 
@@ -1466,6 +1469,12 @@ deleteIndexes(ClauseList clist, int isnew)
 }
 
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Only called from destroyDefinition(), which is only called when removing
+a temporary module or during  final  cleanup.   So,  there  are  no more
+references.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
 void
 deleteIndexesDefinition(Definition def)
 { ClauseList clist = &def->impl.clauses;
@@ -1482,6 +1491,9 @@ deleteIndexesDefinition(Definition def)
 
       deleteIndexP(def, clist, cip);
     }
+
+    freeHeap(cip0, 0);
+    clist->clause_indexes = NULL;
   }
 }
 
@@ -1513,7 +1525,7 @@ addClauseToIndex(ClauseIndex ci, Clause cl, ClauseRef where)
       case H_RFUNCTOR:
       case H_RLIST:
 	pc = stepPC(pc);
-        argKey(pc, 0, &arg1key);
+	argKey(pc, 0, &arg1key);
     }
   }
 
@@ -1745,7 +1757,7 @@ cmp_indexes(const void *p1, const void *p2)
     return -1;
 
   return ci1->speedup < ci2->speedup ?  1 :
-         ci1->speedup > ci2->speedup ? -1 : 0;
+	 ci1->speedup > ci2->speedup ? -1 : 0;
 }
 
 
@@ -1870,7 +1882,6 @@ insertIndex(Definition def, ClauseList clist, ClauseIndex ci)
   if ( (ocip=clist->clause_indexes) )
   { ClauseIndex *cip = ocip;
     ClauseIndex *ncip;
-    int dead = 0;
 
     for(; *cip; cip++)
     { if ( ISDEADCI(*cip) )
@@ -1878,7 +1889,6 @@ insertIndex(Definition def, ClauseList clist, ClauseIndex ci)
 	if ( isSortedIndexes(ocip) )
 	  return;
 	*cip = DEAD_INDEX;
-	dead++;
       }
     }
 
@@ -2366,7 +2376,7 @@ skipToTerm(Clause clause, const iarg_t *position)
       }
       case I_CHP:
 	pc = stepPC(pc);
-        goto again;
+	goto again;
 #if defined(O_DEBUG) || defined(O_MAINTENANCE)
       case H_FIRSTVAR:
       case H_VAR:
@@ -2421,7 +2431,6 @@ assess_scan_clauses(ClauseList clist, size_t arity,
   ClauseRef cref;
   int i;
   bit_vector *ai = alloca(sizeof_bitvector(arity));
-  int ac = 0;
   int kp[MAXINDEXARG+1];			/* key-arg positions */
   int nk = 0;					/* number of key args */
   int *kpp;
@@ -2435,7 +2444,6 @@ assess_scan_clauses(ClauseList clist, size_t arity,
     for(j=0; a->args[j]; j++)
     { if ( !true_bit(ai, a->args[j]-1) )
       { set_bit(ai, a->args[j]-1);
-	ac++;
       }
     }
   }
@@ -2665,7 +2673,7 @@ bestHash(DECL_LD Word av, size_t ac, ClauseList clist, float min_speedup,
 	hints->ln_buckets = MSB(nbest->size);
 	hints->speedup    = nbest->speedup;
 
-        free_keys_in_assessment_set(&aset);
+	free_keys_in_assessment_set(&aset);
 	free_assessment_set(&aset);
 	return TRUE;
       }
@@ -2703,7 +2711,7 @@ Where is one of
 
   - Simple index		single(ArgN)
   - Multi-argument index	multi([Arg1,Arg2,...])
-  - Deep index		        deep([Arg1,Arg2,...])
+  - Deep index			deep([Arg1,Arg2,...])
 
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
@@ -2805,7 +2813,7 @@ unify_clause_index(term_t t, ClauseIndex ci)
 			 PL_FUNCTOR, FUNCTOR_hash4,
 			   PL_INT, (int)ci->buckets,
 			   PL_DOUBLE, (double)ci->speedup,
-		           PL_INT64, (int64_t)sizeofClauseIndex(ci),
+			   PL_INT64, (int64_t)sizeofClauseIndex(ci),
 			   PL_BOOL, ci->is_list);
 }
 
