@@ -3,8 +3,10 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1996-2015, University of Amsterdam
+    Copyright (c)  1996-2022, University of Amsterdam
                               VU University Amsterdam
+			      CWI, Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -36,24 +38,15 @@
 #define _UNICODE 1
 #define UNICODE 1
 
-#ifdef __MINGW32__
 #include <winsock2.h>
 #include <windows.h>
-#endif
+#include "config/wincfg.h"
 
-#ifdef WIN64
-#include "config/win64.h"
-#else
-#include "config/win32.h"
-#endif
-
-#define PL_ARITY_AS_SIZE 1
 #include <tchar.h>
 #include <malloc.h>
 #include <stdio.h>
 #include "os/SWI-Stream.h"
 #include "SWI-Prolog.h"
-#include <windows.h>
 #include <ctype.h>
 #include "win32/console/console.h"
 #include <signal.h>
@@ -61,7 +54,7 @@
 #include <pthread.h>
 #endif
 
-#include "os/pl-utf8.c"			/* we're not in the libpl.dll module */
+#include "os/pl-utf8.c"			/* we're not in the libswipl.dll module */
 
 #ifndef streq
 #define streq(s,q) (strcmp((s), (q)) == 0)
@@ -76,8 +69,8 @@ Main program for running SWI-Prolog from   a window. The window provides
 X11-xterm like features: scrollback for a   predefined  number of lines,
 cut/paste and the GNU readline library for command-line editing.
 
-This module combines swipl.dll and plterm.dll  with some glue to produce
-the final executable swipl-win.exe.
+This module combines libswipl.dll  and  plterm.dll   with  some  glue to
+produce the final executable swipl-win.exe.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 __declspec(dllexport)	rlc_console	PL_current_console(void);
@@ -238,7 +231,7 @@ Srlc_write(void *handle, char *buffer, size_t size)
     memcpy(buf, buffer+n, i);
     for(; i<sizeof(TCHAR); i++)
       buf[i] = '?';
-    rlc_write(c, (TCHAR*)buffer, 1);
+    rlc_write(c, (TCHAR*)buf, 1);
 
     return size;
   }
@@ -247,22 +240,23 @@ Srlc_write(void *handle, char *buffer, size_t size)
 }
 
 
+
 static int
 Srlc_close(void *handle)
 { rlc_console c = handle;
   uintptr_t v;
   int closed = 0;
 
-  if ( rlc_get(handle, RLC_PROLOG_INPUT, &v) && v &&
-       ((IOSTREAM *)v)->flags && SIO_CLOSING )
+#define ison(p,f) (((p)->flags)&(f))
+#define IS_CLOSING(v) ison((IOSTREAM *)v, SIO_CLOSING)
+
+  if ( rlc_get(handle, RLC_PROLOG_INPUT, &v) && v && IS_CLOSING(v) )
   { rlc_set(handle, RLC_PROLOG_INPUT, 0L, NULL);
     closed++;
-  } else if ( rlc_get(handle, RLC_PROLOG_OUTPUT, &v) && v &&
-	      ((IOSTREAM *)v)->flags && SIO_CLOSING )
+  } else if ( rlc_get(handle, RLC_PROLOG_OUTPUT, &v) && v && IS_CLOSING(v) )
   { rlc_set(handle, RLC_PROLOG_OUTPUT, 0L, NULL);
     closed++;
-  } else if ( rlc_get(handle, RLC_PROLOG_ERROR, &v) && v &&
-	      ((IOSTREAM *)v)->flags && SIO_CLOSING )
+  } else if ( rlc_get(handle, RLC_PROLOG_ERROR, &v) && v && IS_CLOSING(v) )
   { rlc_set(handle, RLC_PROLOG_ERROR, 0L, NULL);
   }
 
@@ -364,7 +358,7 @@ pl_win_open_console(term_t title, term_t input, term_t output, term_t error,
   size_t len;
 
   memset(&attr, 0, sizeof(attr));
-  if ( !PL_get_wchars(title, &len, &s, CVT_ALL|BUF_RING|CVT_EXCEPTION) )
+  if ( !PL_get_wchars(title, &len, &s, CVT_ALL|BUF_STACK|CVT_EXCEPTION) )
     return FALSE;
   attr.title = (const TCHAR*) s;
 
@@ -594,7 +588,7 @@ get_chars_arg_ex(int a, term_t t, TCHAR **v)
 { term_t arg = PL_new_term_ref();
 
   if ( PL_get_arg(a, t, arg) &&
-       PL_get_wchars(arg, NULL, v, CVT_ALL|BUF_RING|CVT_EXCEPTION) )
+       PL_get_wchars(arg, NULL, v, CVT_ALL|BUF_STACK|CVT_EXCEPTION) )
     return TRUE;
 
   return FALSE;
@@ -816,8 +810,8 @@ HiddenFrameClass()
   HINSTANCE instance = rlc_hinstance();
 
   if ( !winclassname[0] )
-  { snwprintf(winclassname, sizeof(winclassname)/sizeof(TCHAR),
-	      _T("SWI-Prolog-hidden-win%d"), instance);
+  { _snwprintf(winclassname, sizeof(winclassname)/sizeof(TCHAR),
+	       _T("SWI-Prolog-hidden-win%d"), (int)(intptr_t)instance);
 
     wndClass.style		= 0;
     wndClass.lpfnWndProc	= (LPVOID) pl_wnd_proc;
@@ -871,6 +865,7 @@ capture them in the application and tell   Prolog to print the stack and
 abort.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
+#if !defined(O_DEBUG) && !defined(_DEBUG)
 static void
 exit_immediately(rlc_console c, int sig)
 { _exit(1);
@@ -888,11 +883,11 @@ fatalSignal(int sig)
   }
 }
 
-
 static void
 initSignals()
 { signal(SIGABRT, fatalSignal);
 }
+#endif
 
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -989,15 +984,67 @@ set_window_title(rlc_console c)
     *o++ = *s++;
   *o = 0;
 
-  snwprintf(title, sizeof(title)/sizeof(TCHAR),
-	    _T("SWI-Prolog (%s%sversion %d.%d.%d%s%s)"),
-	    w64, mt, major, minor, patch,
-	    wtag[0] ? _T("-") : _T(""), wtag);
+  _snwprintf(title, sizeof(title)/sizeof(TCHAR),
+	     _T("SWI-Prolog (%s%sversion %d.%d.%d%s%s)"),
+	     w64, mt, major, minor, patch,
+	     wtag[0] ? _T("-") : _T(""), wtag);
 
   rlc_title(c, title, NULL, 0);
 }
 
 
+static int
+get_color_component(term_t rgb, int component, int *color)
+{ term_t a;
+
+  if ( (a = PL_new_term_ref()) &&
+       PL_get_arg(component, rgb, a) &&
+       PL_get_integer_ex(a, color) )
+  { if ( *color < 0 || *color > 255 )
+      return PL_domain_error("rgb_value", a);
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
+static foreign_t
+win_window_color(term_t which, term_t color)
+{ char *s;
+  int r, g, b;
+  static functor_t FUNCTOR_rgb3 = 0;
+  int wcolor;
+
+  if ( !FUNCTOR_rgb3 )
+    FUNCTOR_rgb3 = PL_new_functor(PL_new_atom("rgb"), 3);
+
+  if ( !PL_get_chars(which, &s, CVT_ATOM|CVT_STRING|CVT_EXCEPTION) )
+    return FALSE;
+  if ( strcmp(s, "foreground") == 0 )
+    wcolor = RLC_TEXT;
+  else if ( strcmp(s, "background") == 0 )
+    wcolor = RLC_WINDOW;
+  else if ( strcmp(s, "selection_foreground") == 0 )
+    wcolor = RLC_HIGHLIGHTTEXT;
+  else if ( strcmp(s, "selection_background") == 0 )
+    wcolor = RLC_HIGHLIGHT;
+  else
+    return PL_domain_error("window_color", which);
+
+  if ( PL_is_functor(color, FUNCTOR_rgb3) &&
+       get_color_component(color, 1, &r) &&
+       get_color_component(color, 2, &g) &&
+       get_color_component(color, 3, &b) )
+  { rlc_color(PL_current_console(), wcolor, RGB(r,g,b));
+    return TRUE;
+  } else
+  { return PL_type_error("rgb", color);
+  }
+
+  return FALSE;
+}
+
+
 PL_extension extensions[] =
 {
 /*{ "name",	arity,  function,	PL_FA_<flags> },*/
@@ -1006,6 +1053,7 @@ PL_extension extensions[] =
   { "$win_insert_menu_item", 3, pl_win_insert_menu_item, 0 },
   { "win_insert_menu",       2, pl_win_insert_menu,      0 },
   { "win_window_pos",        1, pl_window_pos,           0 },
+  { "win_window_color",      2, win_window_color,        0 },
   { NULL,                    0, NULL,                    0 }
 };
 
