@@ -4,7 +4,7 @@
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
     Copyright (c)  1985-2021, University of Amsterdam
-                              VU University Amsterdam
+			      VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
     All rights reserved.
@@ -396,7 +396,7 @@ tracePort(DECL_LD LocalFrame frame, Choice bfr, int port, Code PC)
   LocalFrame fr = NULL;
 
   if ( (!isDebugFrame(frame) && !SYSTEM_MODE) || /* hidden */
-       debugstatus.suspendTrace )	        /* called back */
+       debugstatus.suspendTrace )		/* called back */
     return ACTION_CONTINUE;
 
   if ( port == EXCEPTION_PORT )		/* do not trace abort */
@@ -681,7 +681,7 @@ traceAction(char *cmd, int port, LocalFrame frame, Choice bfr,
 		return ACTION_AGAIN;
     case '.':   if ( LD->trace.find &&
 		     LD->trace.find->type != TRACE_FIND_NONE )
-	        { FeedBack("repeat search\n");
+		{ FeedBack("repeat search\n");
 		  LD->trace.find->searching = TRUE;
 		  clear(frame, FR_SKIPPED);
 		  return ACTION_CONTINUE;
@@ -736,10 +736,10 @@ traceAction(char *cmd, int port, LocalFrame frame, Choice bfr,
 		debugstatus.skiplevel = levelFrame(frame) - 1;
 		return ACTION_CONTINUE;
     case 'd':   FeedBack("depth\n");
-                setPrintOptions(def_arg ? 10 : consInt(num_arg));
+		setPrintOptions(def_arg ? 10 : consInt(num_arg));
 		return ACTION_AGAIN;
     case 'w':   FeedBack("write\n");
-                setPrintOptions(ATOM_write);
+		setPrintOptions(ATOM_write);
 		return ACTION_AGAIN;
     case 'p':   FeedBack("print\n");
 		setPrintOptions(ATOM_print);
@@ -765,7 +765,7 @@ traceAction(char *cmd, int port, LocalFrame frame, Choice bfr,
 		}
 		return ACTION_AGAIN;
     case 'm':	FeedBack("Exception details\n");
-	        if ( port & EXCEPTION_PORT )
+		if ( port & EXCEPTION_PORT )
 		{ exceptionDetails();
 		} else
 		   Warn("No exception\n");
@@ -1245,11 +1245,9 @@ _PL_backtrace(IOSTREAM *out, int depth, int flags)
       ctx = from;
     }
 
-    for(; depth > 0; PL_step_context(&ctx))
-    { LocalFrame frame;
-
-      if ( !(frame=ctx.fr) )
-	return;
+    startCritical();
+    for(; depth > 0 && ctx.fr; PL_step_context(&ctx))
+    { LocalFrame frame = ctx.fr;
 
       if ( frame->predicate == def )
       { if ( ++same_proc >= 10 )
@@ -1274,6 +1272,7 @@ _PL_backtrace(IOSTREAM *out, int depth, int flags)
 	depth--;
       }
     }
+    endCritical();
   } else
   { Sfprintf(out, "No stack??\n");
   }
@@ -1368,7 +1367,7 @@ traceInterception(LocalFrame frame, Choice bfr, int port, Code PC)
       case CUT_EXIT_PORT:  portfunc = FUNCTOR_cut_exit1; break;
       default:
 	assert(0);
-        goto out;
+	goto out;
     }
     RESTORE_PTRS();
 
@@ -1564,7 +1563,7 @@ PL_describe_context(pl_context_t *c, char *buf, size_t len)
     int printed;
 
     if ( !onStack(local, fr) )
-      return snprintf(buf, len, "<invalid frame reference %p>", fr);
+      return snprintf(buf, len, "<invalid frame reference %p>", (void*)fr);
 
     level = levelFrame(fr);
     if ( !fr->predicate )
@@ -1705,7 +1704,7 @@ alternativeFrame(LocalFrame frame)
 	  }
 	}
 
-        return NULL;
+	return NULL;
       }
     }
 
@@ -1728,7 +1727,7 @@ resetTracer(void)
   debugstatus.skiplevel    = 0;
   debugstatus.retryFrame   = 0;
 
-  setPrologFlagMask(PLFLAG_LASTCALL);
+  setPrologRunMode(RUN_MODE_NORMAL);
 }
 
 
@@ -1802,7 +1801,7 @@ interruptHandler(int sig)
 again:
   if ( safe )
   { if ( !printMessage(ATOM_debug, PL_FUNCTOR, FUNCTOR_interrupt1,
-		                     PL_ATOM, ATOM_begin) )
+				     PL_ATOM, ATOM_begin) )
       PL_clear_exception();
   } else
   { if ( first )
@@ -1858,7 +1857,7 @@ again:
 		print_backtrace_named("INT");
 		goto again;
     case 'p':	Sfprintf(Sdout, "PID: %d\n", getpid());
-                goto again;
+		goto again;
     case 'h':
     case '?':	helpInterrupt();
 		goto again;
@@ -1892,12 +1891,16 @@ PL_interrupt(int sig)
 #endif
 }
 
+static
+PRED_IMPL("prolog_interrupt", 0, prolog_interrupt, PL_FA_NOTRACE)
+{ PL_interrupt(2);
+  return !PL_exception(0);
+}
+
 
 void
-initTracer(void)
-{ GET_LD
-
-  debugstatus.visible      =
+initTracer(DECL_LD)
+{ debugstatus.visible      =
   debugstatus.leashing     = CALL_PORT|FAIL_PORT|REDO_PORT|EXIT_PORT|
 			     EXCEPTION_PORT;
   debugstatus.showContext  = FALSE;
@@ -1908,10 +1911,9 @@ initTracer(void)
 }
 
 int
-enable_debug_on_interrupt(int enable)
-{ GET_LD
-
-#ifdef SIGINT
+enable_debug_on_interrupt(DECL_LD int enable)
+{
+#if O_SIGNALS && defined(SIGINT)
   if ( enable )
   { if ( truePrologFlag(PLFLAG_SIGNALS) )
     { PL_signal(SIGINT, PL_interrupt);
@@ -2054,7 +2056,7 @@ debugmode(debug_type doit, debug_type *old)
 	return FALSE;
 
       debugstatus.skiplevel = SKIP_VERY_DEEP;
-      clearPrologFlagMask(PLFLAG_LASTCALL);
+      clearPrologRunMode(RUN_MODE_NORMAL);
       if ( doit == DBG_ALL )
       { QueryFrame qf;
 
@@ -2064,13 +2066,15 @@ debugmode(debug_type doit, debug_type *old)
 	doit = DBG_ON;
       }
     } else
-    { setPrologFlagMask(PLFLAG_LASTCALL);
+    { setPrologRunMode(RUN_MODE_NORMAL);
     }
     debugstatus.debugging = doit;
     updateAlerted(LD);
     return printMessage(ATOM_silent,
 			PL_FUNCTOR_CHARS, "debug_mode", 1,
 			  PL_ATOM, doit ? ATOM_on : ATOM_off);
+  } else if ( !doit )
+  { setPrologRunMode(RUN_MODE_NORMAL);
   }
 
   return TRUE;
@@ -2313,6 +2317,7 @@ static int
 prolog_frame_attribute(term_t frame, term_t what, term_t value)
 { GET_LD
   LocalFrame fr;
+  term_t fref;			/* Save/restore fr over possible shifts */
   atom_t key;
   size_t arity;
   term_t result = PL_new_term_ref();
@@ -2322,6 +2327,7 @@ prolog_frame_attribute(term_t frame, term_t what, term_t value)
     return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_frame_reference, frame);
   if ( !fr )
     return FALSE;				/* frame == 'none' */
+  fref = consTermRef((Word)fr);
   if ( !PL_get_name_arity(what, &key, &arity) )
     return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_callable, what);
   if ( !PL_strip_module(value, &m, value) )
@@ -2351,15 +2357,15 @@ prolog_frame_attribute(term_t frame, term_t what, term_t value)
     checkData(argFrameP(fr, argn-1));
 #endif
 
-   if ( !hasGlobalSpace(0) )
-   { int rc;
+    if ( !hasGlobalSpace(0) )
+    { int rc;
 
-     if ( (rc=ensureGlobalSpace(0, ALLOW_GC)) != TRUE )
-       return raiseStackOverflow(rc);
-     PL_get_frame(frame, &fr);
-   }
+      if ( (rc=ensureGlobalSpace(0, ALLOW_GC)) != TRUE )
+	return raiseStackOverflow(rc);
+      fr = (LocalFrame)valTermRef(fref);
+    }
 
-   return PL_unify(value, consTermRef(argFrameP(fr, argn-1)));
+    return PL_unify(value, consTermRef(argFrameP(fr, argn-1)));
   }
 
   if ( !(arity == 0 || (arity == 1 && key == ATOM_parent_goal)) )
@@ -2418,13 +2424,18 @@ prolog_frame_attribute(term_t frame, term_t what, term_t value)
       fail;
 
     if ( (fid = PL_open_foreign_frame()) )
-    { while( fr )
+    { fr = (LocalFrame)valTermRef(fref);
+
+      while( fr )
       { while(fr && fr->predicate != proc->definition)
-	  fr = parentFrame(fr);
+	{ fr = parentFrame(fr);
+	  assert(!fr || isFrame(fr));
+	}
 
 	if ( fr )
 	{ int i, garity = fr->predicate->functor->arity;
-	  term_t fref = consTermRef(fr);
+
+	  fref = consTermRef((Word)fr);
 
 	  for(i=0; i<garity; i++)
 	  { term_t fa;
@@ -2433,15 +2444,17 @@ prolog_frame_attribute(term_t frame, term_t what, term_t value)
 
 	    _PL_get_arg(i+1, head, a);
 	    if ( !PL_unify(a, fa) )
-	      break;
+	      break;				/* Argument does not unify */
 	    fr = (LocalFrame)valTermRef(fref);	/* deal with possible shift */
 	  }
-	  if ( i == garity )
-	  { if ( arity == 1 )
+	  /* fr is valid because restored after every unification */
+	  if ( i == garity )			/* All arguments unified: found a match */
+	  { if ( arity == 1 )			/* parent_frame(Next) */
 	    { LocalFrame parent;
 	      term_t arg = PL_new_term_ref();
 
 	      _PL_get_arg(1, what, arg);
+	      fr = (LocalFrame)valTermRef(fref);
 	      if ( (parent = parentFrame(fr)) )
 	      { if ( PL_unify_frame(arg, parent) )
 		  return TRUE;
@@ -2464,10 +2477,11 @@ prolog_frame_attribute(term_t frame, term_t what, term_t value)
 	  }
 	} else
 	{ PL_close_foreign_frame(fid);
-	  return FALSE;
+	  return FALSE;				/* No frame running this predicate */
 	}
       }
-    }
+    } else
+      return FALSE;
   } else if ( key == ATOM_pc )
   { if ( fr->programPointer &&
 	 fr->parent &&
@@ -2624,4 +2638,5 @@ BeginPredDefs(trace)
   PRED_DEF("prolog_choice_attribute", 3, prolog_choice_attribute, 0)
   PRED_DEF("prolog_skip_frame", 1, prolog_skip_frame, PL_FA_NOTRACE)
   PRED_DEF("prolog_skip_level", 2, prolog_skip_level, PL_FA_NOTRACE)
+  PRED_DEF("prolog_interrupt", 0, prolog_interrupt, PL_FA_NOTRACE)
 EndPredDefs
